#!/bin/csh -f

#######################################################################
#                     Batch Parameters for Run Job
#######################################################################

#PBS -l walltime=06:00:00
#@RUN_FP
#PBS -N @RUN_FN
#@RUN_Q
#@BATCH_GROUP
#PBS -o STATOUT

#######################################################################
#                  System Environment Variables
#######################################################################

umask 022

limit stacksize unlimited

@SETENVS

#######################################################################
#           Architecture Specific Environment Variables
#######################################################################

setenv ARCH `uname`

setenv SITE             @SITE
setenv GEOSBIN          @GEOSBIN
setenv GEOSUTIL         @GEOSUTIL
setenv RUN_CMD         "@RUN_CMD"

source $GEOSBIN/g5_modules
setenv LD_LIBRARY_PATH ${LD_LIBRARY_PATH}:${BASEDIR}/${ARCH}/lib

#######################################################################
#             Experiment Specific Environment Variables
#######################################################################

setenv    EXPID   @EXPID
setenv    EXPDIR  @EXPDIR
setenv    HOMDIR  @HOMDIR

#######################################################################
#                  Set Forecast Run Parameters
#######################################################################

set ANA = NCEP

set nymdb = 20150129
set nymde = 20150129
set nymd  = $nymdb

set yyyymm = `echo $nymdb | cut -b1-6`

mkdir -p $EXPDIR/forecasts/G5${ANA}.stats.$yyyymm
cd       $EXPDIR/forecasts/G5${ANA}.stats.$yyyymm

#######################################################################
#                               Create STATS
#######################################################################

while( $nymd <= $nymde )

set date0  =  $nymd
set time0  =  000000
set yyyymm = `echo $nymd | cut -b1-6`

set year0      = `echo $nymd | cut -b1-4`
set month0     = `echo $nymd | cut -b5-6`
set fcst_files = `/bin/ls -1 $EXPDIR/forecasts/${nymd}_00z/G5${ANA}*geosgcm_fcst*nc4`

set anal_files = ''
@ n = 1
while ($n <= 6)

set year  = `echo $nymd | cut -b1-4`
set month = `echo $nymd | cut -b5-6`


# G5REPLAY Verification
# ---------------------
set next = `/bin/ls -1 $EXPDIR/holding/geosgcm_prog/${year}*/$EXPID.geosgcm_prog.${nymd}*nc4`

set anal_files = `echo $anal_files $next`
set date = `$GEOSUTIL/post/tick $nymd 0 86400`
set nymd = $date[1]
@ n = $n + 1
end

$GEOSUTIL/post/stats.x -fcst $fcst_files \
                       -ana  $anal_files \
                       -cli $SHARE/gmao_ops/verification/stats/merrasc.197901-200812.clim_00z.144x91.data.nc \
                            $SHARE/gmao_ops/verification/stats/merrasc.197901-200812.clim_06z.144x91.data.nc \
                            $SHARE/gmao_ops/verification/stats/merrasc.197901-200812.clim_12z.144x91.data.nc \
                            $SHARE/gmao_ops/verification/stats/merrasc.197901-200812.clim_18z.144x91.data.nc \
                       -tag $EXPID -nfreq 060000 -rc $GEOSUTIL/post/stats.rc &

set date = `$GEOSUTIL/post/tick $date0 $time0 86400`
set nymd = $date[1]

end

wait
