!-------------------------------------------------------------------------
!     NASA/GSFC, Global Modeling and Assimilation Office, Code 610.1     !
!-------------------------------------------------------------------------
!BOP
!
! !MODULE: GACL_ConstantsMod
!
! !INTERFACE:
!
   module GACL_ConstantsMod
!
! !USES:
!
#ifdef MAPL
   use MAPL_ConstantsMod, only : MAPL_PI, MAPL_GRAV, MAPL_AVOGAD, MAPL_RUNIV, &
                                 MAPL_AIRMW, MAPL_H2OMW, MAPL_O3MW, &
                                 MAPL_TICE
#endif

   implicit none
   private

!
! !PUBLIC MEMBER FUNCTIONS:

!
! !PUBLIC PARAMETERS:
#ifdef MAPL
   real, parameter, public :: pi       = MAPL_PI                ! pi 
   real, parameter, public :: N_avog   = 1e-3 * MAPL_AVOGAD     ! Avogadro's constant,                '1 mol-1'
   real, parameter, public :: R_univ   = 1e-3 * MAPL_RUNIV      ! Universal/ideal gas constant,       'J K-1 mol-1'
   real, parameter, public :: g_earth  = MAPL_GRAV              ! standard gravity,                   'm s-2'
   real, parameter, public :: T_ice    = MAPL_TICE              ! melting point of ice,               'K'
   real, parameter, public :: mw_air   = MAPL_AIRMW             ! molar mass of dry air,              'g mol-1'
   real, parameter, public :: mw_H2O   = MAPL_H2OMW             ! molar mass of water,                'g mol-1'
   real, parameter, public :: mw_O3    = MAPL_O3MW              ! molar mass of ozone,                'g mol-1'
#else
   real, parameter, public :: pi       = 3.141592653589793      ! pi 
   real, parameter, public :: N_avog   = 6.022e23               ! Avogadro's constant,                '1 mol-1'
   real, parameter, public :: R_univ   = 8.31447                ! Universal/ideal gas constant,       'J K-1 mol-1'
   real, parameter, public :: g_earth  = 9.80665                ! standard gravity,                   'm s-2'
   real, parameter, public :: T_ice    = 273.16                 ! Melting point of ice,               'K'
   real, parameter, public :: mw_air   = 28.965                 ! molar mass of dry air,              'g mol-1'
   real, parameter, public :: mw_H2O   = 18.015                 ! molar mass of water,                'g mol-1'
   real, parameter, public :: mw_O3    = 47.9982                ! molar mass of ozone,                'g mol-1'
#endif

   real, parameter, public :: mw_S     = 32.065                 ! atomic mass of sulfur,              'g mol-1'
   real, parameter, public :: mw_SO2   = 64.066                 ! molar mass of sulfur dioxide,       'g mol-1'
   real, parameter, public :: mw_SO4   = 96.07                  ! molar mass of sulfate,              'g mol-1'
   real, parameter, public :: mw_H2SO4 = 98.079                 ! molar mass of sulfuric acid,        'g mol-1'
   real, parameter, public :: mw_DMS   = 62.13                  ! molar mass of dimethyl sulfide,     'g mol-1'
   real, parameter, public :: mw_MSA   = 96.11                  ! molar mass of methanesulfonic acid, 'g mol-1'
   real, parameter, public :: mw_OH    = 17.01                  ! molar mass of hydroxide ion,        'g mol-1'
   real, parameter, public :: mw_NO3   = 62.0049                ! molar mass of nitrate ion,          'g mol-1'
   real, parameter, public :: mw_NH3   = 17.031                 ! molar mass of ammonia,              'g mol-1'
   real, parameter, public :: mw_NH4   = 28.97                  ! molar mass of ammonium,             'g mol-1'
   real, parameter, public :: mw_OCS   = 60.075                 ! molar mass of carbonyl sulfide,     'g mol-1'
   real, parameter, public :: mw_SOAg  = 12.0                   ! molar mass of SOA(gas),             'g mol-1'

!
! !PRIVATE PARAMETERS:

!
! !DESCRIPTION: 
!
!  {\tt GACL\_ConstantsMod} defines physics and chemistry constants.
!
!
! !REVISION HISTORY:
!
!  06June2014  A. Darmenov  Initial version
!
!EOP
!-------------------------------------------------------------------------

   end module GACL_ConstantsMod

