!**********************************************************************
      SUBROUTINE CLEAN(NDFUNCS)
!**********************************************************************
      IMPLICIT NONE
#     include "CMN_OH"
      INTEGER NDFUNCS,NALL,K
!**********************************************************************
! Select proper identification number of parameterization needed for 
!   box characterized by unpolluted conditions and no isoprene using
!   SR PARAMA. Store this value in COCOUNT. Then select the corresponding 
!   polynomials and store in NDFUNCS.
!**********************************************************************
!   Created by Bryan Duncan.
!**********************************************************************
! A) Clean
!***************************************************
!  A1) Surface
!***************************************************
!
      IF(OH_PRESS.GE.PRESSES(1)) THEN
!
! Determine the identification number of the appropriate parameterization.
!
        CALL PARAMA(OH_LAT,OH_SEASON,COCOUNT,NSEAS,NLATS,ALATS)
!
! Error Check.
!
         IF(COCOUNT.EQ.0) THEN
           PRINT*,'COCOUNT=0!'
           PRINT*,'No parameterization was chosen!'
           PRINT*,'Stopped in SR CLEAN.'
           STOP 
         ENDIF
!
! Determine the polynomials associated with selected identification
!   number.
!
            NALL=3*NPARAMB
         IF(COCOUNT.GT.1) THEN
          DO K=NALL+1,NALL+COCOUNT-1
           NDFUNCS=NDFUNCS+NDONE(K)
          ENDDO
         ENDIF
!
            COCOUNT=COCOUNT+NALL
!
! Error Check.
!
            NALL=NALL+NPARAMA
         IF(COCOUNT.GT.NALL) THEN
           PRINT*,'COCOUNT=',COCOUNT
           PRINT*,'NALL   =',NALL
           PRINT*,'COCOUNT > NALL'
           PRINT*,'Stopped in SR CLEAN.'
           STOP 
         ENDIF
!
            GOTO 321
!
      ENDIF
!
!***************************************************
!  END A1
!***************************************************
! Sum of all NDONE functions in A).
!
            NDFUNCS=NDFUNCS+NDFUNCSA1
!
!***************************************************
!  A2) Middle Troposphere
!***************************************************
!
      IF(OH_PRESS.LT.PRESSES(1).AND.OH_PRESS.GE.PRESSES(2)) THEN
!
! Determine the identification number of the appropriate parameterization.
!
        CALL PARAMA(OH_LAT,OH_SEASON,COCOUNT,NSEAS,NLATS,ALATS)
!
! Error Check.
!
         IF(COCOUNT.EQ.0) THEN
           PRINT*,'COCOUNT=0!'
           PRINT*,'No parameterization was chosen!'
           PRINT*,'Stopped in SR CLEAN.'
           STOP 
         ENDIF
!
! Determine the polynomials associated with selected identification
!   number.
!
            NALL=3*NPARAMB+NPARAMA
         IF(COCOUNT.GT.1) THEN
          DO K=NALL+1,NALL+COCOUNT-1
           NDFUNCS=NDFUNCS+NDONE(K)
          ENDDO
         ENDIF
!
          COCOUNT=COCOUNT+NALL
!
! Error Check.
!
            NALL=NALL+NPARAMA
         IF(COCOUNT.GT.NALL) THEN
           PRINT*,'COCOUNT=',COCOUNT
           PRINT*,'NALL   =',NALL
           PRINT*,'COCOUNT > NALL in SR CLEAN'
           STOP
         ENDIF
!
            GOTO 321
!
      ENDIF
!
!***************************************************
!  END A2
!***************************************************
!
            NDFUNCS=NDFUNCS+NDFUNCSA2
!
!***************************************************
!  A3) Upper Troposphere
!***************************************************
!
      IF(OH_PRESS.LT.PRESSES(2)) THEN
!
! Determine the identification number of the appropriate parameterization.
!
        CALL PARAMA(OH_LAT,OH_SEASON,COCOUNT,NSEAS,NLATS,ALATS)
!
! Error Check.
!
         IF(COCOUNT.EQ.0) THEN
           PRINT*,'COCOUNT=0!'
           PRINT*,'No parameterization was chosen!'
           PRINT*,'Stopped in SR CLEAN.'
           STOP 
         ENDIF
!
! Determine the polynomials associated with selected identification
!   number.
!
            NALL=3*NPARAMB+2*NPARAMA
         IF(COCOUNT.GT.1) THEN
          DO K=NALL+1,NALL+COCOUNT-1
           NDFUNCS=NDFUNCS+NDONE(K)
          ENDDO
         ENDIF
!
          COCOUNT=COCOUNT+NALL
!
! Error Check.
!
           NALL=NALL+NPARAMA
         IF(COCOUNT.GT.NALL) THEN
           PRINT*,'COCOUNT=',COCOUNT
           PRINT*,'NALL   =',NALL
           PRINT*,'COCOUNT > NALL in SR CLEAN'
           STOP
         ENDIF
!
            GOTO 321
!
      ENDIF
!
!***************************************************
!  END A3
!***************************************************
! Error check.
!    
      PRINT*,'********************************'
      PRINT*,'Stopped in SR CLEAN!'
      PRINT*,'No parameterization was chosen!'
      PRINT*,'********************************'
      STOP
!
!***************************************************
 321  RETURN

      END SUBROUTINE CLEAN
