! ****************************************************************************
      SUBROUTINE GETINFO(OH_MONTH2,ALBD, &
                         INDVARA, INDVARB, INDVARC, INDVARD, &
                         Wavg, Pavg, Tavg, Ravga, Ravgb, AD, &
                         STT, BBIJ, o3up, scaleO3, toms_climO3, &
                         METHVAR, RLON, RLAT, JDAY, &
                         MLONBX, MLATBX, MVRTBX)
! ****************************************************************************
! THIS SUBROUTINE IS TO BE MODIFIED BY THE USER!
! The user can interface it with a model by passing necessary
! information through the subroutine's argument list or
! through the user's own common blocks.
! ****************************************************************************
! ****************************************************************************
! In this subroutine, the user supplies the parameterization code
! with information it needs to calculate the 24-hour averaged [OH].
! ****************************************************************************
! Created by Bryan Duncan.
! ****************************************************************************
! The user is to put whatever common blocks and declaration 
! statements here which are necessary to supply this subroutine
! with the model input it requires.
!

      IMPLICIT NONE

#     include "CMN_SIZE"       ! Size parameters
#     include "CMN"            ! STT
#     include "CMN_CO"         ! CO arrays
#     include "CMN_CO_BUDGET"  ! FMOL_CO
#     include "CMN_OH"

      INTEGER, intent(in) :: MLONBX, MLATBX, MVRTBX
      INTEGER, intent(in) :: JDAY
      INTEGER, intent(in) :: OH_MONTH2
      REAL*8,  intent(in) :: AD   (MLONBX, MLATBX, MVRTBX) ! dry air mass
      REAL*8,  intent(in) :: Wavg(MLONBX, MLATBX, MVRTBX)  ! 24h avg spc. humidity
      REAL*8,  intent(in) :: Tavg(MLONBX, MLATBX, MVRTBX)  ! 24h avg temperature
      REAL*8,  intent(in) :: scaleO3    (MLONBX, MLATBX)
      REAL*8,  intent(in) :: o3up       (MLONBX, MLATBX, MVRTBX)
      REAL*8,  intent(in) :: toms_climO3(MLONBX, MLATBX, 12)
      REAL*8,  intent(in) :: STT        (MLONBX, MLATBX, MVRTBX,NNPAR)
      REAL*8,  intent(in) :: RLON       (MLONBX)   ! Array of longitudes
      REAL*8,  intent(in) :: RLAT       (MLATBX)   ! Array of latitudes
      REAL*8,  intent(in) :: METHVAR    (MLONBX, MLATBX, MVRTBX)
      ! BBIJ stores species concentration for:
      ! 1. NOt             2. ALK4          3. Isoprene
      ! 4. Acetone         5. Propene       6. Propane
      ! 7. Ethane          8. O3            9. CO
      REAL*8,  intent(in) :: BBIJ       (MLONBX, MLATBX, MVRTBX,NFIELDS2)

      REAL*8,  intent(inOut) :: Ravga(MLONBX, MLATBX, MVRTBX) ! 24h avg cloud flux above box
      REAL*8,  intent(inOut) :: Ravgb(MLONBX, MLATBX, MVRTBX) ! 24h avg cloud flux below box
      REAL*8,  intent(inOut) :: Pavg(MLONBX, MLATBX, MVRTBX)  ! 24h avg pressure
      REAL*8,  intent(inOut) :: INDVARA(MLONBX, MLATBX, MVRTBX, MXVAR)
      REAL*8,  intent(inOut) :: INDVARB(MLONBX, MLATBX, MVRTBX, MXVAR)
      REAL*8,  intent(inOut) :: INDVARC(MLONBX, MLATBX, MVRTBX, MXVAR)
      REAL*8,  intent(inOut) :: INDVARD(MLONBX, MLATBX, MVRTBX, MXVAR)

      REAL*8   ALBD(MLONBX, MLATBX)

! !LOCAL VARIABLES:
      REAL*8   PI,DEC,A0,A1,A2,A3,B1,B2,B3,R,BLAT
      PARAMETER ( PI = 3.141592 )
      INTEGER  KK,NDOCO,NCHECKIT,NDOCH4
      REAL*8   AISOP,ddd,sumo3up
      REAL*8 dummyo3(MLONBX, MLATBX)

      REAL*8 :: STTTOPPB   (MLONBX, MLATBX, MVRTBX)

      ! Weight of air (taken from "comode.h") 
      REAL*8, PARAMETER :: WTAIR = 28.966d0
!
! ****************************************************************************
! This common block and these declaration statements are not to be
! modified by the user.
!
      INTEGER I,J,K,L,NSTOP
!
! ****************************************************************************
! The polynomials that are used to calculate the parameterized
! OH concentrations are a function of the following independent
! variables:
!
!   Independent Variables                   Units
!   -----------------------------------------------------------
!   Solar Declination Angle                 degrees
!   Total Ozone Column Above                DU
!   Total Nitrogen Oxides                   pptv
!   Ozone                                   ppbv
!   Carbon Monoxide                         ppbv
!   Methane                                 ppbv
!   Water Vapor                             ppmv
!   Acetone                                 pptv
!   Pressure                                mb
!   Propene                                 pptv
!   Ethane                                  pptv
!   Propane                                 pptv
!   ALK4                                    pptv
!   Surface Albedo                          unitless
!   Latitude                                degrees
!   Temperature                             K
!   Cloude Albedo - Above                   unitless
!   Cloude Albedo - Below                   unitless
!   Isoprene (daylight)                     pptv
!   -----------------------------------------------------------
!
! All these independent variables need to be 24-hour averages!!
!
! All the independent variables are REAL*8!
!
! ****************************************************************************
!
! The independent variables are stored in the arrays INDVARA-D:
!
! INDVARA is used in surface layer parameterizations without isoprene.
! INDVARB is used in middle and upper tropospheric layer parameterizations
!         without isoprene.
! INDVARC is used in surface layer parameterizations with isoprene.
! INDVARD is used in middle and upper tropospheric layer parameterizations
!         with isoprene.
!
! --------------------------------------------------------------------------
!   MXVAR    INDVARA        INDVARB        INDVARC         INDVARD
! --------------------------------------------------------------------------
!     1       DEC            DEC            DEC             DEC
!     2       O3COL          O3COL          O3COL           O3COL
!     3       NOt            NOt            NOt             NOt
!     4       O3             O3             O3              O3
!     5       CO             CO             CO              CO
!     6       CH4            CH4            CH4             CH4
!     7       H2O            H2O            H2O             H2O
!     8       ACET           ACET           ACET            ACET
!     9       PRESS          PRESS          PRESS           PRESS
!    10       PRPE           PRPE           PRPE            PRPE
!    11       ETHA           ETHA           ETHA            ETHA
!    12       PROP           PROP           PROP            PROP
!    13       ALK4           ALK4           ALK4            ALK4
!    14       ALB            ALB            ALB             ALB
!    15       LAT            LAT            LAT             LAT
!    16       TEMP           TEMP           TEMP            TEMP
!    17       CLOUDA         CLOUDA         CLOUDA          CLOUDA
!    18         *            CLOUDB         ISOP            CLOUDB
!    19         *              *              *             ISOP
! --------------------------------------------------------------------------
!   *No independent variable value placed in this position.
!
! ****************************************************************************
! Error Check.  If the user does not totally fill in the INDVARA-D arrays,
! then the code will stop to tell the user.
!
      INDVARA(:,:,:,:)=-1000.D0
      INDVARB(:,:,:,:)=-1000.D0
      INDVARC(:,:,:,:)=-1000.D0
      INDVARD(:,:,:,:)=-1000.D0
!
      INDVARA(:,:,:,18:19)=0.D0
      INDVARB(:,:,:,19)=0.D0
      INDVARC(:,:,:,19)=0.D0
!
! ****************************************************************************

      PRINT*,'ENTERING SR GETINFO'
     
!
! (1) Solar declination angle (low precision formula)
!     JDAY = julian day
!
      A0 = 0.006918
      A1 = 0.399912
      A2 = 0.006758
      A3 = 0.002697
      B1 = 0.070257
      B2 = 0.000907
      B3 = 0.000148
 
      R  = 2.* PI * float(JDAY-1) / 365.
!
      DEC = A0 - A1*COS(  R) + B1*SIN(  R) &
     &         - A2*COS(2*R) + B2*SIN(2*R) &
     &         - A3*COS(3*R) + B3*SIN(3*R)
!
      DEC = DEC*180./PI
!      TESTDEC(:,:) = DEC
!
      IF (DEC.GT.24..OR.DEC.LT.-24.) THEN
         PRINT*,'Stopped in SR GETINFO!'
         PRINT*,'  Declination angle is too high!'
         PRINT*,'DEC = ',DEC
              CALL FLUSH( 6 )
         STOP
      ENDIF
!
      INDVARA(:,:,:,1)=DEC
!
! (2) O3 Column Above (DU)


!KNJR      INDVARA(:,:,:,2)=o3up(:,:,:,OH_MONTH2)
      INDVARA(:,:,:,2)=o3up(:,:,:)
      dummyo3(:,:)=INDVARA(:,:,1,2)

! EY DEBUG - no TOMS rescaling
!  Rescale O3 column to match TOMS data.
!   O3Col(L) = O3Col(L) * TOMSO3Col / O3Col(1)
!
      GOTO 321
      DO I=1,MLONBX
         DO J=1,MLATBX
            DO L=1,MVRTBX

!*********************************
               IF (L.EQ.1) THEN
!*********************************
                  IF (scaleO3(I,J).GT.100.) THEN
                     INDVARA(I,J,1,2)=scaleO3(I,J)
                  ENDIF

!bnd if no monthly scale factor from TOMS/SBUV, then
!bnd use a monthly average scale factor from 1988 to 1997
!bnd of TOMS/SBUV
                  IF (scaleO3(I,J).LE.100.) THEN
                     IF (toms_climO3(I,J,OH_MONTH2).GT.100.) THEN
                        INDVARA(I,J,1,2)=toms_climO3(I,J,OH_MONTH2)
                     ENDIF
                  ENDIF

!*********************************
               ELSE
!*********************************
                  IF (scaleO3(I,J).GT.100..AND.INDVARA(I,J,1,2).GT.100.)THEN
                     INDVARA(I,J,L,2)=INDVARA(I,J,L,2)* &
                                      scaleO3(I,J)/dummyo3(I,J)
                  ENDIF

                  IF (scaleO3(I,J).LE.100..AND.INDVARA(I,J,1,2).GT.100.)THEN
                     INDVARA(I,J,L,2)=INDVARA(I,J,L,2)* &
                                      toms_climO3(I,J,OH_MONTH2)/dummyo3(I,J)
                  ENDIF
!*********************************
               ENDIF
!*********************************
            ENDDO
         ENDDO
      ENDDO
321   CONTINUE
!
! (3) NOt (pptv)
! Note:  if you change the units of NOx here, you need to update
!        the NOx in the isoprene yield calculation!
!
!        NOt = NO + NO2 + 2N2O5 + NO3 + HNO2 + HNO4
!

      INDVARA(:,:,:,3)=BBIJ(:,:,:,1)*1.D3

!
      DO I=1,MLONBX
         DO J=1,MLATBX
            DO L=1,MVRTBX  ! DEBUG EY - changed from 5 when had just 2 levels
               IF (INDVARA(I,J,L,3).LT.1.D-5) THEN
                  PRINT*,'Stopped in SR getinfo.f!'
                  PRINT*,'NOx is mixing ratio for box:'
                  PRINT*,I,J,L,INDVARA(I,J,L,3)
                  CALL FLUSH( 6 )
!bnd               STOP
               ENDIF 
            ENDDO
         ENDDO
      ENDDO
!       
! (4) O3 (ppbv)
!

      INDVARA(:,:,:,4)=BBIJ(:,:,:,8)

      DO I=1,MLONBX
         DO J=1,MLATBX
            DO L=1,MVRTBX            ! DEBUG EY WAS set to 9
               IF (INDVARA(I,J,L,4).LT.1.D-5) THEN
                  PRINT*,'Stopped in SR getinfo.f!'
                  PRINT*,'O3 is mixing ratio for box:'
                  PRINT*,I,J,L,INDVARA(I,J,L,4)
                  CALL FLUSH( 6 )
                  STOP
               ENDIF
            ENDDO
         ENDDO
      ENDDO

!
! (5) CO (ppbv)
!
      NDOCO = 1
      IF (NDOCO.EQ.0) THEN
!
! Variable CO.
!
! Convert STT(CO) from kgs/box to ppb.
!
         DO J = 1,MVRTBX
            DO K = 1,MLATBX
               DO L = 1,MLONBX
                  goto 322
                  STTTOPPB(L,K,J)=1.D0/AD(L,K,J)*WTAIR/FMOL_CO/1000.D0*1.D9

                  IF (AD(L,K,J).LT.0.) THEN
                     PRINT*,'Stopped in SR getinfo!'
                     PRINT*,'Air density is negative!'
                     PRINT*,'vert/lat/lon=',J,K,L
                     CALL FLUSH( 6 )
                     STOP
                  ENDIF

                  IF (STT(L,K,J,1).LT.-1.) THEN
                     PRINT*,'Stopped in SR getinfo!'
                     PRINT*,'CO is negative = ',STT(L,K,J,1)
                     PRINT*,'vert/lat/lon=',J,K,L
                     CALL FLUSH( 6 )
                     STOP
                  ENDIF
322               CONTINUE


               ENDDO
            ENDDO
         ENDDO
         CALL FLUSH( 6 )
! 
      ELSE
!
! Monthly mean CO.
!
         INDVARA(:,:,:,5)=BBIJ(:,:,:,9)

!
      ENDIF

      DO I=1,MLONBX
         DO J=1,MLATBX
            DO L=1,MVRTBX          ! EY DEBUG changed from 9 when had jsut 2 vertboxs
               IF (INDVARA(I,J,L,5).LT.1.D-5) THEN
                  PRINT*,'Stopped in SR getinfo.f!'
                  PRINT*,'CO is lt 1e-5 for box:'
                  PRINT*,I,J,L,INDVARA(I,J,L,5)
                  CALL FLUSH( 6 )
                  STOP
               ENDIF
            ENDDO
         ENDDO
      ENDDO

!
! (6) CH4 (ppbv)
! See SR CO_fromHCs to see where I got the [CH4] from.
!

! EY DEBUG - get mean CH4 for latitude band
!************
! Constant [CH4].
!KNJR      A3090S=1700.0
!KNJR      A0030S=1700.0
!KNJR      A0030N=1700.0
!KNJR      A3090N=1700.0

!************
!
      INDVARA(:,:,:,6) = METHVAR(:,:,:)

!      DO L=1,MVRTBX
!         DO J=1,MLATBX
!            DO I=1,MLONBX
!! 
!               METHVAR=0.0
!!
!               IF(J.GE.1.AND.J.LE.12)  METHVAR=A3090S
!               IF(J.GE.13.AND.J.LE.23) METHVAR=A0030S
!               IF(J.GE.24.AND.J.LE.34) METHVAR=A0030N
!               IF(J.GE.35.AND.J.LE.46) METHVAR=A3090N
!!
!               INDVARA(:,J,:,6)=METHVAR
!!
!            ENDDO
!         ENDDO
!      ENDDO
!
! (7) H2O (ppmv)
!
      INDVARA(:,:,:,7)=Wavg(:,:,:)
      
!
! (8) Acetone (pptv) 
!
      
      INDVARA(:,:,:,8)=BBIJ(:,:,:,4)*0.333D0*1.E3
      DO I=1,MLONBX
         DO J=1,MLATBX
            DO L=1,MVRTBX          ! EY DEBUG changed from 9 when had jsut 2 vertboxs
               IF(INDVARA(I,J,L,8).LT.1.D-5) THEN
                  PRINT*,'Stopped in SR getinfo.f!'
                  PRINT*,'ACET is mixing ratio for box:'
                  PRINT*,I,J,L,INDVARA(I,J,L,8)
                  CALL FLUSH( 6 )
                  STOP
               ENDIF
            ENDDO
         ENDDO
      ENDDO

!
! (9) Pressure (mb) - 24 hr avg
!
      WHERE(Pavg(:,:,:).GT.1020.D0) Pavg(:,:,:)=1019.9D0

      DO L=1,MVRTBX
         INDVARA(:,:,L,9)=Pavg(:,:,L)  
      ENDDO
!
! (10) Propene (pptv)
!
      INDVARA(:,:,:,10)=BBIJ(:,:,:,5)*0.333D0*1.E3
!
! (11) Ethane (pptv)
!
      INDVARA(:,:,:,11)=BBIJ(:,:,:,7)*0.5D0*1.E3

      DO I=1,MLONBX
         DO J=1,MLATBX
            DO L=1,MVRTBX          ! EY DEBUG changed from 9 when had jsut 2 vertboxs
               IF(INDVARA(I,J,L,11).LT.1.D-5) THEN
                  PRINT*,'Stopped in SR getinfo.f!'
                  PRINT*,'ETHA is mixing ratio for box:'
                  PRINT*,I,J,L,INDVARA(I,J,L,11)
                  CALL FLUSH( 6 )
                  STOP
               ENDIF
            ENDDO
         ENDDO
      ENDDO

!
! (12) Propane (pptv)
!
      INDVARA(:,:,:,12)=BBIJ(:,:,:,6)*0.333D0*1.E3

      DO I=1,MLONBX
         DO J=1,MLATBX
            DO L=1,MVRTBX          ! EY DEBUG changed from 9 when had jsut 2 vertboxs
               IF(INDVARA(I,J,L,12).LT.1.D-5) THEN
                  PRINT*,'Stopped in SR getinfo.f!'
                  PRINT*,'PROP is mixing ratio for box:'
                  PRINT*,I,J,L,INDVARA(I,J,L,12)
                  CALL FLUSH( 6 )
                  STOP
               ENDIF
            ENDDO
         ENDDO
      ENDDO

!
! (13) ALK4 (pptv)
!     
      INDVARA(:,:,:,13)=BBIJ(:,:,:,2)*0.25D0*1.E3

      DO I=1,MLONBX
         DO J=1,MLATBX
            DO L=1,MVRTBX          ! EY DEBUG changed from 9 when had jsut 2 vertboxs
               IF(INDVARA(I,J,L,13).LT.1.D-5) THEN
                  PRINT*,'Stopped in SR getinfo.f!'
                  PRINT*,'ALK4 is mixing ratio for box:'
                  PRINT*,I,J,L,INDVARA(I,J,L,13)
                  CALL FLUSH( 6 )
!bnd              STOP
               ENDIF
            ENDDO
         ENDDO
      ENDDO
!
! (14) Albedo (fraction of 1)
! While visible albedo rarely goes below 0.06 (6%),
! the uv albedo does go below 0.06 over land occasionally.
! In general though, most of the earth's surface uv
! albedo is above this 0.06 minimum set for the params
! initially designed for use with visible albedo.
!
!vis       WHERE(AVGF(:,:).LT.0.06D0) AVGF(:,:)=0.061D0
      WHERE(ALBD(:,:).LT.0.06D0) ALBD(:,:)=0.061D0
      
!vis       WHERE(AVGF(:,:).GT.0.80D0) AVGF(:,:)=0.79D0
      WHERE(ALBD(:,:).GT.0.80D0) ALBD(:,:)=0.79D0

      DO L=1,MVRTBX
!vis       INDVARA(:,:,L,14)=AVGF(:,:)
         INDVARA(:,:,L,14)=ALBD(:,:)
      ENDDO
!
! (15) Latitude (deg = -90 to 90)
!
! EY DEBUG - INSERT LATITUDE ARRAY HERE  ; Originally - 2 degrees at the poles ; 4 degrees elsewehere (4x4.5 grid)
      !BLAT=35.D0  ! BLAT SET HERE
      DO J=1,MLATBX
        !IF(J.NE.MLATBX.AND.J.NE.1) THEN
        !   BLAT=BLAT+4.D0
        !ELSE
        !   BLAT=BLAT+2.D0
        !ENDIF
        !IF(J.EQ.2) BLAT=BLAT-1.D0
        !IF(J.EQ.MLATBX) BLAT=BLAT+1.D0
         INDVARA(:,J,:,15)=RLAT(J) ! BLAT
      ENDDO
      CONTINUE
      
!
! (16) Temperature (K)
!
      DO L=1,MVRTBX
         INDVARA(:,:,L,16)=Tavg(:,:,L)  
      ENDDO
!
! (17) Cloud flux fraction above box
!
      WHERE(Ravga(:,:,:).LT.0.D0) Ravga(:,:,:)=0.D0

      WHERE(Ravga(:,:,:).GT.0.6D0) Ravga(:,:,:)=0.59999D0

      INDVARA(:,:,:,17)=Ravga(:,:,:)
!ccc      INDVARA(:,:,:,17)=Ravga(:,:,:)*1.2
!*****************************************************************************
!
! The first 17 independent variables in INDVARA are the
! same for INDVARB, INDVARC and INDVARD.  However, they
! are different for the 18-19 independent variables.
!
      INDVARB(:,:,:,1:17)=INDVARA(:,:,:,1:17)
      INDVARC(:,:,:,1:17)=INDVARA(:,:,:,1:17)
      INDVARD(:,:,:,1:17)=INDVARA(:,:,:,1:17)
!
!*****************************************************************************
!
! (18) Cloud flux fraction below box
!
      WHERE(Ravgb(:,:,:).LT.0.D0) Ravgb(:,:,:)=0.D0

      WHERE(Ravgb(:,:,:).GT.0.6D0) Ravgb(:,:,:)=0.59999D0

      INDVARB(:,:,:,18)=Ravgb(:,:,:)
      INDVARD(:,:,:,18)=Ravgb(:,:,:)
!ccccc      INDVARB(:,:,:,18)=Ravgb(:,:,:)*1.2
!ccccc      INDVARD(:,:,:,18)=Ravgb(:,:,:)*1.2

!*****************************************************************************
! Resetting cloud flux fractions to 0 for test!
!      INDVARA(:,:,:,17)=0.0001
!      INDVARB(:,:,:,17)=0.0001
!      INDVARC(:,:,:,17)=0.0001
!      INDVARD(:,:,:,17)=0.0001
!      INDVARB(:,:,:,18)=0.0001
!      INDVARD(:,:,:,18)=0.0001
!*****************************************************************************
!
! (19) Isoprene (pptv) - daylight average?
!
      DO L=1,MVRTBX
         DO J=1,MLATBX
            DO I=1,MLONBX
!
!         AISOP=BBIJ(I,J,L,3)*0.2D0*1.E12
               AISOP=BBIJ(I,J,L,3)*0.2D0*1.E3
!
! are you reading in 24-Hr averages or daylight averages?
! see CO_fillfields.f

               IF(AISOP.LT.2.D0) AISOP=2.D0

               INDVARC(I,J,L,18)=AISOP
               INDVARD(I,J,L,19)=AISOP
            ENDDO
         ENDDO
      ENDDO
!
! ****************************************************************************
! Error Check. Print out input variables.
!
       NCHECKIT=0
!
      IF (NCHECKIT.EQ.1) THEN       
!
         PRINT*,'DOING ERROR CHECK BY PRINTING fort.819!'
!
         open(65,file='fort.819',status='unknown')

         DO I=1,MLONBX
            DO J=1,MLATBX
               DO L=1,MVRTBX
                  DO KK=1,19
!c       if(INDVARD(I,J,L,3).le.350..and.
!c     *    INDVARD(I,J,L,19).gt.11.) then     
                     WRITE(65,*) INDVARD(I,J,L,KK)
!c       else
!c         ddd=0. 
!c        write(819,*) ddd
!c       endif
                  ENDDO
               ENDDO
            ENDDO
         ENDDO
         close(65)
         CALL FLUSH( 6 )
         STOP 
!
      ENDIF
!
! ***************************************************************************
! ***************************************************************************
! Other parameters needed.
! ***************************************************************************
! ***************************************************************************
!
! A) Longitude (deg = -180 - 180)
!   Longitude is not an independent variable used in the polynomials
!   to calculate the parameterized OH, but is used to locate 
!   regions of biomass burning.  There are separate parameterizations
!   for regions affected by biomass burning.  For convenience, however,
!   longitude is stored in the INDVAR* arrays.
!
! EY DEBUG SET LON HERE
      !BLON=-90.D0
      DO I=1,MLONBX
         !BLON=BLON+5.D0

         INDVARA(I,:,:,20)=RLON(I)
         INDVARB(I,:,:,20)=RLON(I)
         INDVARC(I,:,:,20)=RLON(I)
         INDVARD(I,:,:,20)=RLON(I)
      ENDDO
!
! B) Season
!     OH_MONTH2 = month (January-December = 1-12)
!     OH_SEASON = Northern Hemispheric season
!        1 --> winter (Dec, Jan, Feb)
!        2 --> spring (Mar, Apr, May)
!        3 --> summer (Jun, Jul, Aug)
!        4 --> autumn (Sep, Oct, Nov)
!
! From month, determine season.  The parameterizations are
! functions of season and not month.
!
      IF(OH_MONTH2.LE.2)  OH_SEASON=1
      IF(OH_MONTH2.EQ.12) OH_SEASON=1
      IF(OH_MONTH2.LE.5.AND.OH_MONTH2.GE.3)  OH_SEASON=2
      IF(OH_MONTH2.LE.8.AND.OH_MONTH2.GE.6)  OH_SEASON=3
      IF(OH_MONTH2.LE.11.AND.OH_MONTH2.GE.9) OH_SEASON=4
!     
!*****************************************************************************
! Error Check.
!     
      NSTOP=0
!

      
      DO I = 1,MXVAR
         DO J = 1,MVRTBX
            DO K = 1,MLATBX
               DO L = 1,MLONBX

!bnd         IF ( IEEE_IS_NAN( INDVARA(L,K,J,I) ) ) THEN
!bnd           PRINT*,'Stopped in SR getinfo.'
!bnd           PRINT*, 'INDVARA(:,:,:,I) is NaN!'
!bnd           PRINT*,I,L,K,J,INDVARA(L,K,J,I)
!bnd              CALL FLUSH( 6 )
!bnd           STOP
!bnd         ENDIF
!
                  IF(INDVARA(L,K,J,I).LT.-200.) THEN
                     PRINT*,'******************************************'
                     PRINT*,'ERROR IN SR GETINFO!'
                     PRINT*,'The array INDVARA is not completely filled.'
                     PRINT*,'Check independent variable number = ',I
                     PRINT*,'vert/lat/lon = ',J,K,L
                     PRINT*,'INDVARA(L,K,J,I)=',INDVARA(L,K,J,I)
                     NSTOP=1
                  ENDIF
!
                  IF(INDVARB(L,K,J,I).LT.-200.) THEN
                     PRINT*,'******************************************'
                     PRINT*,'ERROR IN SR GETINFO!'
                     PRINT*,'The array INDVARB is not completely filled.'
                     PRINT*,'Check independent variable number = ',I
                     PRINT*,'vert/lat/lon = ',J,K,L
                     PRINT*,'INDVARB(L,K,J,I)=',INDVARB(L,K,J,I)
                     NSTOP=1
                  ENDIF
!
                  IF(INDVARC(L,K,J,I).LT.-200.) THEN
                     PRINT*,'******************************************'
                     PRINT*,'ERROR IN SR GETINFO!'
                     PRINT*,'The array INDVARC is not completely filled.'
                     PRINT*,'Check independent variable number = ',I
                     PRINT*,'vert/lat/lon = ',J,K,L
                     PRINT*,'INDVARC(L,K,J,I)=',INDVARC(L,K,J,I)
                     NSTOP=1
                  ENDIF
!
                  IF(INDVARD(L,K,J,I).LT.-200.) THEN
                     PRINT*,'******************************************'
                     PRINT*,'ERROR IN SR GETINFO!'
                     PRINT*,'The array INDVARD is not completely filled.'
                     PRINT*,'Check independent variable number = ',I
                     PRINT*,'vert/lat/lon = ',J,K,L
                     PRINT*,'INDVARD(L,K,J,I)=',INDVARD(L,K,J,I)
                     NSTOP=1
                  ENDIF
!
                  IF(NSTOP.EQ.1) THEN
                     PRINT*,'L,K,J=',L,K,J
                     PRINT*,'******************************************'
                       CALL FLUSH( 6 )
                     STOP
                  ENDIF
!
               ENDDO
            ENDDO
         ENDDO
      ENDDO
!
      IF(OH_SEASON.LT.1.OR.OH_SEASON.GT.4) THEN
         PRINT*,'******************************************'
         PRINT*,'ERROR IN SR GETINFO!'
         PRINT*,'OH_SEASON does not have a value between 1 & 4.'
         PRINT*,'OH_SEASON = ',OH_SEASON
         PRINT*,'******************************************'
              CALL FLUSH( 6 )
         STOP
      ENDIF
!
!*****************************************************************************
!
      PRINT*,'LEAVING SR GETINFO'
      RETURN
      END SUBROUTINE GETINFO
