!*****************************************************************************
      SUBROUTINE GETOH(COLON,COLAT,COALT,FIRSTDT,BOH,OHLMN,DOskip, &
                       INDVARA, INDVARB, INDVARC, INDVARD)
!*****************************************************************************
      IMPLICIT NONE
#     include "CMN_OH"
      REAL*8 :: BOH(:,:,:)
      REAL*8 :: INDVARA(:,:,:,:)
      REAL*8 :: INDVARB(:,:,:,:)
      REAL*8 :: INDVARC(:,:,:,:)
      REAL*8 :: INDVARD(:,:,:,:)
!
      LOGICAL FIRSTDT
      INTEGER I,J,K,L,M
      INTEGER NDFUNCS,NALL,NLOOK,OHLMN
      INTEGER COLON,COLAT,COALT,DOskip,OOB
!
!*****************************************************************************
! Created by Bryan Duncan.
!*****************************************************************************
! This SR determines which parameterization is needed for an individual 
! box and calculates the parameterized [OH] (SR CALC_OH).
!
! The parameterizations are defined by latitude, pressure, [NOx], [ISOP]
! and season.  (The corresponding model variables are OH_LAT, OH_PRESS,
! OH_NOX, OH_ISOP and OH_SEASON, resp.)

      OH_LAT   = INDVARA(COLON,COLAT,COALT,15)
      OH_LON   = INDVARA(COLON,COLAT,COALT,20)
      OH_PRESS = INDVARA(COLON,COLAT,COALT,9)
      OH_NOX   = INDVARA(COLON,COLAT,COALT,3)
      OH_ISOP  = INDVARC(COLON,COLAT,COALT,18)
      OH_O3    = INDVARA(COLON,COLAT,COALT,4)
!
!*****************************************************************************
! List of Variables & Arrays
!
! BOH = array holding parameterized OH.
!
! Dimensions of troposphere.  User specified values in CMN_OH.
!   COALT = position of box in vertical in troposphere
!   COLAT = position of box from pole to pole (following one longitude band)
!   COLON = position of box circling globe (following one latitude band)
!
! NDFUNCS = number of polynomials.  NDFUNCS is greater than the number
!           of parameterizations (NPARAM). A parameterization of
!           a subdomain may be described by more than one polynomial
!           due to domain divisions.
!
      NDFUNCS=0
!
! FIRSTDT = 0 on first chemistry time step.
!         = 1 on any time chemistry step, but first. 
!
! PARAMOH = parameterized [OH].
!
      PARAMOH=100.
!
! COCOUNT = identification number of parameterization.
!
      COCOUNT=0
!
! OOB     = bookkeeping for error check in SR CALC_OH.
!
      OOB=0
!
! INDVAR  = independent variables needed to calculate parameterized [OH].
!
      INDVAR(1:MXVAR)=0.
!
! DOskip  = Counter to skip parameterization if necessary.
!         = 0 : use parameterization to get [OH].
!         = 1 : set to low value and skip parameterization.
!         = 2 : use climatological [OH] (see SR READAVGOH & SR INTERPOH).
!         = 3 : use [OH] = 1x10^5 if isoprene > 0.6 ppbv and skip parameterization
!         = 4 : use [OH] = 1x10^4 if isoprene > 3 ppbv and skip parameterization
!
! Variables specific to box in question:
!
! OH_NOX    = NOx concentration
! OH_PRESS  = pressure
! OH_LAT    = latitude
! OH_ISOP   = isoprene concentration
! OH_LON    = longitude
! OH_MONTH = month (January-December = 1-12)
! OH_SEASON = (Northern Hemispheric) season
!        1 --> winter (Dec, Jan, Feb)
!        2 --> spring (Mar, Apr, May)
!        3 --> summer (Jun, Jul, Aug)
!        4 --> autumn (Sep, Oct, Nov)
!
!*****************************************************************************
! 1)  Determine which parameterization the box needs.
!*****************************************************************************
! Error Check.
! If you'd like to see the specifics for the model box you're
! calculating OH, change value of NLOOK.
!
      NLOOK=1  ! DEBUG EY 
      IF (NLOOK.EQ.1) THEN
         print*,''
         print*,'OH_LAT   =',OH_LAT
         print*,'OH_LON   =',OH_LON
         print*,'OH_PRESS =',OH_PRESS
         print*,'OH_NOX   =',OH_NOX
         print*,'OH_ISOP  =',OH_ISOP
         print*,'OH_O3    =',OH_O3
         print*,'OH_SEASON=',OH_SEASON
      ENDIF
!
!*****************************************************************************
! SR SKIP prevents calling OH parameterizations in regions where OH is very low,
! resets the values of isoprene and NOx to keep from calling nonexistent
! parameterizations, and performs other miscellaneous functions.
!
      CALL SKIP(DOskip,COLON,COLAT,COALT, &
                INDVARA, INDVARB, INDVARC, INDVARD)
!
! If DOskip = 1, set [OH] to low value (i.e., skip parameterization).
!
      IF (DOskip.EQ.1) GOTO 320
!
! In subdomains where OH is very low or negligible due to low
!  sunlight (e.g., high latitudes in winter), concentrations of OH are
!  set to climatological mean values as a function of latitude,
!  altitude and season. No parameterized OH is calculated. 
!
      IF (OH_O3.LT.1.) DOskip=2
      IF (DOskip.EQ.2) THEN
         CALL INTERPOH()
         GOTO 320
      ENDIF
!
!*************
! Error Check.
! Surface
!       IF(OH_PRESS.LT.PRESSES(1)) goto 320
!
! MT
!       IF(OH_PRESS.GE.PRESSES(1)) goto 320
!       IF(OH_PRESS.LT.PRESSES(2)) goto 320
!
! UT
!       IF(OH_PRESS.GE.PRESSES(2)) goto 320
!
!
!*****************************************************************************
! B) REMOTE:  Unpolluted Domain:  Low NOx, No Isoprene (<10 ppt)
!             Surface Layer, MT, & UT (NOx <50 ppt)
!*****************************************************************************
! Surface & MT
      IF (OH_ISOP.LE.10..AND.OH_O3.LE.45.AND. &
     &    OH_NOX.LE.CNOXS(7).AND.OH_PRESS.GE.PRESSES(2)) THEN
!     
         CALL REMOTE(NDFUNCS)
!
!    Assign appropriate independent variables to be used in
!     OH calculation.
!
         IF (OH_PRESS.GE.PRESSES(1)) THEN
            INDVAR(1:MXVAR)=INDVARA(COLON,COLAT,COALT,1:MXVAR)
            OOB=1
         ELSE
            INDVAR(1:MXVAR)=INDVARB(COLON,COLAT,COALT,1:MXVAR)
            OOB=2
         ENDIF
!
         GOTO 321
!
      ENDIF
! UT
      IF (OH_ISOP.LE.10..AND.OH_O3.LE.100..AND. &
     &    OH_NOX.LE.CNOXS(6).AND.OH_PRESS.LT.PRESSES(2)) THEN
!
         print*, ' REMOTE PARAM '
         CALL REMOTE(NDFUNCS)
!
!    Assign appropriate independent variables to be used in
!     OH calculation.
!
         INDVAR(1:MXVAR)=INDVARB(COLON,COLAT,COALT,1:MXVAR)
         OOB=2
!
         GOTO 321
!
      ENDIF
!
!*****************************************************************************
! END B) REMOTE
!*****************************************************************************
!
      NDFUNCS=NDFUNCS+NDFUNCSB1+NDFUNCSB2+NDFUNCSB3
!
!*****************************************************************************
! A) CLEAN:  Unpolluted Domain:  Low NOx, No Isoprene (<10 ppt) 
!            Surface Layer (NOx <300 ppt), Middle & Up Tropospheres
!*****************************************************************************
!
      IF (OH_ISOP.LE.10.) THEN
         IF((OH_NOX.LE.CNOXS(1).AND.OH_PRESS.GE.PRESSES(2)).OR.   &
     &        (OH_NOX.LE.CNOXS(2).AND.OH_PRESS.LT.PRESSES(2))) THEN
!     
            PRINT*, 'CLEAN PARAM'
	    CALL CLEAN(NDFUNCS)
	    
!
!    Assign appropriate independent variables to be used in
!     OH calculation.
!
            IF (OH_PRESS.GE.PRESSES(1)) THEN
               INDVAR(1:MXVAR)=INDVARA(COLON,COLAT,COALT,1:MXVAR)
               OOB=1
            ELSE
               INDVAR(1:MXVAR)=INDVARB(COLON,COLAT,COALT,1:MXVAR)
               OOB=2
            ENDIF
!
            GOTO 321
!     
         ENDIF
!
      ENDIF
!
!*****************************************************************************
! END A) CLEAN
!*****************************************************************************
!
      NDFUNCS=NDFUNCS+NDFUNCSA1+NDFUNCSA2+NDFUNCSA3
!
!*****************************************************************************
! C) CLEAN with ISOPRENE:  Low NOx, Isoprene (>10 ppt) 
!            Surface Layer (NOx <300 ppt), Mid Troposphere 
!            (NOx <300 ppt), Up Troposphere (NOx <500ppt)
!*****************************************************************************
!
      IF (OH_ISOP.GT.10.) THEN
!
         IF ((OH_NOX.LE.CNOXS(1).AND.OH_PRESS.GE.PRESSES(2)).OR.   &
     &        (OH_NOX.LE.CNOXS(2).AND.OH_PRESS.LT.PRESSES(2))) THEN
!
            IF (OH_PRESS.GE.PRESSES(1)) THEN
               IF (DOskip.EQ.3) THEN
                  BOH(COLON,COLAT,COALT)=1.E5
                  GOTO 322
               ENDIF
!
               IF (DOskip.EQ.4) THEN
                  BOH(COLON,COLAT,COALT)=1.E4
                  GOTO 322
               ENDIF
            ENDIF
!
            PRINT*, 'CLEANISOP PARAM'		
            CALL CLEANISOP(NDFUNCS)
!
            IF (OH_PRESS.GE.PRESSES(1)) THEN
               INDVAR(1:MXVAR)=INDVARC(COLON,COLAT,COALT,1:MXVAR)
               OOB=3
            ELSE
               INDVAR(1:MXVAR)=INDVARD(COLON,COLAT,COALT,1:MXVAR)
               OOB=4
            ENDIF
!     
            GOTO 321
!     
         ENDIF
!     
      ENDIF
!
!*****************************************************************************
! END C) CLEAN with ISOPRENE
!*****************************************************************************
!
      NDFUNCS=NDFUNCS+NDFUNCSC1+NDFUNCSC2+NDFUNCSC3
!
!*****************************************************************************
! D) MIDNOx SUBDOMAIN with ISOPRENE: 
!            Mid NOx (>300 ppt <1000), Isoprene
!            Surface (see F for Middle Troposphere)
!*****************************************************************************
!
      IF (OH_NOX.GT.CNOXS(1).AND.   &
     &     OH_NOX.LE.CNOXS(3).AND.OH_PRESS.GE.PRESSES(1)) THEN
!
	 PRINT*, 'MIDNOx with ISOP PARAM'
         CALL MODPOLLUTED(NDFUNCS)
!
         INDVAR(1:MXVAR)=INDVARC(COLON,COLAT,COALT,1:MXVAR)
         OOB=3
!
         GOTO 321
!
      ENDIF
!     
!*****************************************************************************
! END D) MIDNOx SUBDOMAIN with ISOPRENE - Surface
!*****************************************************************************
!
      NDFUNCS=NDFUNCS+NDFUNCSD1
!
!*****************************************************************************
! E) HINOx SUBDOMAIN with ISOPRENE
!               Hi NOx (>1000), Isoprene
!*****************************************************************************
!
      IF (OH_NOX.GT.CNOXS(3).AND.OH_PRESS.GE.PRESSES(1)) THEN
!
          PRINT*, 'Hi NOX with ISOP PARAM'
         CALL HIPOLLUTED(NDFUNCS)
!
         INDVAR(1:MXVAR)=INDVARC(COLON,COLAT,COALT,1:MXVAR)
         OOB=3
!
         GOTO 321
!     
      ENDIF
!
!*****************************************************************************
! END E) HINOx SUBDOMAIN with ISOPRENE
!*****************************************************************************
!
      NDFUNCS=NDFUNCS+NDFUNCSE1
!
!*****************************************************************************
! F) MIDNOx SUBDOMAIN with ISOPRENE
!            Mid NOx (>300 ppt <1000), Isoprene
!            Middle Troposphere 
!****************************************************************************
!     
      IF(OH_NOX.GT.CNOXS(1).AND.OH_NOX.LE.CNOXS(3).AND. &
     &   OH_PRESS.LT.PRESSES(1).AND.OH_PRESS.GE.PRESSES(2)) THEN
!     
	 PRINT*, 'MIDNOx with ISOPRENE '
         CALL MODPOLLUTED(NDFUNCS)
!     
         INDVAR(1:MXVAR)=INDVARD(COLON,COLAT,COALT,1:MXVAR)
         OOB=4
!     
         GOTO 321
!
      ENDIF
!
!*****************************************************************************
! END F) MIDNOx SUBDOMAIN with ISOPRENE - MT
!*****************************************************************************
!
      NDFUNCS=NDFUNCS+NDFUNCSF1
!
!*****************************************************************************
! Error Check.
!
      PRINT*,'BOX LIES NOWHERE!'
      PRINT*,'STOPPED IN SR GETOH!'
      PRINT*,'*********************'
      PRINT*,'OH_LAT=',OH_LAT
      PRINT*,'OH_PRESS=',OH_PRESS
      PRINT*,'OH_NOX=',OH_NOX
      PRINT*,'OH_ISOP=',OH_ISOP
      PRINT*,'OH_SEASON=',OH_SEASON
      PRINT*,'*********************'
      STOP
!     
!*****************************************************************************
!*****************************************************************************
! 2)  Calculate the parameterized OH.
!*****************************************************************************
!*****************************************************************************
!
 321  CALL CALC_OH(NDFUNCS,OOB)

 320  BOH(COLON,COLAT,COALT)=PARAMOH

!bnd
      IF(DOskip.EQ.2) GOTO 322
!bnd
!
!***********************************************************
! SR correctOH: Correct aerosol problem from chem1d.
!***********************************************************
      IF(BOH(COLON,COLAT,COALT).GT.0.) THEN
         CALL correctOH(BOH,COLON,COLAT,COALT)
      ENDIF

!bnd      GOTO 330

       
! Correct problem in the tropical UT
!  SH
       IF(OH_LAT.LE.ALATS(4).AND.OH_LAT.GT.-50.) THEN
         IF(OH_PRESS.LT.130) THEN
              BOH(COLON,COLAT,COALT)=BOH(COLON,COLAT,COALT)*valSH(OHLMN*4)
           GOTO 322
         ENDIF
         IF(OH_PRESS.LT.150) THEN
              BOH(COLON,COLAT,COALT)=BOH(COLON,COLAT,COALT)*valSH(OHLMN*4-1)
           GOTO 322
         ENDIF
         IF(OH_PRESS.LT.180) THEN
              BOH(COLON,COLAT,COALT)=BOH(COLON,COLAT,COALT)*valSH(OHLMN*4-2)
           GOTO 322
         ENDIF
         IF(OH_PRESS.LT.220) THEN
              BOH(COLON,COLAT,COALT)=BOH(COLON,COLAT,COALT)*valSH(OHLMN*4-3)
           GOTO 322
         ENDIF

       ENDIF
!  NH
       IF(OH_LAT.LE.50.AND.OH_LAT.GT.ALATS(4)) THEN
         IF(OH_PRESS.LT.130) THEN
              BOH(COLON,COLAT,COALT)=BOH(COLON,COLAT,COALT)*valNH(OHLMN*4)
           GOTO 322
         ENDIF
         IF(OH_PRESS.LT.150) THEN
              BOH(COLON,COLAT,COALT)=BOH(COLON,COLAT,COALT)*valNH(OHLMN*4-1)
           GOTO 322
         ENDIF
         IF(OH_PRESS.LT.180) THEN
              BOH(COLON,COLAT,COALT)=BOH(COLON,COLAT,COALT)*valNH(OHLMN*4-2)
           GOTO 322
         ENDIF
         IF(OH_PRESS.LT.220) THEN
              BOH(COLON,COLAT,COALT)=BOH(COLON,COLAT,COALT)*valNH(OHLMN*4-3)
           GOTO 322
         ENDIF
       ENDIF


! decrease OH by 20%
! this allows feedback of CO - amplification!!!!!!!
!       BOH(COLON,COLAT,COALT)=BOH(COLON,COLAT,COALT)*0.8
       
!***********************************************************
 322   CONTINUE
!bnd
       IF(OH_PRESS.LT.300.AND.BOH(COLON,COLAT,COALT).GT.1.5e6) THEN
          BOH(COLON,COLAT,COALT)=BOH(COLON,COLAT,COALT)*0.33
       ENDIF
!bnd
 330    CONTINUE        
!
       RETURN

       END SUBROUTINE GETOH
