!**********************************************************************
      SUBROUTINE PARAMC(BLAT,BSEASON,COCOUNT,NSEAS,NLATS,ALATS)
!**********************************************************************
      IMPLICIT NONE
      INTEGER K,M,J,BSEASON,COCOUNT,NSEAS,NLATS
      REAL*8 BLAT,ALATS(NLATS)
!
! Information used in Error Check.
!
      INTEGER NSTOP,NERROR,CSEASON1(5),CSEASON2(5),CSEASON3(6), &
     &        CSEASON4(6)
      REAL*8 DLATS2(2),DLATS3(4),DLATS4(4),DLATS5(4),DLATS6(4), &
     &        DLATS7(3),DLATS8(1)
      DATA CSEASON1/1,3,7,11,15/
      DATA CSEASON2/4,8,12,16,19/
      DATA CSEASON3/5,9,13,17,20,22/
      DATA CSEASON4/2,6,10,14,18,21/
      DATA DLATS2/1,2/
      DATA DLATS3/3,4,5,6/
      DATA DLATS4/7,8,9,10/
      DATA DLATS5/11,12,13,14/
      DATA DLATS6/15,16,17,18/
      DATA DLATS7/19,20,21/
      DATA DLATS8/22/
!**********************************************************************
! Created by Bryan Duncan.
!**********************************************************************
! SR PARAMC is called by SR CLEANISOP.  Based on season and latitude, this
! subroutine choses the appropriate identification number of the
! parameterization required.
!
!  ----------------------------------------------------------
!  | Latitude   |   DJF    |   MAM    |   JJA    |   SON    |
!  ----------------------------------------------------------
!  |   > 80 N   |   AVG    |   AVG    |    *     |   AVG    |
!  |  60-80 N   |   AVG    |    *     | 22 44 66 |   AVG    |
!  |  40-60 N   |   AVG    | 19 41 63 | 20 42 64 | 21 43 65 |
!  |  30-40 N   | 15 37 59 | 16 38 60 | 17 39 61 | 18 40 62 |
!  |   0-30 N   | 11 33 55 | 12 34 56 | 13 35 57 | 14 36 58 |
!  |   0-30 S   |  7 29 51 |  8 30 52 |  9 31 53 | 10 32 54 |
!  |  30-40 S   |  3 25 47 |  4 26 48 |  5 27 49 |  6 28 50 |
!  |  40-60 S   |  1 23 45 |    *     |    *     |  2 24 46 |
!  |  60-80 S   |     *    |   AVG    |   AVG    |     *    |
!  |   > 80 S   |     *    |   AVG    |   AVG    |    AVG   |
!  ----------------------------------------------------------
!
! The table above illustrates the spatial and temporal coverage of the
! parameterizations found in the "CLEANISOP" subdomain.  In the boxes labeled
! "AVG", the OH is sufficiently low so that the climatological mean OH for
! that season and latitude is used (Spivakovsky et al., 1999).  (For these
! boxes, the parameterized OH is not calculated.)  For all other boxes, the
! parameterized OH is calculated. The 3 numbers in the box correspond
! to the identification numbers of the parameterization for that box.
! The first number is for the surface layer, the second for the middle
! tropospheric layer and the third for the upper tropospheric layer.
! For instance, the parameterization the code would use to predict OH for
! April, 15N, middle troposphere has the identification number 34.
! The boxes designated by an asterisk are not covered by this domain.
!**********************************************************************
! List of Variables & Arrays
!
! COCOUNT = identification number of parameterization.
!
!**********************************************************************
! Loop over latitudes.
!
      DO K=2,NLATS
!
!***************
!
        IF(BLAT.GE.ALATS(K).AND.K.NE.NLATS) THEN
          GOTO 298
        ENDIF
!
           NSTOP=0
           NERROR=1
!
        IF(BLAT.LT.ALATS(1)) NSTOP=1
        IF(BLAT.LT.ALATS(2).AND.BLAT.GE.ALATS(1)) THEN
           NERROR=0
          IF(BSEASON.EQ.2) NSTOP=1
          IF(BSEASON.EQ.3) NSTOP=1
        ENDIF
!
        IF(BLAT.LT.ALATS(3).AND.BLAT.GE.ALATS(2)) THEN
          COCOUNT=2
        ENDIF
!
        IF(BLAT.LT.ALATS(7).AND.BLAT.GE.ALATS(3)) THEN
          COCOUNT=(K-1)*NSEAS-6
          IF(BLAT.GE.ALATS(6)) THEN
!              COCOUNT=COCOUNT-1
              IF(BSEASON.EQ.1) NSTOP=1
          ENDIF
        ENDIF
        IF(BLAT.GE.ALATS(7)) THEN
          IF(BSEASON.NE.3) NSTOP=1
          COCOUNT=(K-1)*NSEAS-2-1
        ENDIF
!
! Error Check.
!
        IF(NERROR.EQ.1.AND.COCOUNT.EQ.0) THEN
          PRINT*,'Latitude band not found!'
          NSTOP=1
        ENDIF
!
! NSTOP is an error check to see if a parameterization which
!       does not exist has been called.  This error indicates
!       that there is a problem in SR SKIPS.

        IF(NSTOP.EQ.1) THEN
          PRINT*,'STOPPED IN SR PARAMC'
          STOP
        ENDIF
!
!***********
! Loop over seasons.
!
        DO M=1,NSEAS
!
!***********
        IF(BLAT.LT.ALATS(2).AND.BLAT.GE.ALATS(1).AND. &
     &       (M.EQ.2.OR.M.EQ.3)) GOTO 299
        IF(BLAT.LT.ALATS(7).AND.BLAT.GE.ALATS(6).AND.M.EQ.1) GOTO 299
        IF(BLAT.GE.ALATS(7).AND.M.NE.3) GOTO 299
!
          IF(BSEASON.GT.M) THEN
            COCOUNT=COCOUNT+1
            GOTO 299
          ENDIF
!
                 COCOUNT=COCOUNT+1
!
                 GOTO 300
!
 299        CONTINUE
!***********
        ENDDO ! End Loop over Seasons.
!***********
 298     CONTINUE
!***************
      ENDDO   ! End Loop over Latitude Bands.
!***************
!
 300        CONTINUE
!
! Error Check.

      IF(BSEASON.EQ.1) THEN
        DO J=1,5
         IF(COCOUNT.EQ.CSEASON1(J)) GOTO 301
         IF(COCOUNT.EQ.CSEASON1(J)+22) GOTO 301
         IF(COCOUNT.EQ.CSEASON1(J)+44) GOTO 301
        ENDDO
        PRINT*,'Season is DJF:  Did not call a DJF param!'
      ENDIF
      IF(BSEASON.EQ.2) THEN
        DO J=1,5
         IF(COCOUNT.EQ.CSEASON2(J)) GOTO 301
         IF(COCOUNT.EQ.CSEASON2(J)+22) GOTO 301
         IF(COCOUNT.EQ.CSEASON2(J)+44) GOTO 301
        ENDDO
        PRINT*,'Season is MAM:  Did not call a MAM param!'
      ENDIF
      IF(BSEASON.EQ.3) THEN
        DO J=1,6
         IF(COCOUNT.EQ.CSEASON3(J)) GOTO 301
         IF(COCOUNT.EQ.CSEASON3(J)+22) GOTO 301
         IF(COCOUNT.EQ.CSEASON3(J)+44) GOTO 301
        ENDDO
        PRINT*,'Season is JJA:  Did not call a JJA param!'
      ENDIF
      IF(BSEASON.EQ.4) THEN
        DO J=1,6
         IF(COCOUNT.EQ.CSEASON4(J)) GOTO 301
         IF(COCOUNT.EQ.CSEASON4(J)+22) GOTO 301
         IF(COCOUNT.EQ.CSEASON4(J)+44) GOTO 301
        ENDDO
        PRINT*,'Season is SON:  Did not call a SON param!'
      ENDIF

        PRINT*,'Stopped in SR paramC.'
        PRINT*,'COCOUNT = ',COCOUNT
        STOP

 301  CONTINUE
!
      IF(BLAT.LT.ALATS(2).AND.BLAT.GE.ALATS(1)) THEN
        DO J=1,2
         IF(COCOUNT.EQ.DLATS2(J)) GOTO 302
        ENDDO
        PRINT*,'Latitude is not 40-60S: Did not call a 40-60S param!'
      ENDIF

      IF(BLAT.LT.ALATS(3).AND.BLAT.GE.ALATS(2)) THEN
        DO J=1,4
         IF(COCOUNT.EQ.DLATS3(J)) GOTO 302
        ENDDO
        PRINT*,'Latitude is not 30-40S: Did not call a 30-40S param!'
      ENDIF

      IF(BLAT.LT.ALATS(4).AND.BLAT.GE.ALATS(3)) THEN
        DO J=1,4
         IF(COCOUNT.EQ.DLATS4(J)) GOTO 302
        ENDDO
        PRINT*,'Latitude is not 0-30S: Did not call a 0-30S param!'
      ENDIF

      IF(BLAT.LT.ALATS(5).AND.BLAT.GE.ALATS(4)) THEN
        DO J=1,4
         IF(COCOUNT.EQ.DLATS5(J)) GOTO 302
        ENDDO
        PRINT*,'Latitude is not 0-30N: Did not call a 0-30N param!'
      ENDIF

      IF(BLAT.LT.ALATS(6).AND.BLAT.GE.ALATS(5)) THEN
        DO J=1,4
         IF(COCOUNT.EQ.DLATS6(J)) GOTO 302
        ENDDO
        PRINT*,'Latitude is not 30-40N: Did not call a 30-40N param!'
      ENDIF

      IF(BLAT.LT.ALATS(7).AND.BLAT.GE.ALATS(6)) THEN
        DO J=1,3
         IF(COCOUNT.EQ.DLATS7(J)) GOTO 302
        ENDDO
        PRINT*,'Latitude is not 40-60N: Did not call a 40-60N param!'
      ENDIF

      IF(BLAT.GE.ALATS(7)) THEN
        DO J=1,1
         IF(COCOUNT.EQ.DLATS8(J)) GOTO 302
        ENDDO
        PRINT*,'Latitude is not gt 60N: Did not call a 60-80N param!'
      ENDIF

        PRINT*,'Stopped in SR paramC.'
        PRINT*,'COCOUNT = ',COCOUNT
        STOP
!
 302  RETURN

      END SUBROUTINE PARAMC
