!**********************************************************************
      SUBROUTINE PARAMF(BLAT,BSEASON,COCOUNT,NSEAS,NLATS,ALATS)
!**********************************************************************
      IMPLICIT NONE
      INTEGER K,M,BSEASON,COCOUNT
      INTEGER NSEAS,NLATS,NSTOP
      REAL*8 BLAT,ALATS(NLATS)
!**********************************************************************
! Created by Bryan Duncan.
!**********************************************************************
! SR PARAMF is called by SR MODPOLLUTEDISOP.  Based on season and latitude,
! this subroutine choses the appropriate identification number of the
! parameterization required for middle troposphere only.
!
!  ----------------------------------------------------------
!  | Latitude   |   DJF    |   MAM    |   JJA    |   SON    |
!  ----------------------------------------------------------
!  |   > 80 N   |   AVG    |   AVG    |    *     |   AVG    |
!  |  60-80 N   |   AVG    |    *     |    *     |   AVG    |
!  |  40-60 N   |   AVG    |    9     |   10     |   11     |
!  |  30-40 N   |    5     |    6     |    7     |    8     |
!  |   0-30 N   |    3     |    4     |    *     |    *     |
!  |   0-30 S   |    *     |    *     |    1     |    2     |
!  |  30-40 S   |    *     |    *     |    *     |    *     |
!  |  40-60 S   |    *     |    *     |    *     |    *     |
!  |  60-80 S   |    *     |   AVG    |   AVG    |    *     |
!  |   > 80 S   |    *     |   AVG    |   AVG    |   AVG    |
!  ----------------------------------------------------------
!
! The table above illustrates the spatial and temporal coverage of the
! parameterizations found in the "MODPOLLUTEDISOP" subdomain. In boxes labeled
! "AVG", the OH is sufficiently low so that the climatological mean OH for
! that season and latitude is used (Spivakovsky et al., 1999).  (For these
! boxes, the parameterized OH is not calculated.)  For all other boxes,
! the parameterized OH is calculated.  The number in the box corresponds
! to the identification number of the parameterization for that box.
! For instance, the parameterization the code would use to predict OH for
! 35N in April has the identification number 2.
! The boxes designated by an asterisk are not covered by this domain.
!**********************************************************************
! List of Variables & Arrays
!
! COCOUNT = identification number of parameterization.
!
!**********************************************************************
!
          NSTOP=0
!
        IF(BLAT.LE.ALATS(3)) NSTOP=1 
        IF(BLAT.GE.ALATS(7)) NSTOP=1
!
        IF(BLAT.LE.ALATS(4)) THEN
          IF(BSEASON.EQ.3) COCOUNT=1
          IF(BSEASON.EQ.4) COCOUNT=2
        ENDIF
        IF(BLAT.LE.ALATS(5).AND.BLAT.GT.ALATS(4)) THEN
          IF(BSEASON.EQ.1) COCOUNT=3
          IF(BSEASON.EQ.2) COCOUNT=4
        ENDIF
        IF(BLAT.LE.ALATS(6).AND.BLAT.GT.ALATS(5)) THEN
          IF(BSEASON.EQ.1) COCOUNT=5
          IF(BSEASON.EQ.2) COCOUNT=6
          IF(BSEASON.EQ.3) COCOUNT=7
          IF(BSEASON.EQ.4) COCOUNT=8
        ENDIF
        IF(BLAT.GT.ALATS(6)) THEN
          IF(BSEASON.EQ.1) NSTOP=1
          IF(BSEASON.EQ.2) COCOUNT=9
          IF(BSEASON.EQ.3) COCOUNT=10
          IF(BSEASON.EQ.4) COCOUNT=11
        ENDIF
!
!**********************************************************************
! Error Check.
!
! NSTOP is an error check to see if a parameterization which
!       does not exist has been called.  This error indicates
!       that there is a problem in SR SKIPS.

        IF(NSTOP.EQ.1) THEN
          PRINT*,'STOPPED IN SR PARAMF'
          STOP
        ENDIF
!
      IF(BSEASON.EQ.1) THEN
         IF(COCOUNT.EQ.3.OR.COCOUNT.EQ.5) GOTO 301
        PRINT*,'Season is DJF:  Did not call a DJF param!'
      ENDIF
!
      IF(BSEASON.EQ.2) THEN
         IF(COCOUNT.EQ.4.OR.COCOUNT.EQ.6.OR.COCOUNT.EQ.9) GOTO 301
        PRINT*,'Season is MAM:  Did not call a MAM param!'
      ENDIF
!
      IF(BSEASON.EQ.3) THEN
         IF(COCOUNT.EQ.1.OR.COCOUNT.EQ.7.OR.COCOUNT.EQ.10) GOTO 301
        PRINT*,'Season is JJA:  Did not call a JJA param!'
      ENDIF
!
      IF(BSEASON.EQ.4) THEN
         IF(COCOUNT.EQ.2.OR.COCOUNT.EQ.8.OR.COCOUNT.EQ.11) GOTO 301
        PRINT*,'Season is SON:  Did not call a SON param!'
      ENDIF
!
        PRINT*,'Stopped in SR paramF - 1.'
        PRINT*,'COCOUNT = ',COCOUNT
        STOP
!
 301  CONTINUE
!
      IF(BLAT.LE.ALATS(4)) THEN
       IF(COCOUNT.LE.2) GOTO 302
      ENDIF
      IF(BLAT.LE.ALATS(5).AND.BLAT.GT.ALATS(4)) THEN
       IF(COCOUNT.EQ.3.OR.COCOUNT.EQ.4) GOTO 302
      ENDIF
      IF(BLAT.LE.ALATS(6).AND.BLAT.GT.ALATS(5)) THEN
       IF(COCOUNT.GE.5.AND.COCOUNT.LE.8) GOTO 302
      ENDIF
      IF(BLAT.GT.ALATS(6)) THEN
       IF(COCOUNT.GE.9) GOTO 302
      ENDIF

!
        PRINT*,'Stopped in SR paramF - 2.'
        PRINT*,'COCOUNT = ',COCOUNT
        STOP
!
!**********************************************************************
!
 302  RETURN

      END SUBROUTINE PARAMF
