! **********************************************************************
      SUBROUTINE READ_COEFF (unitNum, fileName)
! **********************************************************************
! USES: 
      USE MAPL_Mod

      IMPLICIT NONE
#     include "CMN_OH"
      INTEGER :: unitNum
      CHARACTER (len=*) :: fileName

      INTEGER JTERM,PARSUM,NFT
      INTEGER J,L,K,KK,LH,KR,IDONE,M
! **********************************************************************
! Created by Bryan Duncan.
! **********************************************************************
! This SR reads in information from the file 'start_information'. 
! The information is needed to calculate the OH using parameterizations
! and includes polynomial coefficients, ranges of independent variables,
! etc.  See the text of this code for descriptions of information. 
! *********************************************************************
!      print*,'Reading info for parameterization in SR read_coeff.'
!
! NGENERICFILE = filenumber specified by user in CMN_OH.
!
      OPEN(UNIT=unitNum, FILE=TRIM(fileName),STATUS='OLD')

!      unitNum = NGENERICFILE
!      OPEN(UNIT=NGENERICFILE,FILE='start_information',STATUS='OLD')
      REWIND(unitNum)
!
!*********************************************************************
! Loop over all NPARAM parameterizations.  There are over 200 individual
! parameterizations that are used to describe the tropospheric OH
! field.
!
! NPARAM  = total number of individual parameterizations.
!*********************************************************************
!
      DO M=1,NPARAM
!
!*********************************************************************
! The 24-hour average concentration of OH is represented as a set 
!  of high-order polynomials in variables such as temperature, latitude,
!  declination and the concentrations of ozone, water vapor, carbon
!  monoxide, nitrogen oxides (as a family), and hydrocarbons.  (See
!  SR GETINFO for a complete list of independent variables.)
!
! MOVAR = the total number of independent variables.
!         
      READ(unitNum,*) MOVAR(M)

!
! Loop over MOVAR independent variables reading in parameter ranges.
! These ranges for chemical species and physical parameters were estimated from
! observations and monthly average values predicted by a global model. The global
! model used for estimating ranges was the Harvard-GEOS model of transport and
! chemistry driven by assimilated meteorological data from the Goddard Earth
! Observing System Data Assimilation Office (GEOSDAO) (Bey et al. [1999]). 
!
! Bey, I., R. Yantosca and D. Jacob:  Export of Pollutants from Eastern Asia: A
! Simulation of the PEM-West (B) Aircraft Mission Using a 3-D Model Driven by
! Assimilated Meteorological Fields, Presentation at American Geophysical Union
! Spring Meeting, Boston, 1999.
!
! RANGEM = ranges for chemical species and physical parameters
!
        DO J=1,MOVAR(M)
!
          READ(unitNum,*)RANGEM(M,J,1),RANGEM(M,J,2)
!
        ENDDO
!
! IDENTOLD,ELTODO,IROWST,L = information of domain divisions. 
!
! NDONE = number of polynomials used in one parameterization. If there are
!         no domain divisions, NDONE=1.
!
      READ(unitNum,*) L
!
      DO K=1,L 
        READ(unitNum,*) IROWST(M,K,0), &
     &       (IROWST(M,K,KR),KR=1,IROWST(M,K,0))
      ENDDO
!
        READ(unitNum,*) KK
!
      DO LH=1,KK
        READ(unitNum,*) (ELTODO(M,LH,J),J=1,MOVAR(M)+2)
      ENDDO
!
        READ(unitNum,*)NDONE(M),(IDENTOLD(M,J),J=1,NDONE(M))
!
! *********************************************
! Error Check.
       IF(NDONE(M).GT.MXDONE) THEN
         PRINT*,'SR READ_COEFF: NDONE > MXDONE'
         PRINT*,'NDONE(M)=',NDONE(M)
         PRINT*,'MXDONE  =',MXDONE
         PRINT*,'Increase MXDONE in CMN_OH to be greater than NDONE.'
         STOP
       ENDIF 
! *********************************************
! Read in number of terms and coefficients used in each polynomial.
!
! NENDW = number of terms in polynomial.
!
! COEFF = polynomial coefficients.
!
       DO IDONE=1,NDONE(M)
        READ(unitNum,*) NFT,NENDW(M,IDONE)
          DO JTERM=1,NENDW(M,IDONE)
            READ(unitNum,*) COEFF(M,IDONE,JTERM)
          ENDDO
       ENDDO 
!
! ACCUR = RMS error associated with individual parameterization.
!
      READ(unitNum,*) ACCUR(M)
!
!*********************************************************************
      ENDDO
!*********************************************************************
!
      CLOSE(unitNum)
!
!*********************************************************************
!
! Sum up the total number of individual parameterizations in PARSUM.  
! Since each parameterization may be described by more than one
! polynomial, the total number of polynomials are summed in the
! variables NDFUNCS*. NDFUNCS* are used only for bookkeeping. 
! See SR GETOH for a description of variables NPARAMA-F.
!
!*********************************************************************
        PARSUM=0
!*********************************************************************

        NDFUNCSB1=0
      DO J=1,NPARAMB
        NDFUNCSB1=NDFUNCSB1+NDONE(J)
      ENDDO

!*********************************************************************
        PARSUM=PARSUM+NPARAMB
!*********************************************************************

        NDFUNCSB2=0
      DO J=PARSUM+1,PARSUM+NPARAMB
        NDFUNCSB2=NDFUNCSB2+NDONE(J)
      ENDDO

!*********************************************************************
       PARSUM=PARSUM+NPARAMB
!*********************************************************************

        NDFUNCSB3=0
      DO J=PARSUM+1,PARSUM+NPARAMB
        NDFUNCSB3=NDFUNCSB3+NDONE(J)
      ENDDO

!*********************************************************************
       PARSUM=PARSUM+NPARAMB
!*********************************************************************
 
        NDFUNCSA1=0
      DO J=PARSUM+1,PARSUM+NPARAMA
        NDFUNCSA1=NDFUNCSA1+NDONE(J)       
      ENDDO

!*********************************************************************
       PARSUM=PARSUM+NPARAMA
!*********************************************************************

        NDFUNCSA2=0
      DO J=PARSUM+1,PARSUM+NPARAMA
        NDFUNCSA2=NDFUNCSA2+NDONE(J)       
      ENDDO

!*********************************************************************
       PARSUM=PARSUM+NPARAMA
!*********************************************************************

        NDFUNCSA3=0
      DO J=PARSUM+1,PARSUM+NPARAMA
        NDFUNCSA3=NDFUNCSA3+NDONE(J)       
      ENDDO

!*********************************************************************
       PARSUM=PARSUM+NPARAMA
!*********************************************************************

        NDFUNCSC1=0
      DO J=PARSUM+1,PARSUM+NPARAMC
        NDFUNCSC1=NDFUNCSC1+NDONE(J)       
      ENDDO

!*********************************************************************
       PARSUM=PARSUM+NPARAMC
!*********************************************************************

        NDFUNCSC2=0
      DO J=PARSUM+1,PARSUM+NPARAMC
        NDFUNCSC2=NDFUNCSC2+NDONE(J)       
      ENDDO

!*********************************************************************
       PARSUM=PARSUM+NPARAMC
!*********************************************************************

       NDFUNCSC3=0
      DO J=PARSUM+1,PARSUM+NPARAMC
        NDFUNCSC3=NDFUNCSC3+NDONE(J)
      ENDDO

!*********************************************************************
       PARSUM=PARSUM+NPARAMC
!*********************************************************************

        NDFUNCSD1=0
      DO J=PARSUM+1,PARSUM+NPARAMD
        NDFUNCSD1=NDFUNCSD1+NDONE(J)       
      ENDDO

!*********************************************************************
       PARSUM=PARSUM+NPARAMD
!*********************************************************************

        NDFUNCSE1=0
      DO J=PARSUM+1,PARSUM+NPARAME
        NDFUNCSE1=NDFUNCSE1+NDONE(J)       
      ENDDO

!*********************************************************************
       PARSUM=PARSUM+NPARAME
!*********************************************************************

        NDFUNCSF1=0
      DO J=PARSUM+1,PARSUM+NPARAMF
        NDFUNCSF1=NDFUNCSF1+NDONE(J)       
      ENDDO

!*********************************************************************
       PARSUM=PARSUM+NPARAMF
!*********************************************************************

! Error Check.
       IF(PARSUM.NE.NPARAM) THEN
          PRINT*,'SR READ_COEFF: PARSUM.NE.NPARAM!'
          PRINT*,'This means that NPARAM specified in CMN_OH does'
          PRINT*,'not equal NPARAM (i.e., PARSUM here) calculated'
          PRINT*,'in this SR.  Check to see why they differ and'
          PRINT*,'correct the problem.'
          STOP
       ENDIF
!
!*********************************************************************
 300  RETURN 
      END
