!**********************************************************************
      SUBROUTINE read_correction (unitNum, fileName)
!**********************************************************************
      IMPLICIT NONE
#     include "CMN_CF"

      INTEGER :: unitNum
      CHARACTER (len=*) :: fileName

      INTEGER I,J,K
!**********************************************************************
! Created by Bryan Duncan.
!**********************************************************************
! This SR reads in correction factors to remove the problem with high
! background aerosol (tau=0.1) in the OH parameterization.
! This file contains ratios of OH calculated assuming a uniform
! tau=0.01 over the globe and assuming tau=0.1. OH calculated by CMS, June, 2001.
! Ratio created by bnd, June, 2001.
!*************************************************************************
! List of Variables & Arrays
!
! correction = correction factor 
!
! NCMSALTS2 = number of altitude levels.
!
! NCMSLATS2 = number of latitude bands.
!
! NSEAS_CF  = number of seasons.
!
! NFILENUM  = filenumber specified by user in CMN_OH.
!**********************************************************************
      print*,'Reading in OH correction factors in SR read_correction.'
!
      OPEN(UNIT=unitNum, FILE=TRIM(fileName),STATUS='OLD')

!      unitNum = NFILENUM
!      OPEN(UNIT=NFILENUM,FILE='CorrectRatio',STATUS='OLD')
!
      correction(:,:,:)=1.
!
      READ(unitNum,2)
 2    FORMAT(//////////////)
!
      DO I=1,NSEAS_CF
!
         READ(unitNum,3)
 3       FORMAT()
!
         DO J=1,NCMSLATS2
            READ(unitNum,4)(correction(I,J,K),K=1,NCMSALTS2)
         ENDDO
!
      ENDDO
!
 4    FORMAT(4x,9(f5.3,2x)) 
!
      CLOSE(unitNum)
!
      DO I=1,NSEAS_CF
         DO J=1,NCMSLATS2
            DO K=1,NCMSALTS2
               IF (correction(I,J,K).LT.0..OR.correction(I,J,K).GT.2.) THEN
                  PRINT*,'Correction factor < 0!'
                  PRINT*,I,J,K,correction(I,J,K)
                  PRINT*,'Stopped in SR READ_CORRECTION.'
                  STOP
              ENDIF
            ENDDO
         ENDDO
      ENDDO
!
      print*,'Leaving SR read_correction.'

!**********************************************************************
      RETURN

      END SUBROUTINE read_correction
