!**********************************************************************
      SUBROUTINE readavgOH(unitNum, fileName)
!**********************************************************************
      IMPLICIT NONE
#     include "CMN_OH"
 
      INTEGER :: unitNum
      CHARACTER (len=*) :: fileName

      INTEGER I,J,K
!**********************************************************************
! Created by Bryan Duncan.
!**********************************************************************
! In subdomains where OH is very low or negligible due to low 
!  sunlight (e.g., high latitudes in winter), concentrations of OH are 
!  set to climatological mean values as a function of latitude,
!  altitude and season. This SR reads in the average OH
!  fields from Spivakovsky et al., "Three-dimensional climatological 
!  distribution of tropospheric OH: update and evaluation", accepted
!  to JGR, 1999.
!**********************************************************************
! List of Variables & Arrays
!
! avgOH = array containing the climatological OH values. 
!
! NCMSALTS = number of altitude levels of climatology.
!
! NCMSLATS = number of latitude bands of climatology.
!
! NSEAS    = number of seasons of climatlogy.
!
! NGENERICFILE   = filenumber specified by user in CMN_OH.
!**********************************************************************
! 
!      print*,'Reading in OH climatlogy of cms in readavgOH.'
!
      OPEN(UNIT=unitNum, FILE=TRIM(fileName),STATUS='OLD')
!
!      unitNum = NGENERICFILE
!      OPEN(UNIT=NGENERICFILE,FILE='avgOH',STATUS='OLD')
!
      READ(unitNum,2)
 2    FORMAT(//)
!
      DO I=1,NSEAS
!
         READ(unitNum,3)
 3       FORMAT(////)
!
         DO J=1,NCMSLATS
            READ(unitNum,4)(avgOH(I,J,K),K=1,NCMSALTS)
         ENDDO
!
      ENDDO
!
 4    FORMAT(5x,7(F6.2))
!
      CLOSE(unitNum)
!
!**********************************************************************
      RETURN

      END SUBROUTINE readavgOH
