#include "Raster.h"

 PROGRAM mkCatchParam

! !INTERFACE:
!
! !ARGUMENTS:
!
!  Usage = "mkCatchParam -x nx -y ny -g Gridname -b DL -m MA -l LD -s SD -e EASE"       
!     -x: Size of longitude dimension of raster. DEFAULT: 8640
!     -y: Size of latitude dimension of raster.  DEFAULT: 4320
!     -b: position of the dateline in the first box. DEFAULT: DC 
!     -g: Gridname  (name of the .til or .rst file without file extension)  
!     -l: Choice of LAI data set. DEFAULT : GSWP2
!         MODGEO  : Merged below GEOLAND2, MODIS, GSWP-2
!         GEOLAND2: 10-day climatology from the period 1999-2011 on 40320x20160 grid               
!         GSWP2   : Monthly climatology from the period 1982-1998 on 360x180 grid                  
!         MODIS   : 8-day climatology from the period 2000-2011 on  43200x21600 grid
!     -m: Choice of MODIS Albedo data. DEFAULT : MODIS2                                            
!         MODIS1 : 16-day Climatology from 1'x1 (21600x10800) MODIS data from the period 2000-2004 
!         MODIS2 : 8-day Climatology from 30"x30"(43200x21600) MODIS data from the period 2001-2011 
!     -s: Choice of Soil Data. DEFAULT :HWSD                                                       
!         HWSD : Merged HWSD-STATSGO2 soil properties on 43200x21600 with Woesten Parameters   
!         NGDC : Reynolds soil texture clsses on 4320x2160 with Cosby parameters                   
!     -e: EASE : This is optional if catchment.def file is available already or                    
!         the til file format is pre-Fortuna-2.                                                    
!     
!
! This program is good to generate  
! model, vegetation, soil, and MODIS albedo parameter files for the 
! catchment model implementation
!  
! Sarith Mahanama - March 23, 2012 
! Email: sarith.p.mahanama@nasa.gov

   use rmTinyCatchParaMod
   use process_hres_data

  implicit none
    integer              :: NC = i_raster, NR = j_raster
    character*128        :: GridName = ''
    character*128        :: ARG
    character*256        :: CMD
    character*1          :: opt
    character*8          :: LD = 'GSWP2'
    character*4          :: SD = 'NGDC'
    character*4          :: EASE ='    '
    character*2          :: DL ='DC'
    character*6          :: MA = 'MODIS2'
    integer              :: II, JJ, Type
    integer              :: I, J, iargc, nxt
    real*8               :: dx, dy, lon0
    logical :: regrid
    character(len=400), dimension (18) ::  Usage 
    character*128        ::  Grid2
    character*2 :: poles
    CHARACTER*100 :: gfile,fname,pdir,rstdir,c_data,gtopo30
    character*128        :: GridNameR = ''
    character*128        :: GridNameT = ''
    logical :: file_exists
    logical, parameter :: F25Tag = .false.
    logical :: ease_grid=.false., redo_modis=.false.
    character*40       :: lai_name 
    integer, parameter :: log_file = 998
    include 'netcdf.inc'	


   call system('cd data/ ; ln -s /discover/nobackup/rreichle/l_data/LandSurfaceData_for_Catchment_Parameters/ CATCH')
   call system('cd ..')

   gtopo30   = 'data/CATCH/srtm30_withKMS_2.5x2.5min.data'
   c_data    = 'data/CATCH/'

    USAGE(1) ="Usage: mkCatchParam -x nx -y ny -g Gridname -b DL -m MA -l LD -s SD -e EASE                       "
    USAGE(2) ="     -x: Size of longitude dimension of raster. DEFAULT: 8640                                     "
    USAGE(3) ="     -y: Size of latitude dimension of raster.  DEFAULT: 4320                                     "
    USAGE(4) ="     -g: Gridname  (name of the .til or .rst file without file extension)                         "
    USAGE(5) ="     -b: Position of the dateline in the first grid box (DC or DE). DEFAULT: DC                   "
    USAGE(6) ="     -l: Choice of LAI data set. DEFAULT : GSWP2                                                  "
    USAGE(7) ="         MODGEO  : Merged below GEOLAND2, MODIS, GSWP-2                                           "
    USAGE(8) ="         GEOLAND2: 10-day climatology from the period 1999-2011 on 40320x20160 grid               "
    USAGE(9) ="         GSWP2   : Monthly climatology from the period 1982-1998 on 360x180 grid                  "
    USAGE(10)="         MODIS   : 8-day climatology from the period 2000-2011  on 43200x21600 grid               "
    USAGE(11)="     -s: Choice of Soil Data. DEFAULT :HWSD                                                       "
    USAGE(12)="         HWSD : Merged HWSD-STATSGO2 soil properties on 43200x21600 with Woesten Parameters   "
    USAGE(13)="         NGDC : Reynolds soil texture classes on 4320x2160 with Cosby parameters                  "
    USAGE(14)="     -m: Choice of MODIS Albedo data. DEFAULT : MODIS2                                            "
    USAGE(15)="         MODIS1: 16-day Climatology from 1'x1'(21600x10800) MODIS data from the period 2000-2004  "
    USAGE(16)="         MODIS2: 8-day Climatology from 0.5'x0.5'(43200x21600) MODIS data from the period 2001-2011"
    USAGE(17)="     -e: EASE : This is optional if catchment.def file is available already or                    "          
    USAGE(18)="                    the til file format is pre-Fortuna-2.                                         "           

! Process Arguments                            
!------------------                            
    open (log_file, file ='clsm/mkCatchParam.log', status='unknown',form='formatted',action='write',POSITION='APPEND')
    CALL get_command (cmd)
    write (log_file,'(a)')trim(cmd)
    write (log_file,'(a6,a3,L1)')'F25Tag',' : ' ,F25Tag

    I = iargc()

    if(I < 1 .or. I > 16) then
       write (log_file,'(a)') "Wrong Number of arguments: ", i
       do j = 1,size(usage)
          print "(sp,a100)", Usage(j)
       end do
       call exit(1)
    end if

    nxt = 1
    call getarg(nxt,arg)
    do while(arg(1:1)=='-')
       opt=arg(2:2)
       if(len(trim(arg))==2) then
          if(scan(opt,'zvh')==0) then
             nxt = nxt + 1
             call getarg(nxt,arg)
          endif
       else
          arg = arg(3:)
       end if
       select case (opt)
       case ('x')
          read(arg,'(i6)') nc
       case ('y')
          read(arg,'(i6)') nr
       case ('g')
          GridName = trim(arg)
       case ('l')
          LD = trim(arg)
       case ('s')
          SD = trim(arg)
       case ('b')
          DL = trim(arg)
       case ('m')
          MA = trim(arg)
       case ('e')
          EASE = trim(arg)
	  if(EASE=='EASE') ease_grid=.true.
       case default
          do j = 1,size(usage)
             print "(sp,a100)", Usage(j)
          end do
          call exit(1)
       end select
       nxt = nxt + 1
       call getarg(nxt,arg)
    end do

    if(F25Tag) then
!
! Going back to f2.5 tag
!  
      LD ='GSWP2'
      SD ='NGDC'
      MA ='MODIS1'

    endif

    write (log_file,'(a)')trim(LD)
    write (log_file,'(a)')trim(SD)
    write (log_file,'(a)')trim(MA)

    if(trim(Gridname) == '') then
       write (log_file,'(a)')'Unable to create parameters without til/rst files.... !'
       stop
    endif

    regrid = nc/=i_raster .or. nr/=j_raster

    if(index(Gridname,'Pfaf.notiny')/=0) then 
      GridnameR='clsm/'//trim(Gridname)
      GridnameT='clsm/'//trim(Gridname)
      else
      GridnameR='rst/'//trim(Gridname)  
      GridnameT='til/'//trim(Gridname)  
    endif 

    write (log_file,'(a)')'                              '
    write (log_file,'(a)')'>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>'
    write (log_file,'(a)')'Begin CLSM parameter generation............................:'    
    write (log_file,'(a)')'                                               '
    write (log_file,'(a)')'CLSM parameters are being generated for the tile space :'
    write (log_file,'(a)')'     ',trim(GridnameT)
    write (log_file,'(a)')'                                               '
    write (log_file,'(a)')'>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>'
    write (log_file,'(a)')'                                               '

    if(index(Gridname,'CF')/=0) then 
       DL = 'DE'
       write (log_file,'(a)')'Cube Grid - assuming DE'
    endif

    inquire(file='clsm/cti_stats.dat', exist=file_exists)
    if (.not.file_exists) call cti_stat_file (ease_grid,c_data,gridnamet)
    write (log_file,'(a)')'Done creating CTI stat file ...............................1'	 

    inquire(file='clsm/mosaic_veg_typs_fracs', exist=file_exists)      
    if (.not.file_exists) call compute_mosaic_veg_types (nc,nr,ease_grid,regrid,c_data,gridnamet,gridnamer)
    write (log_file,'(a)')'Done creating vegetation types file .......................2'
    
    if(SD=='NGDC') call create_soil_types_files (nc,nr,ease_grid,c_data,gridnamet,gridnamer)    
    if(SD=='NGDC') write (log_file,'(a)')'Done creating NGDC soil types file ........................3'	   
    
    if(.not.ease_grid) then  
       inquire(file='clsm/catchment.def', exist=file_exists)
       if (.not.file_exists) call catchment_def (nc,nr,regrid,dl,gridnamet,gridnamer, gtopo30) 
       write (log_file,'(a)')'Done creating catchment.def file ..........................4'
    endif

    ! Creating soil_param.first and tau_param.dat files that has 2 options: 
    !  1) NGDC soil properties, 2) HWSD-STATSGO2 Soil Properties

    inquire(file='clsm/soil_param.first', exist=file_exists)

    if (.not.file_exists) then
       if(SD=='NGDC')  then 
          if(F25Tag) call soil_para_high (nc,nr,regrid,gridnamer,F25Tag=F25Tag)
          if(.not.F25Tag) call soil_para_high (nc,nr,regrid,gridnamer)
       endif

       if(SD=='HWSD')  call soil_para_hwsd (nc,nr,gridnamer,c_data) 
    endif
    write (log_file,'(a,a,a)')'Done computing ',trim(SD),' soil parameters .......................5'
    
    ! Processing Vegetation Climatology 

    inquire(file='clsm/green.dat', exist=file_exists)
    if (.not.file_exists) call process_gswp2_veg (nc,nr,regrid,'grnFrac',gridnamer,c_data)

    inquire(file='clsm/lai.dat', exist=file_exists)
    if (.not.file_exists) then
       redo_modis = .true.

       if (trim(LD) == 'GSWP2')    call process_gswp2_veg (nc,nr,regrid,'LAI',gridnamer,c_data) 
!!       if (trim(LD) == 'MODIS')    call modis_lai (nc,nr,regrid,gridnamer)

       if (trim(LD) == 'MODIS') then
          lai_name = 'MODIS_8-DayClim/MODIS_'
          call process_gswp2_veg (nc,nr,regrid,'LAI',gridnamer,c_data,merge=1) 
          call hres_lai (nc,nr,gridnamer,c_data,lai_name) 
       endif

       if (trim(LD) == 'GEOLAND2') then
          lai_name = 'GEOLAND2_10-DayClim/geoland2_'
          call process_gswp2_veg (nc,nr,regrid,'LAI',gridnamer,c_data,merge=1) 
          call hres_lai (nc,nr,gridnamer,c_data,lai_name) 
       endif

       if (trim(LD) == 'MODGEO') then
          inquire(file='clsm/lai.dat.gswp2', exist=file_exists)
          if (.not.file_exists)call process_gswp2_veg (nc,nr,regrid,'LAI',gridnamer,c_data,merge=1)     
          lai_name = 'MODIS_8-DayClim/MODIS_'
          inquire(file='clsm/lai.MODIS_8-DayClim', exist=file_exists)
          if (.not.file_exists)call hres_lai (nc,nr,gridnamer,c_data,lai_name,merge=1)     
          lai_name = 'GEOLAND2_10-DayClim/geoland2_'
          inquire(file='clsm/lai.GEOLAND2_10-DayClim', exist=file_exists)
          if (.not.file_exists)call hres_lai (nc,nr,gridnamer,c_data,lai_name,merge=1)     
          call merge_lai_data
        endif
    endif
    write (log_file,'(a,a,a)')'Done computing ', trim(LD),' vegetation climatologies .............6'
    
    ! call modis_alb_on_tiles (nc,nr,ease_grid,regrid,gridnamet,gridnamer,c_data)
    ! call modis_scale_para (ease_grid,gridnamet)
    ! NOTE: modis_alb_on_tiles uses monthly climatological raster data on 8640x4320 to produce 
    ! MODIS albedo on tile space. The subroutine was replaced with "modis_alb_on_tiles_high" that process
    ! MODIS data on native 21600x10800 grid and produces 16-day MODIS Albedo climatology

    if(MA == 'MODIS1') then 
        inquire(file='clsm/AlbMap.WS.16-day.tile.0.7_5.0.dat', exist=file_exists)
        if (.not.file_exists) then
           if(.not.F25Tag) call modis_alb_on_tiles_high (nc,nr,MA,gridnamer,c_data)
           if(F25Tag) call modis_alb_on_tiles (nc,nr,ease_grid,regrid,gridnamet,gridnamer,c_data)
        endif
    endif

    if(MA == 'MODIS2') then 
        inquire(file='clsm/AlbMap.WS.8-day.tile.0.7_5.0.dat', exist=file_exists)
        if (.not.file_exists) call modis_alb_on_tiles_high (nc,nr,MA,gridnamer,c_data)
    endif
    write (log_file,'(a,a,a)')'Done putting ',trim(MA), ' Albedo on the tile space  .............7'

    inquire(file='clsm/visdf.dat', exist=file_exists)
    if ((redo_modis).or.(.not.file_exists)) then
         if(.not.F25Tag) then
            call modis_scale_para_high (ease_grid,MA,gridnamet)
         else
            inquire(file='clsm/modis_scale_factor.albvf.clim', exist=file_exists)
            if ((redo_modis).or.(.not.file_exists)) then
               call modis_scale_para (ease_grid,gridnamet)
               call REFORMAT_VEGFILES
            endif
         endif
    endif

    write (log_file,'(a,a,a)')'Done computing ',trim(MA), ' scale factors .......................8'

    inquire(file='clsm/ts.dat', exist=file_exists)
    if (.not.file_exists) then
       if(SD=='NGDC') call create_model_para (c_data)
       if(SD=='HWSD') call create_model_para_woesten (c_data) 
    endif
    write (log_file,'(a,a,a)')'Done computing CLSM model parameters based on ',trim(SD),'.........9'
        
   ! Commented out this call because 7.5-minute raster file is only used
   ! for plotting purposes
   !  call make_75 (nc,nr,regrid,c_data,gridnamer)
   !    write (log_file,'(a)')'Done creating 7.5 minute raster file ......................'
   !    write (log_file,'(a)')'Not created 7.5 minute raster file   ......................'


    write (log_file,'(a)')'============================================================'
    write (log_file,'(a)')'DONE creating CLSM data files...............................'
    write (log_file,'(a)')'============================================================'

close (log_file,status='keep')    

END PROGRAM mkCatchParam
