PROGRAM mkLISTilesPara
  
  implicit none

  character(40) :: arg
  integer ::  i, N_args, iargc
  character*300 :: latlon_vector_file
  integer, parameter :: nc=8640,nr=4320
  character*100 :: gfile

  N_args = iargc()

  if(N_args /= 2) then
     print *,'USAGE : bin/mkLISTilesPara -vfile filename'
     stop
  end if

  i=0      
  
  do while ( i < N_args )
     
     i = i+1
     
     call getarg(i,arg)
     
     if     ( trim(arg) == '-vfile' ) then
        i = i+1
        call getarg(i,latlon_vector_file)
	
     else ! stop for any other arguments

        print *,'USAGE : bin/mkLISTilesPara -vfile filename'
        stop        

     endif     
end do

call system('mkdir -p data/ ; mkdir -p til/ ; mkdir -p rst/ ; mkdir -p clsm/')
call system('cd data/ ; ln -s /discover/nobackup/rreichle/l_data/LandSurfaceData_for_Catchment_Parameters/ CATCH')  
call system('cd ..')

call create_files (nc,nr,gfile,latlon_vector_file)

call system('bin/mkCatchParam')

print *,'                           '
print *,'-----------------------------------------------------------------------------'
print *,'                           '

print *,'Land BCS pre-processor input files have been created for the LIS vector file:'
print *, trim(latlon_vector_file)
print *,'ENTER: bin/mkCatchParam -e EASE -g ',trim(gfile)
print *,' ---along with any option from the above list for your application.'
print *,'                           '
print *,'-----------------------------------------------------------------------------'
print *,'                           '

END PROGRAM mkLISTilesPara

!
! --------------------------------------------------------------------------------------------
!

SUBROUTINE create_files (nc,nr,gfile,filename)

  implicit none
  integer, parameter :: nc_gswp2 = 360, nr_gswp2 = 180, n_gswp2 =15238 
  real, allocatable, dimension (:,:) :: &
     cti_mean, cti_std, cti_min, cti_max, cti_skew
  character (*), intent (in) :: filename
  character*100, intent (out) :: gfile
  integer, intent(in) :: nc,nr
  integer :: ncells,status,i,j,k,ix1,ix2,iy1,iy2,im,jm
  real :: dx, dy, dxh, dyh,d2r,lats
  integer :: n,nc_domain,nr_domain,i_offset,j_offset, nc_global,nr_global
  character*9 :: glabel1,glabel2 
  character*100 ::  gtopo30,char_string
  real :: x0, y0
  real :: lat,lon,da, x1,x2,x3,x4,x5, ox1,ox2,ox3,ox4,ox5
  real, parameter :: PI     = 3.14159265358979323846
  real, parameter :: RADIUS = 6376.0E3
  integer, dimension(:,:), allocatable :: tileid
  real*4, dimension (:,:), allocatable ::    q0
  real, dimension (:), allocatable :: tile_ele,tile_area_land
  integer,allocatable :: catid(:,:)

  allocate(catid       (1:nc,1:nr))
  nc_domain = 0
  nr_domain = 0
  i_offset  = 0
  j_offset  = 0
 
  open (10,file='data/CATCH/global.cat_id.catch.DL', form='formatted', &
       action='read', status='old')!
  
  do j=1,nr
     read(10,*)(catid(i,j),i=1,nc)
  end do

  print *,'CATID MIN/MAX : ',minval(catid),maxval(catid)

  close (10,status='keep') 
 !
 ! opening LIS latlon vector file
 !

  open (10,file= trim(filename), form = 'formatted',action= 'read', status = 'old')
  
  read (10,'(a)') char_string

  ! Reading #of grid cells in the domain or the global file, dx, dy, ,nc_domain,nr_domain,i_offset,j_offset 
  ! nc_domain,nr_domain,i_offset,j_offset are optional, they are only used when working on a rectangular subset of the globe 
  !(for e.g. NLDAS), where nc_domain = # of columns in the domain, nr_domain  = # of rows in the domain, 
  ! i_offset = i index of LL grid cell of the domain minus 1, when the domain grid is overlayed on the similar global grid, 
  !   counting columns from the dateline to the dateline, 
  ! j_offset is the same counting rows from the south pole to the north pole. Note that the global grid has the 
  ! dateline and the south pole as the western and sothern edges 

  read (char_string,*,IOSTAT=status) ncells, dx,dy,nc_domain,nr_domain,i_offset,j_offset
!  print *,ncells, dx,dy,nc_domain,nr_domain,i_offset,j_offset
 
  da = dx*dy*radius*radius*pi*pi/180./180./1000000. 
 ! For MERRA, GEOS5 type
 ! x0 = -180.! + dx/2.
 ! y0 = -90. + 0.25 + dy/2.
 ! For GSWP-2, NLDAS
   x0 = -180. + dx/2.
   y0 = -90.  + dy/2.
  dxh= 360._8/nc
  dyh= 180._8/nr
  d2r= PI/180._8

  nc_global = nint(360./dx)
  nr_global = nint(180./dy)

  if (nc_domain == 0) nc_domain = nc_global
  if (nr_domain == 0) nr_domain = nr_global

  write(glabel1,'(i4.4,1a,i4.4)')nc_global,'x',nr_global
  write(glabel2,'(i4.4,1a,i4.4)')nc_domain,'x',nr_domain

  gfile = 'PE_'//glabel1//'_DE_'//glabel2
!
! reading in other input data files
!
  
  allocate (cti_mean(1:nc_gswp2,1:nr_gswp2))
  allocate (cti_std (1:nc_gswp2,1:nr_gswp2))
  allocate (cti_min (1:nc_gswp2,1:nr_gswp2))
  allocate (cti_max (1:nc_gswp2,1:nr_gswp2))
  allocate (cti_skew(1:nc_gswp2,1:nr_gswp2))
  allocate (q0      (1:nc      ,1:nr      ))
  allocate (tile_ele(1:ncells))
  allocate (tile_area_land (1:ncells))

  cti_mean = 0.
  cti_std  = 0.
  cti_min  = 0.
  cti_max  = 0.
  cti_skew = 0.
  q0       = 0.
  tile_ele = 0.
  tile_area_land = 0.
!
! opening and writing  .til and .rst cti_stat and catchment.def files
!
  open (11,file='/discover/nobackup/rreichle/l_data/geos5/bcs/SiB2_V2/DE/GSWP2_1by1/FV_360x180_DE_360x180_DE.til', &
       form='formatted',action='READ',status='OLD')
  open (12,file='/discover/nobackup/rreichle/l_data/geos5/bcs/SiB2_V2/DE/GSWP2_1by1/cti_stats.dat',                &
       form='formatted',action='READ',status='OLD')

  read (11,*) i
  read (11,*) i
  read (11,'(a)') 
  read (11,*) i
  read (11,*) i
  read (11,'(a)') 
  read (11,*) i
  read (11,*) i
  read (12,*) i
  
   do n =1,n_gswp2

     read(11,'(i10,i9,2f10.4,2i5,f16.12,3i8,f16.12,i8,f13.4)')k,k,lon,lat,i,j 
     read(12,'(i8,i8,5(1x,f8.4))')k,k, x1,x2,x3,x4,x5
     cti_mean(i,j) = x1
     cti_std (i,j) = x2
     cti_min (i,j) = x3
     cti_max (i,j) = x4
     cti_skew(i,j) = x5

  end do

  close(11,status='keep')
  close(12,status='keep')

  gtopo30 = 'data/CATCH/srtm30_withKMS_2.5x2.5min.data'
  open (13,file=trim(gtopo30),form='unformatted',status='old',convert='little_endian')
  read (13) q0
  close(13,status='keep')
  
  open  (20,file ='til/'//trim(gfile)//'.til',form = 'formatted'  ,action= 'write', status = 'unknown') 
  open  (21,file ='rst/'//trim(gfile)//'.rst',form = 'unformatted',action= 'write', status = 'unknown')
  open  (22,file ='clsm/cti_stats.dat',       form = 'formatted'  ,action= 'write', status = 'unknown')
  open  (23,file ='clsm/catchment.def',       form = 'formatted'  ,action= 'write', status = 'unknown')

  allocate (tileid  (1:nc      ,1:nr      ))

  tileid   = 0

  write (20,*) ncells
  write (20,*) 2
  write (20,'(a9)') glabel1
  write (20,*) nc_global
  write (20,*) nr_global
  write (20,'(a9)') glabel2
  write (20,*) nc_domain
  write (20,*) nr_domain

  write(22,*) ncells

  do n = 1, ncells 

     read (10,*) k,lat,lon
     i = nint((lon - x0)/dx) + 1
     j = nint((lat - y0)/dy) + 1

     ix2 = i*(nc/nc_global)
     ix1 = ix2 - (nc/nc_global -1) 
     
     iy2 = j*(nr/nr_global)
     iy1 = iy2 - (nr/nr_global -1) 

     im = (ix1 + ix2)/2
     jm = (iy1 + iy2)/2

     tileid (ix1:ix2,iy1:iy2) = n

     x1 =cti_mean(ceiling(real(i)/(real(nc_global)/360.)),ceiling(real(j)/(real(nr_global)/180.))) 
     x2 =cti_std (ceiling(real(i)/(real(nc_global)/360.)),ceiling(real(j)/(real(nr_global)/180.)))
     x3 =cti_min (ceiling(real(i)/(real(nc_global)/360.)),ceiling(real(j)/(real(nr_global)/180.)))
     x4 =cti_max (ceiling(real(i)/(real(nc_global)/360.)),ceiling(real(j)/(real(nr_global)/180.)))
     x5 =cti_skew(ceiling(real(i)/(real(nc_global)/360.)),ceiling(real(j)/(real(nr_global)/180.)))
     
     if((n == 1).and.(x1==0.)) then
        x1 = 12.5830
        x2 = 2.6039 
        x3 = 7.8878 
        x4 = 23.1378 
        x5 = 0.8388    
     endif

     if(x1.gt.0.) then
        ox1 = x1 
        ox2 = x2 
        ox3 = x3 
        ox4 = x4 
        ox5 = x5 
     endif

     if (nr_global < 1000) then
        write(20,'(i10,i9,2f10.4,2i5,f5.2,i10,f13.4,i8)') 100,catid(im,jm),lon,lat,i-i_offset,j-j_offset,1.,n,da*cos(lat*pi/180.)
        if(x1.gt.0.) then 
           write(22,'(i8,i8,5(1x,f8.4))')n,i*1000+j, x1,x2,x3,x4,x5
        else
           write(22,'(i8,i8,5(1x,f8.4))')n,i*1000+j, ox1,ox2,ox3,ox4,ox5 
        endif        
     else
        write(20,'(i10,i9,2f10.4,2i5,f5.2,i10,f13.4,i8)') 100,catid(im,jm),lon,lat,i-i_offset,j-j_offset,1.,n,da*cos(lat*pi/180.) 
        if(x1.gt.0.) then 
           write(22,'(i8,i8,5(1x,f8.4))')n,catid(im,jm), x1,x2,x3,x4,x5
        else
           write(22,'(i8,i8,5(1x,f8.4))')n,catid(im,jm), ox1,ox2,ox3,ox4,ox5 
        endif          
     endif
  end do

  do j = 1,nr
     write(21) tileid(:,j)
     lats = -90._8 + (j - 0.5_8)*dyh

     do i = 1,nc
        if((tileid(i,j) > 0).and.(tileid(i,j) <= ncells))then
           tile_ele(tileid(i,j)) = tile_ele(tileid(i,j)) + q0(i,j)*   &	
                  (sin(d2r*(lats+0.5*dyh)) -sin(d2r*(lats-0.5*dyh)))*(dxh*d2r)
           tile_area_land(tileid(i,j)) = tile_area_land(tileid(i,j)) +  &	
                  (sin(d2r*(lats+0.5*dyh)) -sin(d2r*(lats-0.5*dyh)))*(dxh*d2r)
        endif
     enddo
  end do

    tile_ele = tile_ele/tile_area_land

    ! adjustment Global Mean Topography to 614.649 (615.662 GTOPO 30) m
    ! ---------------------------
    tile_ele =tile_ele*(614.649D0 /656.7712) 

    close (10,status = 'keep')
    close (20,status = 'keep')
    open  (20,file ='til/'//trim(gfile)//'.til',form = 'formatted'  ,action= 'read', status = 'old') 
    read (20,*) i
    read (20,*) i
    read (20,'(a)') 
    read (20,*) i
    read (20,*) i
    read (20,'(a)') 
    read (20,*) i
    read (20,*) i

 
    write(23,*) ncells

   do n =1,ncells
     read (20,'(i10,i9,2f10.4,2i5,f16.12,3i8,f16.12,i8,f13.4)')k,ix1,lon,lat,i,j 
     write(23,'(i8,i8,5(2x,f9.4))')n,ix1,lon-dx/2.,lon+dx/2.,lat-dy/2.,lat+dy/2.,tile_ele(n)
   end do

   close (20,status = 'keep')
   close (21,status = 'keep')
   close (22,status = 'keep')
   close (23,status = 'keep')

END SUBROUTINE create_files
