!   $Id: mkLandRaster.F90,v 1.3.46.2.24.1 2012-04-19 14:16:41 smahanam Exp $

#include "Raster.h"

  Program MakeLandRaster

    use LogRectRasterizeMod
    use MAPL_HashMod

! Program to create a surface raster file at 2.5' that has
! the ocean divided with a regular lat-lon DE grid. Its inputs
! are Sarith's formatted 2.5' raster of the Pfafstetter catchments with 
! ocean points set to 0. Output is an unformatted integer*4 2.5" raster
! with the ocean points set to negative numbers.

    implicit none


    integer, parameter     :: InUNIT   = 20
    integer, parameter     :: nx0 = 8640
    integer, parameter     :: ny0 = 4320

    integer                :: IARGC
    integer                :: i,j,k,status
    integer                :: ip, nxt
    integer                :: type, maxtiles, nx, ny
    integer                :: count0,count1,count_rate

    REAL_                  :: dx, dy, d2r   ! Grid spacing of raster grid
    REAL_                  :: xmin, ymin, xmax, ymax, xs, ys, da

    REAL_,     allocatable :: cc(:), ss(:)
    REAL_ ,    allocatable :: rTable(:,:)

    integer,       pointer :: Raster(:,:)
    integer,   allocatable, target :: Raster0(:,:)
    integer,   allocatable :: iTable(:,:)

    integer :: Pix
    integer :: CreateHash, IncrementHash, Hash

    logical                :: DoZip
    logical                :: Verb
    logical                :: regrid

    REAL_                  :: VV(4)
    REAL_                  :: PI=RASTER_PI
    character*1            :: Opt
    character*128          :: arg
    character*128          :: TilFile, RstFile
    character*128          :: Tildir,  Rstdir
    character*128          :: GridName
    character*128          :: InputFile
    character*128          :: &
      Usage = "mkLandRaster -x nx -y ny -v -h -z -t maxtiles -l LandFile -g GridName"
   call system('cd data/ ; ln -s /discover/nobackup/rreichle/l_data/LandSurfaceData_for_Catchment_Parameters/ CATCH')
   call system('cd ..')

! Process Arguments
!------------------

    nx        = nx0
    ny        = ny0
    GridName  = "Pfafstetter"
    DoZip     = .false.  ! No zipping
    Verb      = .false.  ! Run quiet
    tildir    = 'til/'   ! Write in current dir
    rstdir    = 'rst/'   ! Write in current dir
    maxtiles  = 50000
    InputFile = &
          "data/CATCH/global.cat_id.catch.DL"
    I = iargc()

    if(I > 13) then
       print *, "Wrong Number of arguments: ", i
       print *, trim(Usage)
       call exit(1)
    end if

    nxt = 1

    do while(nxt<=I)
       call getarg(nxt,arg)
       if(arg(1:1)/='-') then
          print *, trim(Usage)
          call exit(1)
       end if
       opt=arg(2:2)
       if(len(trim(arg))==2) then
          if(scan(opt,'zvh')==0) then
             nxt = nxt + 1
             call getarg(nxt,arg)
          end if
       else
          arg = arg(3:)
       end if
       select case (opt)
       case ('x')
          read(arg,'(i6)') nx
       case ('y')
          read(arg,'(i6)') ny
       case ('l')
          InputFile = arg
       case ('g')
          Gridname = arg
       case ('v')
          Verb = .true.
       case ('z')
          DoZip = .true.
       case ('h')
          tildir = ''
          rstdir = ''
       case ('t')
          read(arg,*) maxtiles
       case default
          print *, trim(Usage)
          call exit(1)
       end select

       nxt = nxt + 1
    end do

    call system_clock(count0,count_rate)

    if(DoZip) then
       TilFile = trim(tildir)//trim(Gridname)//'.til.gz'
       RstFile = trim(rstdir)//trim(Gridname)//'.rst.gz'
    else
       TilFile = trim(tildir)//trim(Gridname)//'.til'
       RstFile = trim(rstdir)//trim(Gridname)//'.rst'
    end if

    allocate(raster0(nx0,ny0),stat=STATUS); VERIFY_(STATUS)

    regrid = nx/=nx0 .or. ny/=ny0

    if(regrid) then
       allocate(raster(nx,ny),stat=STATUS); VERIFY_(STATUS)
    else
       raster => raster0
    end if

    allocate(cc(nx),       stat=STATUS); VERIFY_(STATUS)
    allocate(ss(nx),       stat=STATUS); VERIFY_(STATUS)

    allocate(iTable(0:3,maxtiles),stat=status)
    VERIFY_(STATUS)
    allocate(rTable(1:4,maxtiles),stat=status)
    VERIFY_(STATUS)

! Compute ys and xs at centers of raster cell

    xmin = -180.0_8
    xmax =  180.0_8
    ymin =  -90.0_8
    ymax =   90.0_8

    dx    = (xmax-xmin)/nx
    dy    = (ymax-ymin)/ny
    d2r   = (2._8*PI)/360.0_8
 
    do i = 1,nx
       xs = (xmin + dx*(i-0.5_8))*d2r
       cc(i) = cos(xs)
       ss(i) = sin(xs)
    enddo

! Input file: (formatted for now)

    open (InUnit,file=InputFile, form='formatted',status='old')

    if(Verb) then
       call system_clock(count1)
       print *, 'Opened land file. Time = ', (count1-count0)/float(count_rate)
       count0 = count1
    end if

    do j=1,ny0
       read (InUnit,*) raster0(:,j)
    enddo

!    call ReadRaster(InputFile,raster) ! reead unformatted

    if(Verb) then
       call system_clock(count1)
       print *, 'Read land file. Time = ', (count1-count0)/float(count_rate)
       count0 = count1
    end if


    if(regrid) then
       call RegridRaster(Raster0,raster)
    endif

! Loop over ocean latitudes

     if(Verb) then
        write (6, '(A)', advance='NO') 'Processing catchment data'
     end if

    ip = 0
    Hash  = CreateHash(4*1024)

    LATITUDES: do j=1,ny

       where(raster(:,j)>=6190000)
          raster(:,j) = (raster(:,j)/10000)*10000
       end where

       ys  = ymin + dy*(j-0.5_8)
       da  = (sin(d2r*(ys+0.5*dy)) -   &
              sin(d2r*(ys-0.5*dy)) )*(dx*d2r)

       vv(3) = da*ys
       vv(4) = da

       LONGITUDES: do i=1,nx

          vv(1) = ss(i)*da
          vv(2) = cc(i)*da

          Pix = raster(i,j)

          k = MAPL_HashIncrement(Hash,Pix);

          raster(i,j) = k

          if(k<=ip) then    ! Bump the counter and the lon, lat, and area sums
             iTable(1,k) = iTable(1,k) + 1
             rTable(1,k) = rTable(1,k) + vv(1)
             rTable(2,k) = rTable(2,k) + vv(2)
             rTable(3,k) = rTable(3,k) + vv(3)
             rTable(4,k) = rTable(4,k) + vv(4)
          else              ! We have a new tile in the exchange grid
             ip = ip + 1
             if(ip>maxtiles) then
                print *, "Exceeded maxtiles = ", maxtiles, &
                         ". Use -t option to increase."
                call exit(1)
             end if
             
             select case (Pix)
             case (0)
                type = 0
             case(6190000)
                type = 19
             case(6200000)
                type = 20
             case default
                type = 100
             end select

             iTable(0 ,ip) = type
             iTable(1 ,ip) = 1
             rTable(:4,ip) = vv

             iTable( 2,ip) = Pix
             iTable( 3,ip) = 1
          end if

     end do LONGITUDES

     if(Verb) then
        if(mod(j,200)==0) then
           write (6, '(A)', advance='NO') '.'
        end if
     end if

  end do LATITUDES

  close(InUnit)
  
  if(Verb) then
     call system_clock(count1)
     print *
     print *, "Found ",ip," unique tiles."
     print *, "Tiling Time = ",(count1-count0)/float(count_rate)
     count0 = count1
     type = sum(iTable(1,:ip))
     print *, type, nx*ny, nx*ny-type
  end if

  call DestroyHash(Hash)

! Compute proper longitude and latitude in degrees and compress
! the real table for WriteTiling.

  if(Verb) print *, "Computing weighted lons and lats..."
  
  do k=1,ip
     rTable(1,k) = atan2(rTable(1,k),rTable(2,k))/d2r
     rTable(2,k) =       rTable(3,k)/rTable(4,k)     
     rTable(3,k) = rTable(4,k)
  end do

! Sort table by the first grid type in descending order.

  if(Verb) print *, "Sorting..."
  call SortTiling(Raster,rTable(:,:ip),iTable(:,:ip))

  if(Verb) then
     call system_clock(count1)
     print *,  "Done Sorting. Time = ", (count1-count0)/float(count_rate)
     count0 = count1
  end if

  do k=1,ip
     iTable( 3,k) = k
  end do

  if(Verb) print *, 'Writing til file...'
  call WriteTiling(TilFile, (/Gridname/), (/ip/), (/1/), (/ip/),      &
                   nx, ny, iTable(:,:ip), rTable(:4,:ip), Dozip, Verb )

  if(Verb) print *, 'Writing raster file...'
  call WriteRaster( RstFile, Raster, DoZip)
  
  if(Verb) then
     call system_clock(count1)
     print *,  "Done Writing. Time = ", (count1-count0)/float(count_rate)
     count0 = count1
  end if

! All done

  if(Verb) print * , 'Terminated Normally'
  call exit(0)

contains

subroutine RegridRaster(Rin,Rout)

  integer, intent(IN)  :: Rin(:,:)
  integer, intent(OUT) :: Rout(:,:)

  REAL_  :: xx, yy
  integer :: i,j,ii,jj

  xx = size(Rin ,1)/float(size(Rout,1))
  yy = size(Rin ,2)/float(size(Rout,2))

  do j=1,size(Rout,2)
     jj = (j-1)*yy + 1
     do i=1,size(Rout,1)
        ii = (i-1)*xx + 1
        Rout(i,j) = Rin(ii,jj)
     end do
  end do

end subroutine RegridRaster


end Program MakeLandRaster

