#include "Raster.h"

!
! A Collection subroutine that helps process MODIS Albedo, GEOLAND2 LAI, and
! NGDC-HWSD-STATSGO merged soil data on their native grids 3-23-2012
!   Contact: Sarith Mahanama  sarith.p.mahanama@nasa.gov
!   Email  : sarith.p.mahanama@nasa.gov

MODULE process_hres_data
use rmTinyCatchParaMod
use MAPL_SortMod
use date_time_util  
use leap_year

implicit none

include 'netcdf.inc'	

private

public :: soil_para_hwsd,hres_lai, merge_lai_data
public :: modis_alb_on_tiles_high,modis_scale_para_high

contains

!

!
!----------------------------------------------------------------------
!
  SUBROUTINE merge_lai_data 
    implicit none
    type (date_time_type) :: bf_geol2_time,af_geol2_time,date_time_new,bf_lai_time,   &
       af_lai_time
    integer :: n,k,ntiles,t,ierr
    integer, allocatable, dimension (:) :: pfaf
    ! South AMerica is from GEOLAND2
    integer, parameter :: i1 = 1011000, i2 =  1999900
    REAL, ALLOCATABLE, dimension (:) :: geol2_lai_bf,geol2_lai_af,geol2_lai, lai    
    real :: dum, gyr,gmn,gdy,gyr1,gmn1,gdy1, slice1,slice2
    real :: yr,mn,dy,yr1,mn1,dy1

    open (10, file ='clsm/catchment.def',form='formatted',status='old',action='read')
    read (10,*) ntiles

    allocate (pfaf(1:ntiles))
    allocate (geol2_lai_bf(1:ntiles))
    allocate (geol2_lai_af(1:ntiles))
    allocate (geol2_lai   (1:ntiles))
    allocate (lai         (1:ntiles))  
           
    do n =1,ntiles
       read (10,*) k,pfaf(n)
    end do

    close (10,status='keep')

!
        open (41,file='clsm/lai.GEOLAND2_10-DayClim',  &
         form='unformatted',status='old',convert='little_endian',action='read')
        open (42,file='clsm/lai.MODIS_8-DayClim',      &
         form='unformatted',status='old',convert='little_endian',action='read')
        open (43,file='clsm/lai.dat',      &
         form='unformatted',status='unknown',convert='little_endian',action='write')

       read(41) gyr,gmn,gdy,dum,dum,dum,gyr1,gmn1,gdy1       
       read(41) geol2_lai_bf
       call Get_MidTime(gyr,gmn,gdy,gyr1,gmn1,gdy1,bf_geol2_time)

       read(41) gyr,gmn,gdy,dum,dum,dum,gyr1,gmn1,gdy1
       read(41) geol2_lai_af
       call Get_MidTime(gyr,gmn,gdy,gyr1,gmn1,gdy1,af_geol2_time)

       do t = 1, 48

          read(42) yr,mn,dy,dum,dum,dum,yr1,mn1,dy1 
          read(42) lai 
          write(43) yr,mn,dy,0.,0.,0.,yr1,mn1,dy1,0.,0.,0.,float(ntiles),1.
	  call Get_MidTime(yr,mn,dy,yr1,mn1,dy1,date_time_new)

!          date_time_new%year   = nint(yr) + 2001 
!          date_time_new%month  = nint(mn)
!          date_time_new%day    = nint(dy)
!          date_time_new%hour   = 0            
!          date_time_new%min    = 0            
!          date_time_new%sec    = 0 
!          call get_dofyr_pentad(date_time_new)   
          if (datetime_le_refdatetime(date_time_new,af_geol2_time)) then
           
          else
             read(41,IOSTAT=ierr) gyr,gmn,gdy,dum,dum,dum,gyr1,gmn1,gdy1
             if(ierr == 0) then
                geol2_lai_bf = geol2_lai_af
                read(41) geol2_lai_af
                bf_geol2_time = af_geol2_time
                call Get_MidTime(gyr,gmn,gdy,gyr1,gmn1,gdy1,af_geol2_time)
             else
                print *,'END OF GEOL2 LAI FILE'
                stop
             endif
          endif

          if(t==1) then 
             date_time_new%year = date_time_new%year + 1
             geol2_lai_af       = geol2_lai_bf
             af_geol2_time      = bf_geol2_time
             af_geol2_time%year = af_geol2_time%year + 1

             do k = 1,34 
                read(41) gyr,gmn,gdy,dum,dum,dum,gyr1,gmn1,gdy1       
                read(41) geol2_lai_bf
                call Get_MidTime(gyr,gmn,gdy,gyr1,gmn1,gdy1,bf_geol2_time)                
             end do
          endif

!          print *,t
!          print *,'DATE_TIME_NEW :',date_time_new
!          print *,'bf_geol2_time :',bf_geol2_time
!          print *,'af_geol2_time :',af_geol2_time

          call Time_Interp_Fac (date_time_new, bf_geol2_time, af_geol2_time, slice1, slice2)
          geol2_lai    = (slice1*geol2_lai_bf + slice2*geol2_lai_af)
          
          if(t == 1) then
             rewind(41)
             read(41) gyr,gmn,gdy,dum,dum,dum,gyr1,gmn1,gdy1       
             read(41) geol2_lai_bf
             call Get_MidTime(gyr,gmn,gdy,gyr1,gmn1,gdy1,bf_geol2_time)
             
             read(41) gyr,gmn,gdy,dum,dum,dum,gyr1,gmn1,gdy1
             read(41) geol2_lai_af
             call Get_MidTime(gyr,gmn,gdy,gyr1,gmn1,gdy1,af_geol2_time)
          endif

! replace South America with GEOLAND2

          DO n =1,ntiles             
             if((pfaf(n) >= i1).and.(pfaf(n) <= i2)) lai(n) = geol2_lai(n)
          end do
          write (43) lai(:)
       end do

       close (41,status = 'keep')
       close (42,status = 'keep')
       close (43,status = 'keep')

       deallocate (pfaf,geol2_lai_bf, geol2_lai_af,geol2_lai,lai)

  END SUBROUTINE merge_lai_data
!
!----------------------------------------------------------------------
!
  SUBROUTINE modis_scale_para_high (ease_grid,MA,gfile)
    
    implicit none
    type (date_time_type) :: gf_green_time,af_green_time,end_time, &
              bf_lai_time,af_lai_time,date_time_new,bf_modis_time,   &
              af_modis_time
    logical :: ease_grid
    character*6 :: MA
    CHARACTER*20 :: version,resoln,continent
    integer :: nc_gcm,nr_gcm,nc_ocean,nr_ocean
    REAL :: latt,lont,fr_gcm,fr_cat,tsteps,zth, slr,tarea
    INTEGER :: typ,pfs,ig,jg,j_dum,ierr,indx_dum,indr1,indr2,indr3 ,ip2
    character*100 :: path,fname,fout,metpath
    character (*) :: gfile
    integer :: n,maxcat,ip
    integer :: yy,j,month
    integer, allocatable, dimension (:) :: vegcls 
    real, allocatable, dimension (:) :: &
         modisvf, modisnf,albvr,albnr,albvf,albnf,lat,lon, &
         green,lai,snw,lai_before,lai_after,grn_before,grn_after
    real, allocatable, dimension (:) :: &
         calbvr,calbnr,calbvf,calbnf
    character*300 :: ifile1,ifile2,ofile
    integer, dimension(12), parameter :: days_in_month_nonleap = &
         (/ 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 /)
    integer :: day, hour, min ,secs_in_day,k
    real :: yr,mn,dy,yr1,mn1,dy1,dum, slice1,slice2

    fname='clsm/catchment.def'
    open (10,file=fname,status='old',action='read',form='formatted')
    read (10,*)maxcat

    allocate (albvr    (1:maxcat))
    allocate (albvf    (1:maxcat))
    allocate (albnr    (1:maxcat))
    allocate (albnf    (1:maxcat))
    allocate (calbvf   (1:maxcat))
    allocate (calbnf   (1:maxcat))
    allocate (modisvf  (1:maxcat))
    allocate (modisnf  (1:maxcat))
    allocate (lai      (1:maxcat))
    allocate (green    (1:maxcat))
    allocate (lai_before (1:maxcat))
    allocate (grn_before (1:maxcat))
    allocate (lai_after  (1:maxcat))
    allocate (grn_after  (1:maxcat))
    allocate (vegcls     (1:maxcat))
    allocate (snw      (1:maxcat))
    close (10,status='keep')

    fname=trim(gfile)//'.til'

    open (10,file=fname,status='old',action='read',form='formatted')
    fname='clsm/mosaic_veg_typs_fracs'
    open (20,file=fname,status='old',action='read',form='formatted')

    read (10,*)ip
    read (10,*)j_dum
    read (10,'(a)')version
    read (10,*)nc_gcm
    read (10,*)nr_gcm
    read (10,'(a)')version
    read (10,*)nc_ocean
    read (10,*)nr_ocean

    do n = 1,ip
      if (ease_grid) then     
	 read(10,*,IOSTAT=ierr) typ,pfs,lon,lat,ig,jg,fr_gcm
      else
      read(10,'(I10,3E20.12,9(2I10,E20.12,I10))',IOSTAT=ierr)     &    
            typ,tarea,lont,latt,ig,jg,fr_gcm,indx_dum,pfs,j_dum,fr_cat,j_dum
      endif
       if (typ == 100) then
          ip2 = n 
          read (20,'(i8,i8,2(2x,i3),2(2x,f6.4))')     &
            indr1,indr1,vegcls(ip2),indr1,fr_gcm,fr_gcm
       endif
       if(ierr /= 0)write (*,*)'Problem reading'
    end do
    close (10,status='keep')
    close (20,status='keep')

    albvr    =0.
    albvf    =0.
    albnr    =0.
    albnf    =0.
    calbvf   =0.
    calbnf   =0.
    modisvf  =0.
    modisnf  =0.
    snw      =0.

! MODIS Albedo files
    if(MA == 'MODIS1') then 
       open (10,file='clsm/AlbMap.WS.16-day.tile.0.3_0.7.dat',&
            form='unformatted',convert='little_endian', &
            action='read',status='old')   
       open (11,file='clsm/AlbMap.WS.16-day.tile.0.7_5.0.dat',&
            form='unformatted',convert='little_endian', &
            action='read',status='old') 
    endif

    if(MA == 'MODIS2') then
       open (10,file='clsm/AlbMap.WS.8-day.tile.0.3_0.7.dat',&
            form='unformatted',convert='little_endian', &
            action='read',status='old')   
       open (11,file='clsm/AlbMap.WS.8-day.tile.0.7_5.0.dat',&
            form='unformatted',convert='little_endian', &
            action='read',status='old') 
    endif

    read(10) yr,mn,dy,dum,dum,dum,yr1,mn1,dy1
    read(11) yr,mn,dy,dum,dum,dum,yr1,mn1,dy1
    read (10) modisvf (:)
    read (11) modisnf (:)

! MODIS scale parameter files
    open (30,file='clsm/visdf.dat',convert='little_endian', &
             action='write',status='unknown',form='unformatted')
    open (31,file='clsm/nirdf.dat',convert='little_endian', &
             action='write',status='unknown',form='unformatted')
    write(30) yr,mn,dy,0.,0.,0.,yr1,mn1,dy1,0.,0.,0.,float(maxcat),1.
    write(31) yr,mn,dy,0.,0.,0.,yr1,mn1,dy1,0.,0.,0.,float(maxcat),1.
    call Get_MidTime(yr,mn,dy,yr1,mn1,dy1,date_time_new)

    bf_modis_time   = date_time_new

    read(10) yr,mn,dy,dum,dum,dum,yr1,mn1,dy1
    read(11) yr,mn,dy,dum,dum,dum,yr1,mn1,dy1

    call Get_MidTime(yr,mn,dy,yr1,mn1,dy1,af_modis_time)
    end_time        = af_modis_time
    end_time%year   = NINT(yr1) + 2001 + 1 

    rewind (10)
    rewind (11)

    read(10) yr,mn,dy,dum,dum,dum,yr1,mn1,dy1
    read(11) yr,mn,dy,dum,dum,dum,yr1,mn1,dy1
    read (10) modisvf (:)
    read (11) modisnf (:)

    fname='clsm/lai.dat'
    open (40,file=fname,status='old',action='read',form='unformatted', &
         convert='little_endian')
    read(40) yr,mn,dy,dum,dum,dum,yr1,mn1,dy1
    read(40) lai_before
    call Get_MidTime(yr,mn,dy,yr1,mn1,dy1,bf_lai_time)
 
    read(40) yr,mn,dy,dum,dum,dum,yr1,mn1,dy1
    read(40) lai_after
    call Get_MidTime(yr,mn,dy,yr1,mn1,dy1,af_lai_time)
 
   if(date_time_new%dofyr < bf_lai_time%dofyr) then
       do while ((date_time_new%dofyr >  af_lai_time%dofyr)) 
          lai_before = lai_after
          bf_lai_time = af_lai_time
          read(40) yr,mn,dy,dum,dum,dum,yr1,mn1,dy1
          read(40) lai_after
          call Get_MidTime(yr,mn,dy,yr1,mn1,dy1,af_lai_time)
        end do
    endif

    fname='clsm/green.dat'
    open (41,file=fname,status='old',action='read',form='unformatted', &
         convert='little_endian')
    read(41) yr,mn,dy,dum,dum,dum,yr1,mn1,dy1
    read(41) grn_before
    call Get_MidTime(yr,mn,dy,yr1,mn1,dy1,gf_green_time)
    
    read(41) yr,mn,dy,dum,dum,dum,yr1,mn1,dy1
    read(41) grn_after
    call Get_MidTime(yr,mn,dy,yr1,mn1,dy1,af_green_time)

    calbvf   =0.
    calbnf   =0.
    albvr    =0.
    albnr    =0.
    albvf    =0.
    albnf    =0.
    tsteps   =0.    

    do while (datetime_le_refdatetime(date_time_new,end_time))
       
!       write (*,'(a48,i4.4,i2.2,i2.2)') '     Computing MODIS scale parameters for month: ', &
            
              
          if (datetime_le_refdatetime(date_time_new,af_lai_time)) then

          else

            read(40,IOSTAT=ierr) yr,mn,dy,dum,dum,dum,yr1,mn1,dy1 
            if(ierr == 0) then 
               lai_before = lai_after
               read(40) lai_after
               bf_lai_time = af_lai_time
               call Get_MidTime(yr,mn,dy,yr1,mn1,dy1,af_lai_time)
            else
               rewind(40) 
               read(40) yr,mn,dy,dum,dum,dum,yr1,mn1,dy1
               read(40) lai_before
               call Get_MidTime(yr,mn,dy,yr1,mn1,dy1,bf_lai_time)
               read(40) yr,mn,dy,dum,dum,dum,yr1,mn1,dy1
               read(40) lai_after
               call Get_MidTime(yr,mn,dy,yr1,mn1,dy1,af_lai_time)

               if(date_time_new%dofyr < bf_lai_time%dofyr) then
                  do while ((date_time_new%dofyr >  af_lai_time%dofyr))
                     lai_before = lai_after
                     bf_lai_time = af_lai_time
                     read(40) yr,mn,dy,dum,dum,dum,yr1,mn1,dy1
                     read(40) lai_after
                     call Get_MidTime(yr,mn,dy,yr1,mn1,dy1,af_lai_time)
                  end do
               endif
            endif
         endif
         call Time_Interp_Fac (date_time_new, bf_lai_time, af_lai_time, slice1, slice2)
         lai    = (slice1*lai_before + slice2*lai_after)

         if (datetime_le_refdatetime(date_time_new,af_green_time)) then
            
         else
            
            read(41,IOSTAT=ierr) yr,mn,dy,dum,dum,dum,yr1,mn1,dy1
            if(ierr == 0) then 
               grn_before = grn_after
               gf_green_time = af_green_time
               read(41) grn_after
               call Get_MidTime(yr,mn,dy,yr1,mn1,dy1,af_green_time)
            endif
         endif
!            else
!               rewind(41) 
!               read(41) yr,mn,dy,dum,dum,dum,yr1,mn1,dy1
!               read(41) grn_before
!               gf_green_time%month = NINT(mn)
!               gf_green_time%day   = NINT(dy)
!               call get_dofyr_pentad(gf_green_time)
!               af_green_time%month = NINT(mn1)
!               af_green_time%day   = NINT(dy1)
!               call get_dofyr_pentad(af_green_time)    
!               if(date_time_new%dofyr < gf_green_time%dofyr) then
!                  do while ((date_time_new%dofyr >  af_green_time%dofyr)) 
!                     read(41) yr,mn,dy,dum,dum,dum,yr1,mn1,dy1
!                     read(41) grn_before
!                     gf_green_time%year  = date_time_new%year
!                     gf_green_time%month = NINT(mn)
!                     gf_green_time%day   = NINT(dy)
!                     call get_dofyr_pentad(gf_green_time)
!                     af_green_time%year  = date_time_new%year
!                     if ((yr1-yr) == 1.)af_green_time%year = af_green_time%year+1
!                     af_green_time%month = NINT(mn1)
!                     af_green_time%day   = NINT(dy1)
!                     call get_dofyr_pentad(af_green_time)           
!                  end do
!                  read(41) yr,mn,dy,dum,dum,dum,yr1,mn1,dy1
!                  read(41) grn_after
!               endif
!            endif
!         endif

         call Time_Interp_Fac (date_time_new, gf_green_time, af_green_time, slice1, slice2)
         green  = (slice1*grn_before + slice2*grn_after)
          
          call sibalb(                                    &
               albvr,albnr,albvf,albnf,                   &
               lai, green, 0.0, snw, vegcls, maxcat)  
          
          calbvf = calbvf + albvf
          calbnf = calbnf + albnf         
          tsteps = tsteps + 1.  
          call augment_date_time( 86400, date_time_new ) 

          if (datetime_le_refdatetime(date_time_new,af_modis_time)) then
             
          else
             bf_modis_time = af_modis_time
             calbvf = calbvf/tsteps
             calbnf = calbnf/tsteps

             modisvf = modisvf/calbvf
             modisnf = modisnf/calbnf
           
             do n =1, maxcat
!                if(modisvf(n).le.0)print *,'Negative MODISVF scale param at cell',n, modisvf(n)
!                if(modisnf(n).le.0)print *,'Negative MODISNF scale param at cell',n, modisnf(n)
!                if(modisvf(n).gt.100)print *,'Too large MODISVF scale param at cell',n, modisvf(n)
!                if(modisnf(n).gt.100)print *,'Too large MODISNF scale param at cell',n, modisnf(n)
                if(modisvf(n).le.0.) modisvf(n) = 1.
                if(modisnf(n).le.0.) modisnf(n) = 1.
                if(modisvf(n).gt.100)modisvf(n)= 1.
                if(modisnf(n).gt.100)modisnf(n)= 1.
            enddo

            write (30) modisvf (:)
            write (31) modisnf (:)

	    read(10,IOSTAT=ierr) yr,mn,dy,dum,dum,dum,yr1,mn1,dy1       
  
            if(ierr == 0) then
	       
               read(11) yr,mn,dy,dum,dum,dum,yr1,mn1,dy1
               read (10) modisvf (:)
               read (11) modisnf (:)  
               write(30) yr,mn,dy,0.,0.,0.,yr1,mn1,dy1,0.,0.,0.,float(maxcat),1.
               write(31) yr,mn,dy,0.,0.,0.,yr1,mn1,dy1,0.,0.,0.,float(maxcat),1.
 	       bf_modis_time = af_modis_time
               call Get_MidTime(yr,mn,dy,yr1,mn1,dy1,af_modis_time)
               calbvf   =0.
               calbnf   =0.
               albvr    =0.
               albnr    =0.
               albvf    =0.
               albnf    =0.
               tsteps   =0.   
            endif
         endif
      end do
  
      deallocate (modisvf,modisnf,albvr,albvf,albnf,albnr)
      deallocate (green,lai)
      deallocate (vegcls)
      deallocate (calbvf,calbnf)
      deallocate (lai_before,grn_before, lai_after,grn_after,snw)

      close (10, status='keep')
      close (11, status='keep')        
      close (30, status='keep')
      close (31, status='keep')
  
END SUBROUTINE modis_scale_para_high

!
! ---------------------------------------------------------------------------------------
! 
  SUBROUTINE modis_alb_on_tiles_high (nx,ny,MA,gfiler,c_data)
!
! Processing MODIS Albedo and creating 8-day climatological data 
!
  implicit none 
  integer, intent (in) :: nx, ny
  character*6 :: MA
  character(*)  :: gfiler,c_data
  integer :: n,maxcat,i,j,k,ncid,i_highd,j_highd,nx_adj,ny_adj
  integer :: status,iLL,jLL,ix,jx,vid,nc_10,nr_10,n_tslices,d_undef,t,  &
      time_slice,time_slice_next,yr,mn,dd,yr1,mn1,dd1,i1,i2
  character*100 :: fname,fout
  character*10 :: string
  character*2 :: VV,HH
  integer, allocatable, dimension (:,:) :: &
         net_data1,net_data2
  integer (kind=2) , allocatable, target, dimension (:,:) :: AlbVis_HIGH,AlbNir_HIGH
  integer (kind=2), pointer, dimension (:,:) :: Raster1,Raster2
  REAL, ALLOCATABLE, dimension (:) :: vec_AlbVis, count_AlbVis,vec_AlbNir, count_AlbNir
  integer, allocatable, target, dimension (:,:) :: tile_id
  integer, pointer :: iRaster(:,:)
  character(len=4), dimension (:), allocatable :: MMDD, MMDD_next
  logical :: regrid
  character *10 :: vname
  REAL :: sf

!
! Reading number of cathment-tiles from catchment.def file
! 
      fname='clsm/catchment.def' 
      open (10,file=fname,status='old',action='read',form='formatted')
      read(10,*) maxcat
      close (10,status='keep')

      if(MA=='MODIS1') fname =trim(c_data)//'MODIS-Albedo/MODISalb.c004.v2.WS_H11V13.nc'
      if(MA=='MODIS2') fname =trim(c_data)//'MODIS-Albedo2/MCD43GF_wsa_H11V13.nc'
      status = NF_OPEN(trim(fname),NF_NOWRITE, ncid); VERIFY_(STATUS)
      status = NF_GET_att_INT(ncid,NF_GLOBAL,'i_ind_offset_LL',iLL); VERIFY_(STATUS)
      status = NF_GET_att_INT(ncid,NF_GLOBAL,'j_ind_offset_LL',jLL); VERIFY_(STATUS)
      status = NF_GET_att_INT(ncid,NF_GLOBAL,'N_lon_global',i_highd); VERIFY_(STATUS)
      status = NF_GET_att_INT(ncid,NF_GLOBAL,'N_lat_global',j_highd); VERIFY_(STATUS)
      status = NF_INQ_DIM (ncid,1,string, nc_10); VERIFY_(STATUS)
      status = NF_INQ_DIM (ncid,2,string, nr_10); VERIFY_(STATUS)
      status = NF_INQ_DIM (ncid,3,string, n_tslices); VERIFY_(STATUS)
      allocate (MMDD      (0: n_tslices + 1))
      allocate (MMDD_next (0: n_tslices + 1))

      status = NF_GET_VARA_text(ncid, 3,(/1,1/),(/4,n_tslices/),MMDD(1:n_tslices)); VERIFY_(STATUS)
      status = NF_CLOSE(ncid); VERIFY_(STATUS)

       mmdd(0) = mmdd(n_tslices)
       mmdd(n_tslices + 1)= mmdd(1)

       mmdd_next(0:n_tslices - 1) =  mmdd(1:n_tslices)
       mmdd_next(n_tslices: n_tslices + 1) = mmdd (1:2)


       allocate(tile_id(1:nx,1:ny))
       allocate(net_data1 (1:nc_10,1:nr_10))
       allocate(net_data2 (1:nc_10,1:nr_10))       
       fname=trim(gfiler)//'.rst'
       !          
       ! Reading tile-id raster file
       !
       open (10,file=fname,status='old',action='read',  &
            form='unformatted',convert='little_endian')
       
       do j=1,ny
          read(10)tile_id(:,j)
       end do
       
       close (10,status='keep')
       !
       ! reading Albedo data
       !

       if(MA == 'MODIS1') then 
          open (31,file='clsm/AlbMap.WS.16-day.tile.0.3_0.7.dat',        &
               form='unformatted',status='unknown',convert='little_endian')
          open (32,file='clsm/AlbMap.WS.16-day.tile.0.7_5.0.dat',        &
               form='unformatted',status='unknown',convert='little_endian')
       endif

       if(MA == 'MODIS2') then 
          open (31,file='clsm/AlbMap.WS.8-day.tile.0.3_0.7.dat',        &
               form='unformatted',status='unknown',convert='little_endian')
          open (32,file='clsm/AlbMap.WS.8-day.tile.0.7_5.0.dat',        &
               form='unformatted',status='unknown',convert='little_endian')
       endif

       allocate(vec_AlbVis(maxcat))
       allocate(AlbVis_high(1:i_highd,1:j_highd))  
       allocate(count_AlbVis(1:maxcat))    
       allocate(vec_AlbNir(maxcat))
       allocate(AlbNir_high(1:i_highd,1:j_highd))  
       allocate(count_AlbNir(1:maxcat))    

       do t =0,n_tslices+1
       
          time_slice = t
          yr = 1
          yr1= 1
          if(t == 0) then
             time_slice =  n_tslices
             yr         =  1 - 1
          endif

          if(t >= n_tslices) then 
             yr1 = 1 + 1
             if(t ==n_tslices + 1) then
                time_slice =  1
                yr = 1 + 1
             endif
          endif

          read(mmdd(t),'(i2.2,i2.2)') mn,dd
          read(mmdd_next(t),'(i2.2,i2.2)') mn1,dd1
 
          AlbVis_high = -9999
          AlbNir_high = -9999

          do jx = 1,18	
             do ix = 1,36
                write (vv,'(i2.2)')jx
                write (hh,'(i2.2)')ix 
                if(MA=='MODIS1') fname =trim(c_data)//'MODIS-Albedo/MODISalb.c004.v2.WS_H'//hh//'V'//vv//'.nc'
                if(MA=='MODIS2') fname =trim(c_data)//'MODIS-Albedo2/MCD43GF_wsa_H'//hh//'V'//vv//'.nc'
                status = NF_OPEN(trim(fname),NF_NOWRITE, ncid)
                if(status == 0) then
                   status = NF_GET_att_INT  (ncid,NF_GLOBAL,'i_ind_offset_LL',iLL); VERIFY_(STATUS)
                   status = NF_GET_att_INT  (ncid,NF_GLOBAL,'j_ind_offset_LL',jLL); VERIFY_(STATUS)
                   status = NF_GET_att_INT  (ncid,4,'UNDEF',d_undef); VERIFY_(STATUS)
                   status = NF_GET_att_REAL (ncid,4,'ScaleFactor',sf); VERIFY_(STATUS)
                   status = NF_GET_VARA_INT (ncid,4,(/1,1,time_slice/),(/nc_10,nr_10,1/),net_data1); VERIFY_(STATUS)
                   status = NF_GET_VARA_INT (ncid,5,(/1,1,time_slice/),(/nc_10,nr_10,1/),net_data2); VERIFY_(STATUS)

                   do j = jLL,jLL + nr_10 -1 
                      do i = iLL, iLL + nc_10 -1 
                         if(net_data1(i-iLL +1 ,j - jLL +1) /= d_undef) &
                              AlbVis_high(i,j) = net_data1(i-iLL +1 ,j - jLL +1)
                         if(net_data2(i-iLL +1 ,j - jLL +1) /= d_undef) &
                              AlbNir_high(i,j) = net_data2(i-iLL +1 ,j - jLL +1)
                      enddo
                   enddo
                   status = NF_CLOSE(ncid)
                endif
             end do
          end do
          
          ! Regridding 
          
          nx_adj = nx
          ny_adj = ny
          
          regrid = nx/=i_highd .or. ny/=j_highd
          
          if(regrid) then
             if(nx > i_highd) then 
!                if( associated(raster1) ) deallocate( raster1 )
!                if( associated(raster2) ) deallocate( raster2 )
                if( associated(raster1) ) then
                else
                   allocate(raster1(nx,ny),stat=STATUS); VERIFY_(STATUS)
                   allocate(raster2(nx,ny),stat=STATUS); VERIFY_(STATUS)                   
                endif
                call RegridRaster2(AlbVis_high,raster1)
                call RegridRaster2(AlbNir_high,raster2)
                iRaster => tile_id
                if(ny < j_highd) then
                   print *,'nx > i_highd and ny < j_highd'
                   stop 
                endif
             else
                allocate(iraster(i_highd,j_highd),stat=STATUS); VERIFY_(STATUS)
                call RegridRaster(tile_id,iraster)	
                raster1 => AlbVis_high
                raster2 => AlbNir_high
                nx_adj = i_highd
                ny_adj = j_highd
                
                if(ny > j_highd) then
                   print *,'nx < i_highd and ny > j_highd'
                   stop 
                endif
             endif
          else
             raster1 => AlbVis_high
             raster2 => AlbNir_high
             iRaster => tile_id
          end if
          
          ! Interpolation or aggregation on to catchment-tiles
          
          vec_AlbVis   =0.
          count_AlbVis = 0.
          vec_AlbNir   =0.
          count_AlbNir = 0. 
         
          do j=1,ny_adj
             do i=1,nx_adj
                if((iRaster(i,j).gt.0).and.(iRaster(i,j).le.maxcat)) then
                   if ((raster1(i,j).gt.0)) then
                      vec_AlbVis(iRaster(i,j)) = &
                           vec_AlbVis(iRaster(i,j)) + sf*raster1(i,j)
                      count_AlbVis(iRaster(i,j)) = &
                           count_AlbVis(iRaster(i,j)) + 1. 
                   endif

                   if ((raster2(i,j).gt.0)) then
                      vec_AlbNir(iRaster(i,j)) = &
                           vec_AlbNir(iRaster(i,j)) + sf*raster2(i,j)
                      count_AlbNir(iRaster(i,j)) = &
                           count_AlbNir(iRaster(i,j)) + 1. 
                   endif
                endif
             end do
          end do

          deallocate (iraster) ; VERIFY_(STATUS)

          DO n =1,maxcat
             if(count_AlbVis(n)/=0.) vec_AlbVis(n)=vec_AlbVis(n)/count_AlbVis(n)
             if(count_AlbNir(n)/=0.) vec_AlbNir(n)=vec_AlbNir(n)/count_AlbNir(n)
          END DO
          write(31) float((/yr,mn,dd,0,0,0,yr1,mn1,dd1,0,0,0,maxcat,1/))
          write(31)  vec_AlbVis(:)
          write(32) float((/yr,mn,dd,0,0,0,yr1,mn1,dd1,0,0,0,maxcat,1/))
          write(32)  vec_AlbNir(:)
       end do
       close(31,status='keep')
       close(32,status='keep')

       deallocate (net_data1,net_data2)
       deallocate (AlbVis_HIGH, AlbNir_HIGH)
       deallocate (count_AlbVis, count_AlbNir)
       deallocate (vec_AlbVis, vec_AlbNir)
       deallocate (tile_id)

     END SUBROUTINE modis_alb_on_tiles_high
!
! ---------------------------------------------------------------------------------------
! 
  SUBROUTINE hres_lai (nx,ny,gfiler,c_data,lai_name,merge)
!
! Processing GEOLAND2/MODIS LAI and creating 10-day climatological data 
!
  implicit none 
  integer, intent (in) :: nx, ny 
  character(*)  :: gfiler,c_data,lai_name
  integer :: n,maxcat,i,j,k,ncid,i_highd,j_highd,nx_adj,ny_adj,ierr
  integer :: status,iLL,jLL,ix,jx,vid,nc_10,nr_10,n_tslices,d_undef,t,  &
      time_slice,time_slice_next,yr,mn,dd,yr1,mn1,dd1,i1,i2
  real :: dum, gyr,gmn,gdy,gyr1,gmn1,gdy1, slice1,slice2
  character*100 :: fname,fout
  character*10 :: string
  character*2 :: VV,HH
  integer, allocatable, dimension (:,:) :: &
         net_data1
  integer (kind=2) , allocatable, target, dimension (:,:) :: LAI_HIGH
  integer (kind=2), pointer, dimension (:,:) :: Raster
  REAL, ALLOCATABLE, dimension (:) :: vec_lai, count_lai
  REAL, ALLOCATABLE, dimension (:) :: gswp2_lai_bf,gswp2_lai_af,gswp2_lai 
  integer, allocatable, target, dimension (:,:) :: tile_id
  integer, pointer :: iRaster(:,:)
  character(len=4), dimension (:), allocatable :: MMDD, MMDD_next
  logical :: regrid
  REAL :: sf
  logical :: first_entry = .true.
  type (date_time_type) :: bf_gswp2_time,af_gswp2_time,date_time_new,bf_lai_time,   &
       af_lai_time
  integer, intent(in), optional :: merge
!
! Reading number of cathment-tiles from catchment.def file
! 
      if (first_entry) then
          nullify(iraster) ; first_entry = .false.
      end if

      fname='clsm/catchment.def' 
      open (10,file=fname,status='old',action='read',form='formatted')
      read(10,*) maxcat
      close (10,status='keep')

      fname =trim(c_data)//trim(lai_name)//'lai_clim.H11V13.nc'
      status = NF_OPEN(trim(fname),NF_NOWRITE, ncid); VERIFY_(STATUS)
      status = NF_GET_att_INT(ncid,NF_GLOBAL,'i_ind_offset_LL',iLL); VERIFY_(STATUS)
      status = NF_GET_att_INT(ncid,NF_GLOBAL,'j_ind_offset_LL',jLL); VERIFY_(STATUS)
      status = NF_GET_att_INT(ncid,NF_GLOBAL,'N_lon_global',i_highd); VERIFY_(STATUS)
      status = NF_GET_att_INT(ncid,NF_GLOBAL,'N_lat_global',j_highd); VERIFY_(STATUS)
      status = NF_INQ_DIM (ncid,1,string, nc_10); VERIFY_(STATUS)
      status = NF_INQ_DIM (ncid,2,string, nr_10); VERIFY_(STATUS)
      status = NF_INQ_DIM (ncid,3,string, n_tslices); VERIFY_(STATUS)
      allocate (MMDD      (0: n_tslices + 1))
      allocate (MMDD_next (0: n_tslices + 1))

      status = NF_GET_VARA_text(ncid, 3,(/1,1/),(/4,n_tslices/),MMDD(1:n_tslices)); VERIFY_(STATUS)
      status = NF_CLOSE(ncid); VERIFY_(STATUS)

       mmdd(0) = mmdd(n_tslices)
       mmdd(n_tslices + 1)= mmdd(1)

       mmdd_next(0:n_tslices - 1) =  mmdd(1:n_tslices)
       mmdd_next(n_tslices: n_tslices + 1) = mmdd (1:2)


       allocate(tile_id(1:nx,1:ny))
       allocate(net_data1 (1:nc_10,1:nr_10))
       
       fname=trim(gfiler)//'.rst'
       !          
       ! Reading tile-id raster file
       !
       open (10,file=fname,status='old',action='read',  &
            form='unformatted',convert='little_endian')
       
       do j=1,ny
          read(10)tile_id(:,j)
       end do
       
       close (10,status='keep')

       !
       ! writing GEOSLAND2 LAI data
       !

       if(present(merge)) then
       open (31,file='clsm/lai.'//lai_name(1:index(lai_name,'/')-1),  &
            form='unformatted',status='unknown',convert='little_endian')
       else
       open (31,file='clsm/lai.dat',  &
            form='unformatted',status='unknown',convert='little_endian')
       endif

       allocate(vec_lai(maxcat))
       allocate(lai_high(1:i_highd,1:j_highd))  
       allocate(count_lai(1:maxcat))    
       allocate(gswp2_lai_bf (1:maxcat))    
       allocate(gswp2_lai_af (1:maxcat))    
       allocate(gswp2_lai    (1:maxcat))   

       !
       ! reading GSWP2 LAI data
       !
       
       open (41,file='clsm/lai.dat.gswp2',  &
         form='unformatted',status='old',convert='little_endian',action='read')
       read(41) gyr,gmn,gdy,dum,dum,dum,gyr1,gmn1,gdy1
       read(41) gswp2_lai_bf
       call Get_MidTime(gyr,gmn,gdy,gyr1,gmn1,gdy1,bf_gswp2_time)

       read(41) gyr,gmn,gdy,dum,dum,dum,gyr1,gmn1,gdy1
       read(41) gswp2_lai_af
       call Get_MidTime(gyr,gmn,gdy,gyr1,gmn1,gdy1,af_gswp2_time)

       do t =0,n_tslices+1
       
          time_slice = t
          yr = 1
          yr1= 1
          if(t == 0) then
             time_slice =  n_tslices
             yr         =  1 - 1
          endif

          if(t >= n_tslices) then 
             yr1 = 1 + 1
             if(t ==n_tslices + 1) then
                time_slice =  1
                yr = 1 + 1
             endif
          endif

          read(mmdd(t),'(i2.2,i2.2)') mn,dd
          read(mmdd_next(t),'(i2.2,i2.2)') mn1,dd1
 
          lai_high = -9999

          do jx = 1,18	
             do ix = 1,36
                write (vv,'(i2.2)')jx
                write (hh,'(i2.2)')ix 
                fname = trim(c_data)//trim(lai_name)//'lai_clim.H'//hh//'V'//vv//'.nc'
                status = NF_OPEN(trim(fname),NF_NOWRITE, ncid)
                if(status == 0) then
                   status = NF_GET_att_INT  (ncid,NF_GLOBAL,'i_ind_offset_LL',iLL); VERIFY_(STATUS)
                   status = NF_GET_att_INT  (ncid,NF_GLOBAL,'j_ind_offset_LL',jLL); VERIFY_(STATUS)
                   status = NF_GET_att_INT  (ncid,4,'UNDEF',d_undef); VERIFY_(STATUS)
                   status = NF_GET_att_REAL (ncid,4,'ScaleFactor',sf); VERIFY_(STATUS)
                   status = NF_GET_VARA_INT (ncid, 4,(/1,1,time_slice/),(/nc_10,nr_10,1/),net_data1); VERIFY_(STATUS)
                   
                   do j = jLL,jLL + nr_10 -1 
                      do i = iLL, iLL + nc_10 -1 
                         if(net_data1(i-iLL +1 ,j - jLL +1) /= d_undef) &
                              lai_high(i,j) = net_data1(i-iLL +1 ,j - jLL +1)
                      enddo
                   enddo
                   status = NF_CLOSE(ncid)
                endif
             end do
          end do
          
          ! Regridding 
          
          nx_adj = nx
          ny_adj = ny
          
          regrid = nx/=i_highd .or. ny/=j_highd
          
          if(regrid) then
             if(nx > i_highd) then 
                allocate(raster(nx,ny),stat=STATUS); VERIFY_(STATUS)
                call RegridRaster2(lai_high,raster)	
                iRaster => tile_id
                if(ny < j_highd) then
                   print *,'nx > i_highd and ny < j_highd'
                   stop 
                endif
             else
                if(.not. associated(iraster)) then
                   allocate(iraster(i_highd,j_highd),stat=STATUS); VERIFY_(STATUS)  
                endif

!		if( associated(iraster)) deallocate(iraster)
!	       allocate(iraster(i_highd,j_highd),stat=STATUS); VERIFY_(STATUS)    
               call RegridRaster(tile_id,iraster)	
                raster  => lai_high
                nx_adj = i_highd
                ny_adj = j_highd
                
                if(ny > j_highd) then
                   print *,'nx < i_highd and ny > j_highd'
                   stop 
                endif
             endif
          else
             raster  => lai_high
             iRaster => tile_id
          end if
          
          ! Interpolation or aggregation on to catchment-tiles
          
          vec_lai =0.
          count_lai = 0.
          
          do j=1,ny_adj
             do i=1,nx_adj
                if((iRaster(i,j).gt.0).and.(iRaster(i,j).le.maxcat)) then
                   if ((raster(i,j).gt.0)) then
                      vec_lai(iRaster(i,j)) = &
                           vec_lai(iRaster(i,j)) + sf*raster(i,j)
                      count_lai(iRaster(i,j)) = &
                           count_lai(iRaster(i,j)) + 1. 
                   endif
                endif
             end do
          end do

          write(31) float((/yr,mn,dd,0,0,0,yr1,mn1,dd1,0,0,0,maxcat,1/))
	  call Get_MidTime(real(yr),real(mn),real(dd),real(yr1),real(mn1),real(dd1),date_time_new)
!          date_time_new%year   = yr + 2001
!          date_time_new%month  = mn
!          date_time_new%day    = dd
!          date_time_new%hour   = 0            
!          date_time_new%min    = 0            
!          date_time_new%sec    = 0 
!          call get_dofyr_pentad(date_time_new)             

          if (datetime_le_refdatetime(date_time_new,af_gswp2_time)) then
             
          else
             read(41,IOSTAT=ierr) gyr,gmn,gdy,dum,dum,dum,gyr1,gmn1,gdy1
             if(ierr == 0) then
                gswp2_lai_bf = gswp2_lai_af
                read(41) gswp2_lai_af
                bf_gswp2_time = af_gswp2_time
                call Get_MidTime(gyr,gmn,gdy,gyr1,gmn1,gdy1,af_gswp2_time)
             else
                print *,'END OF GSWP2 LAI FILE'
                stop
             endif
          endif

          call Time_Interp_Fac (date_time_new, bf_gswp2_time, af_gswp2_time, slice1, slice2)
          gswp2_lai    = (slice1*gswp2_lai_bf + slice2*gswp2_lai_af)
          
!          print *, 'Merging GEOLAND2-AVHRR'
!          print *,  bf_gswp2_time
!          print *,  date_time_new
!          print *,  af_gswp2_time
!          print *,  slice1, slice2
!          print *, maxval(gswp2_lai), minval(gswp2_lai)

          DO n =1,maxcat
             if(count_lai(n)/=0.) vec_lai(n)= vec_lai(n)/count_lai(n)
             if(vec_lai(n)==0.) vec_lai(n)  = gswp2_lai(n)
          END DO
          
          write(31)  vec_lai(:)
       end do
       close(31,status='keep')
       close(41,status='keep')

       deallocate (net_data1)
       deallocate (LAI_HIGH)
       deallocate (count_lai)
       deallocate (vec_lai, iRaster)
       deallocate (gswp2_lai_bf,gswp2_lai_af,gswp2_lai, tile_id)

  END SUBROUTINE hres_lai
!----------------------------------------------------------------------  

  SUBROUTINE soil_para_hwsd (nx,ny,gfiler,c_data)

!
! Processing NGDC-HWSD-STATSGO merged soil properties with Woesten Soil
! Parameters and produces tau_param.dat and soil_param.dat files
! 
      implicit none	    
      integer, intent (in) :: nx, ny 
      character(*)  :: gfiler,c_data
      real, dimension (:), allocatable ::           &
      	    a_sand,a_clay,a_silt,a_oc,a_bee,a_psis, &
            a_poros,a_wp,a_aksat,atau,btau,a_wpsurf,a_porosurf, &
            atau_2cm,btau_2cm 
      integer, dimension (100,3) :: table_map
      integer, dimension (3) :: nsoil_pcarbon 
      type (mineral_perc) :: min_percs
 
      integer :: n,maxcat,i,j,k,ktop,ncid,i_highd,j_highd,nx_adj,ny_adj
      integer :: status,iLL,jLL,ix,jx,vid,nc_10,nr_10,d_undef,   &
                 i1,i2,icount
      character*100 :: fname,fout
      character*10 :: string
      character*2 :: VV,HH

      logical, allocatable, dimension(:,:) :: land_pixels
      integer, allocatable, dimension (:,:) :: &
         net_data1,net_data2,net_data3,net_data4,net_data5,net_data6 ,net_data7 
      integer (kind=2) , allocatable, target, dimension (:,:) :: SOIL_HIGH,  &
          sand_top,clay_top,oc_top,sand_sub,clay_sub,oc_sub, grav_grid
      integer (kind=2), pointer, dimension (:,:) :: Raster, &  
           Raster1,Raster2,Raster3,Raster4,Raster5,Raster6
      integer (kind=4), allocatable, dimension (:) :: tileid_vec,arrayA,arrayB          
      integer (kind=2), allocatable, dimension (:) ::  &
         data_vec1, data_vec2,data_vec3, data_vec4,data_vec5, data_vec6
      REAL, ALLOCATABLE, dimension (:) :: soildepth, grav_vec,soc_vec,poc_vec,&
             ncells_top,ncells_top_pro,ncells_sub_pro 
      integer(kind=2) , allocatable, dimension (:) :: ss_clay,    &
             ss_sand,ss_clay_all,ss_sand_all,ss_oc_all
      REAL, ALLOCATABLE :: count_soil(:)
      integer, allocatable, target, dimension (:,:) :: tile_id
      integer, pointer :: iRaster(:,:)
      integer :: tindex, pfafindex,fac,o_cl,o_clp,fac_surf,vtype
      real,dimension(4) :: cFamily
      real   ,dimension(5) :: cF_lim
      logical :: first_entry = .true.
      logical :: regrid,write_file
      INTEGER, allocatable, dimension (:) :: soil_class_top,soil_class_com
      REAL :: sf,factor,wp_wetness,fac_count

! --------- VARIABLES FOR *OPENMP* PARALLEL ENVIRONMENT ------------
!
! NOTE: "!$" is for conditional compilation
!
logical :: running_omp = .false.
!
!$ integer :: omp_get_thread_num, omp_get_num_threads
!
integer :: n_threads=1, li, ui, t_count
!
integer, dimension(:), allocatable :: low_ind, upp_ind
!
! ------------------------------------------------------------------
        
  ! ----------- OpenMP PARALLEL ENVIRONMENT ----------------------------
  !
  ! FIND OUT WHETHER -omp FLAG HAS BEEN SET DURING COMPILATION
  !
  !$ running_omp = .true.         ! conditional compilation
  !
  ! ECHO BASIC OMP VARIABLES
  !
  !$OMP PARALLEL DEFAULT(NONE) SHARED(running_omp,n_threads) 
  !
  !$OMP SINGLE
  !
  !$ n_threads = omp_get_num_threads()
  !
  !$ write (*,*) 'running_omp = ', running_omp
  !$ write (*,*)
  !$ write (*,*) 'parallel OpenMP with ', n_threads, 'threads'
  !$ write (*,*)
  !$OMP ENDSINGLE
  !
  !$OMP CRITICAL
  !$ write (*,*) 'thread ', omp_get_thread_num(), ' alive'
  !$OMP ENDCRITICAL
  !
  !$OMP BARRIER
  !
  !$OMP ENDPARALLEL

   if (first_entry) then
      nullify(iraster) ; first_entry = .false.
   endif
      cF_lim(1) =  0.
      cF_lim(2) =  0.4      ! 0.365    ! 0.3
      cF_lim(3) =  0.64     ! 0.585    ! 4.0	   
      cF_lim(4) =  15./1.72 ! 9.885    ! 8.5
      cF_lim(5) =  100.0

      nsoil_pcarbon(1) = 84 ! 84
      nsoil_pcarbon(2) = nsoil_pcarbon(1) + 84 ! 84
      nsoil_pcarbon(3) = nsoil_pcarbon(2) + 84 ! 57

      fname='clsm/catchment.def'
!
! Reading number of cathment-tiles from catchment.def file
! 
      open (10,file=fname,status='old',action='read',form='formatted')
      read(10,*) maxcat
      
      close (10,status='keep')

      fname =trim(c_data)//'SOIL-DATA/GSWP2_soildepth_H11V13.nc'
      status = NF_OPEN(trim(fname),NF_NOWRITE, ncid); VERIFY_(STATUS)
      status = NF_GET_att_INT(ncid,NF_GLOBAL,'i_ind_offset_LL',iLL); VERIFY_(STATUS)
      status = NF_GET_att_INT(ncid,NF_GLOBAL,'j_ind_offset_LL',jLL); VERIFY_(STATUS)
      status = NF_GET_att_INT(ncid,NF_GLOBAL,'N_lon_global',i_highd); VERIFY_(STATUS)
      status = NF_GET_att_INT(ncid,NF_GLOBAL,'N_lat_global',j_highd); VERIFY_(STATUS)
      status = NF_INQ_DIM (ncid,1,string, nc_10); VERIFY_(STATUS)
      status = NF_INQ_DIM (ncid,2,string, nr_10); VERIFY_(STATUS)
      status = NF_CLOSE(ncid); VERIFY_(STATUS)

      allocate(soildepth(1:maxcat))
      allocate(soil_high(1:i_highd,1:j_highd))  
      allocate(count_soil(1:maxcat))  
      allocate(tile_id(1:nx,1:ny))
      allocate(net_data1 (1:nc_10,1:nr_10))

      fname=trim(gfiler)//'.rst'
!          
! Reading tile-id raster file
!
      open (10,file=fname,status='old',action='read',  &
           form='unformatted',convert='little_endian')
      
      do j=1,ny
         read(10)tile_id(:,j)
      end do       

      close (10,status='keep')
!
! reading soil depth data
!
      soil_high = -9999
      do jx = 1,18
      	 do ix = 1,36
	    write (vv,'(i2.2)')jx
	    write (hh,'(i2.2)')ix 
	    fname = trim(c_data)//'SOIL-DATA/GSWP2_soildepth_H'//hh//'V'//vv//'.nc'
            status = NF_OPEN(trim(fname),NF_NOWRITE, ncid)
	    if(status == 0) then
		status = NF_GET_att_INT  (ncid,NF_GLOBAL,'i_ind_offset_LL',iLL); VERIFY_(STATUS)
                status = NF_GET_att_INT  (ncid,NF_GLOBAL,'j_ind_offset_LL',jLL); VERIFY_(STATUS)
		status = NF_GET_att_INT  (ncid,4,'UNDEF',d_undef); VERIFY_(STATUS)
		status = NF_GET_att_REAL (ncid,4,'ScaleFactor',sf); VERIFY_(STATUS)
		status = NF_GET_VARA_INT (ncid, 4,(/1,1/),(/nc_10,nr_10/),net_data1); VERIFY_(STATUS)

		do j = jLL,jLL + nr_10 -1 
                   do i = iLL, iLL + nc_10 -1 
                    if(net_data1(i-iLL +1 ,j - jLL +1) /= d_undef) &
		            soil_high(i,j) = net_data1(i-iLL +1 ,j - jLL +1)
                   enddo
                enddo
	        status = NF_CLOSE(ncid)
	    endif
	 end do 
      end do

      deallocate (net_data1)

! Regridding 

      nx_adj = nx
      ny_adj = ny

      regrid = nx/=i_highd .or. ny/=j_highd

      if(regrid) then
          if(nx > i_highd) then 
               allocate(raster(nx,ny),stat=STATUS); VERIFY_(STATUS)
               call RegridRaster2(soil_high,raster)	
	       iRaster => tile_id
	       if(ny < j_highd) then
	          print *,'nx > i_highd and ny < j_highd'
		  stop 
	       endif
	  else
               if( .not.associated(iraster) ) then 
                   allocate(iraster(i_highd,j_highd),stat=STATUS); VERIFY_(STATUS)
               endif
               call RegridRaster(tile_id,iraster)	
               raster  => soil_high
	       nx_adj = i_highd
               ny_adj = j_highd

	       if(ny > j_highd) then
	          print *,'nx < i_highd and ny > j_highd'
		  stop 
	       endif
          endif	    	              
      else
          raster  => soil_high
	  iRaster => tile_id
      end if
      
! Interpolation or aggregation on to catchment-tiles

      soildepth =0.
      count_soil = 0.
 
      do j=1,ny_adj
         do i=1,nx_adj
            if((iRaster(i,j).gt.0).and.(iRaster(i,j).le.maxcat)) then
               if ((raster(i,j).gt.0)) then
                  soildepth(iRaster(i,j)) = &
                       soildepth(iRaster(i,j)) + sf*raster(i,j)
                  count_soil(iRaster(i,j)) = &
                       count_soil(iRaster(i,j)) + 1. 
               endif
            endif
         end do
      end do

      DO n =1,maxcat
	 	if(count_soil(n)/=0.) soildepth(n)=soildepth(n)/count_soil(n)	
	          soildepth(n) = max(soildepth(n),1334.)
!                  soildepth(n) = soildepth(n) + 2000.
!                  soildepth(n) = min(soildepth(n),8000.) 
       END DO

      deallocate (SOIL_HIGH)
      deallocate (count_soil)
      deallocate (iRaster)
      NULLIFY(Raster)
!
! Reading NGDC-HWSD-STATSGO merged Soil Properties
!
      fname =trim(c_data)//'SOIL-DATA/SoilProperties_H11V13.nc'
      status = NF_OPEN(trim(fname),NF_NOWRITE, ncid); VERIFY_(STATUS)
      status = NF_GET_att_INT(ncid,NF_GLOBAL,'i_ind_offset_LL',iLL); VERIFY_(STATUS)
      status = NF_GET_att_INT(ncid,NF_GLOBAL,'j_ind_offset_LL',jLL); VERIFY_(STATUS)
      status = NF_GET_att_INT(ncid,NF_GLOBAL,'N_lon_global',i_highd); VERIFY_(STATUS)
      status = NF_GET_att_INT(ncid,NF_GLOBAL,'N_lat_global',j_highd); VERIFY_(STATUS)
      status = NF_INQ_DIM (ncid,1,string, nc_10); VERIFY_(STATUS)
      status = NF_INQ_DIM (ncid,2,string, nr_10); VERIFY_(STATUS)
      status = NF_CLOSE(ncid)

      regrid = nx/=i_highd .or. ny/=j_highd
      allocate(net_data1 (1:nc_10,1:nr_10))
      allocate(net_data2 (1:nc_10,1:nr_10))
      allocate(net_data3 (1:nc_10,1:nr_10))
      allocate(net_data4 (1:nc_10,1:nr_10))
      allocate(net_data5 (1:nc_10,1:nr_10))
      allocate(net_data6 (1:nc_10,1:nr_10))
      allocate(net_data7 (1:nc_10,1:nr_10))

      allocate(sand_top (1:i_highd,1:j_highd))  
      allocate(clay_top (1:i_highd,1:j_highd))  
      allocate(oc_top   (1:i_highd,1:j_highd))  
      allocate(sand_sub (1:i_highd,1:j_highd))  
      allocate(clay_sub (1:i_highd,1:j_highd))  
      allocate(oc_sub   (1:i_highd,1:j_highd))  
      allocate(grav_grid(1:i_highd,1:j_highd))   

      sand_top = -9999.
      clay_top = -9999.
      oc_top   = -9999.
      sand_sub = -9999.
      clay_sub = -9999.
      oc_sub   = -9999.
      grav_grid= -9999.

      do jx = 1,18
      	 do ix = 1,36
	    write (vv,'(i2.2)')jx
	    write (hh,'(i2.2)')ix 
	    fname = trim(c_data)//'SOIL-DATA/SoilProperties_H'//hh//'V'//vv//'.nc'
            status = NF_OPEN(trim(fname),NF_NOWRITE, ncid)
	    if(status == 0) then
		status = NF_GET_att_INT  (ncid, NF_GLOBAL,'i_ind_offset_LL',iLL); VERIFY_(STATUS)
                status = NF_GET_att_INT  (ncid, NF_GLOBAL,'j_ind_offset_LL',jLL); VERIFY_(STATUS)
		status = NF_GET_att_INT  (ncid, 4,'UNDEF',d_undef); VERIFY_(STATUS)
		status = NF_GET_att_REAL (ncid, 4,'ScaleFactor',sf); VERIFY_(STATUS)
		status = NF_GET_VARA_INT (ncid, 4,(/1,1/),(/nc_10,nr_10/),net_data1); VERIFY_(STATUS)
		status = NF_GET_VARA_INT (ncid, 5,(/1,1/),(/nc_10,nr_10/),net_data2); VERIFY_(STATUS)
		status = NF_GET_VARA_INT (ncid, 6,(/1,1/),(/nc_10,nr_10/),net_data3); VERIFY_(STATUS)
		status = NF_GET_VARA_INT (ncid, 7,(/1,1/),(/nc_10,nr_10/),net_data4); VERIFY_(STATUS)
		status = NF_GET_VARA_INT (ncid, 8,(/1,1/),(/nc_10,nr_10/),net_data5); VERIFY_(STATUS)
		status = NF_GET_VARA_INT (ncid, 9,(/1,1/),(/nc_10,nr_10/),net_data6); VERIFY_(STATUS)
                status = NF_GET_VARA_INT (ncid,10,(/1,1/),(/nc_10,nr_10/),net_data7); VERIFY_(STATUS)
		do j = jLL,jLL + nr_10 -1 
                   do i = iLL, iLL + nc_10 -1 
                    if(net_data1(i-iLL +1 ,j - jLL +1) /= d_undef) &
		            clay_top(i,j) = net_data1(i-iLL +1 ,j - jLL +1)
                    if(net_data2(i-iLL +1 ,j - jLL +1) /= d_undef) &
		            sand_top(i,j) = net_data2(i-iLL +1 ,j - jLL +1)
                    if(net_data3(i-iLL +1 ,j - jLL +1) /= d_undef) &
		            oc_top  (i,j) = net_data3(i-iLL +1 ,j - jLL +1)
                    if(net_data4(i-iLL +1 ,j - jLL +1) /= d_undef) &
		            clay_sub(i,j) = net_data4(i-iLL +1 ,j - jLL +1)
                    if(net_data5(i-iLL +1 ,j - jLL +1) /= d_undef) &
		            sand_sub(i,j) = net_data5(i-iLL +1 ,j - jLL +1)
                    if(net_data6(i-iLL +1 ,j - jLL +1) /= d_undef) &
		            oc_sub  (i,j) = net_data6(i-iLL +1 ,j - jLL +1)
                    if(net_data7(i-iLL +1 ,j - jLL +1) /= d_undef) &
		           grav_grid(i,j) = net_data7(i-iLL +1 ,j - jLL +1)
                    enddo
                enddo
	    status = NF_CLOSE(ncid)
	    endif
	 end do 
      end do

      deallocate (net_data1)
      deallocate (net_data2)
      deallocate (net_data3)
      deallocate (net_data4)
      deallocate (net_data5)
      deallocate (net_data6)
      deallocate (net_data7)

! now regridding

      nx_adj = nx
      ny_adj = ny

      regrid = nx/=i_highd .or. ny/=j_highd

      if(regrid) then
          if(nx > i_highd) then 
               allocate(raster1(nx,ny),stat=STATUS); VERIFY_(STATUS)
               call RegridRaster2(clay_top,raster1)	

               allocate(raster2(nx,ny),stat=STATUS); VERIFY_(STATUS)
               call RegridRaster2(sand_top,raster2)	

               allocate(raster3(nx,ny),stat=STATUS); VERIFY_(STATUS)
               call RegridRaster2(oc_top,  raster3)	

               allocate(raster4(nx,ny),stat=STATUS); VERIFY_(STATUS)
               call RegridRaster2(clay_sub,raster4)	

               allocate(raster5(nx,ny),stat=STATUS); VERIFY_(STATUS)
               call RegridRaster2(sand_sub,raster5)	

               allocate(raster6(nx,ny),stat=STATUS); VERIFY_(STATUS)
               call RegridRaster2(oc_sub,  raster6)	

               allocate(raster (nx,ny),stat=STATUS); VERIFY_(STATUS)
               call RegridRaster2(grav_grid,raster)

	       iRaster => tile_id

	       if(ny < j_highd) then
	          print *,'nx > i_highd and ny < j_highd'
		  stop 
	       endif
	  else
	       nx_adj = i_highd
               ny_adj = j_highd
               if( .not.associated(iraster) ) then
                  allocate(iRaster(i_highd,j_highd),stat=STATUS); VERIFY_(STATUS)
               endif
               call RegridRaster(tile_id,iRaster)	

               raster1 => clay_top
               raster2 => sand_top
               raster3 => oc_top
               raster4 => clay_sub
               raster5 => sand_sub
               raster6 => oc_sub
               raster  => grav_grid

	       if(ny > j_highd) then
	          print *,'nx < i_highd and ny > j_highd'
		  stop 
	       endif
          endif	    	              
      else
	  iRaster => tile_id
          raster1 => clay_top
          raster2 => sand_top
          raster3 => oc_top
          raster4 => clay_sub
          raster5 => sand_sub
          raster6 => oc_sub
          raster  => grav_grid
      end if

! Deallocate large arrays

      allocate(land_pixels(1:size(iRaster,1),1:size(iRaster,2)))
      land_pixels = (iRaster >=1).and.(iRaster<=maxcat)
      i1 = count(land_pixels)   
      deallocate (land_pixels) 

      allocate (tileid_vec(1:i1))
      allocate (data_vec1 (1:i1))
      allocate (data_vec2 (1:i1))
      allocate (data_vec3 (1:i1))
      allocate (data_vec4 (1:i1))
      allocate (data_vec5 (1:i1))
      allocate (data_vec6 (1:i1))
      allocate (grav_vec  (1:maxcat))
      allocate (soc_vec   (1:maxcat))
      allocate (poc_vec   (1:maxcat))
      allocate (ncells_top  (1:maxcat))
      allocate (ncells_top_pro  (1:maxcat))
      allocate (ncells_sub_pro  (1:maxcat))
      allocate(count_soil(1:maxcat))  
      count_soil = 0.
      grav_vec   = 0.
      soc_vec    = 0.
      poc_vec    = 0.
      ncells_top = 0.
      ncells_top_pro = 0.
      ncells_sub_pro = 0.

      n =1
      do j=1,ny_adj
         do i=1,nx_adj
            if((iRaster(i,j).ge.1).and.(iRaster(i,j).le.maxcat)) then

	       tileid_vec (n) =  iRaster(i,j)
	       data_vec1  (n) =  Raster1(i,j)
	       data_vec2  (n) =  Raster2(i,j)
	       data_vec3  (n) =  Raster3(i,j)
	       data_vec4  (n) =  Raster4(i,j)
	       data_vec5  (n) =  Raster5(i,j)
	       data_vec6  (n) =  Raster6(i,j)

               if ((raster(i,j).gt.0)) then 
                  grav_vec(iRaster(i,j)) = &
                       grav_vec(iRaster(i,j)) + sf*raster(i,j)
                  count_soil(iRaster(i,j)) = &
                       count_soil(iRaster(i,j)) + 1. 
               endif
	       n = n + 1
	     endif
	  end do
      end do    

      DO n =1,maxcat
	 	if(count_soil(n)/=0.) grav_vec(n)=grav_vec(n)/count_soil(n)	
      END DO

      deallocate (grav_grid)
      deallocate (count_soil)
      deallocate (iRaster)
      NULLIFY(Raster)

      NULLIFY(Raster1,Raster2,Raster3,Raster4,Raster5,Raster6)
      deallocate (clay_top,sand_top,oc_top,clay_sub,sand_sub,oc_sub) 
      deallocate (tile_id)

      allocate (arrayA    (1:i1))
      allocate (arrayB    (1:i1))

      arrayA = tileid_vec
      arrayB = data_vec1
      call MAPL_Sort (arrayA, arrayB)
      data_vec1 = arrayB

      arrayA = tileid_vec
      arrayB = data_vec2
      call MAPL_Sort (arrayA, arrayB)
      data_vec2 = arrayB

      arrayA = tileid_vec
      arrayB = data_vec3
      call MAPL_Sort (arrayA, arrayB)
      data_vec3 = arrayB

      arrayA = tileid_vec
      arrayB = data_vec4
      call MAPL_Sort (arrayA, arrayB)
      data_vec4 = arrayB

      arrayA = tileid_vec
      arrayB = data_vec5
      call MAPL_Sort (arrayA, arrayB)
      data_vec5 = arrayB

      arrayA = tileid_vec
      arrayB = data_vec6
      call MAPL_Sort (arrayA, arrayB)
      data_vec6 = arrayB
      tileid_vec= arrayA
      deallocate (arrayA, arrayB)
!
! Reading Woesten Soil Parameters and CLSM tau parameters
!
	allocate(a_sand  (1:n_SoilClasses))
	allocate(a_clay  (1:n_SoilClasses))
	allocate(a_silt  (1:n_SoilClasses))
	allocate(a_oc    (1:n_SoilClasses))
	allocate(a_bee   (1:n_SoilClasses))
	allocate(a_psis  (1:n_SoilClasses))
	allocate(a_poros (1:n_SoilClasses))
	allocate(a_wp    (1:n_SoilClasses))
	allocate(a_aksat (1:n_SoilClasses))
	allocate(atau    (1:n_SoilClasses))
	allocate(btau    (1:n_SoilClasses))
	allocate(atau_2cm(1:n_SoilClasses))
	allocate(btau_2cm(1:n_SoilClasses))
        allocate(a_wpsurf(1:n_SoilClasses))
        allocate(a_porosurf(1:n_SoilClasses))

      fname = trim(c_data)//'SoilClasses-SoilHyd-TauParam.dat'
      table_map = 0
      open (11, file=trim(fname), form='formatted',status='old', &
           action = 'read')
      read (11,'(a)')fout
      do n =1,n_SoilClasses 
      	 read (11,'(4f7.3,4f8.4,e13.5,2f12.7,2f8.4,4f12.7)')a_sand(n),a_clay(n),a_silt(n),a_oc(n),a_bee(n),a_psis(n), &
              a_poros(n),a_wp(n),a_aksat(n),atau(n),btau(n),a_wpsurf(n),a_porosurf(n),atau_2cm(n),btau_2cm(n)

	 min_percs%clay_perc = a_clay(n)
	 min_percs%silt_perc = a_silt(n)
	 min_percs%sand_perc = a_sand(n)
	 if(n <= nsoil_pcarbon(1))                              table_map(soil_class (min_percs),1) = n  
	 if((n > nsoil_pcarbon(1)).and.(n <= nsoil_pcarbon(2))) table_map(soil_class (min_percs),2) = n  
         if((n > nsoil_pcarbon(2)).and.(n <= nsoil_pcarbon(3))) table_map(soil_class (min_percs),3) = n 

      end do
      close (11,status='keep') 
!
!  When Woesten Soil Parameters are not available for a particular Soil Class
!  ,as assumed by tiny triangles in HWSD soil triangle, Woesten Soil
!  parameters from the nearest available tiny triangle will be substituted.
!	     	  
      do n =1,10
	  do k=1,n*2 -1

     	     min_percs%clay_perc = 100. -((n-1)*10 + 5)
	     min_percs%sand_perc = 100. -  min_percs%clay_perc -2.-(k-1)*5.
	     min_percs%silt_perc = 100. -  min_percs%clay_perc - min_percs%sand_perc

	     i = soil_class (min_percs)

	     if(table_map (i,1)== 0) then   
	       j = center_pix (a_clay(1:nsoil_pcarbon(1)),a_sand(1:nsoil_pcarbon(1)),                       &
	           min_percs%clay_perc,min_percs%sand_perc,min_percs%silt_perc,.true.) 

	 	   min_percs%clay_perc = a_clay(j)
	 	   min_percs%silt_perc = a_silt(j)
	 	   min_percs%sand_perc = a_sand(j)		   
	       	   table_map (i,1)= table_map (soil_class (min_percs),1)
	     endif

     	     min_percs%clay_perc = 100. -((n-1)*10 + 5)
	     min_percs%sand_perc = 100. -  min_percs%clay_perc -2.-(k-1)*5.
	     min_percs%silt_perc = 100. -  min_percs%clay_perc - min_percs%sand_perc

	     if(table_map (i,2)== 0) then   
	       j = center_pix(a_clay(nsoil_pcarbon(1)+1 : nsoil_pcarbon(2)),         &
	                      a_sand(nsoil_pcarbon(1)+1 : nsoil_pcarbon(2)),         &
	           min_percs%clay_perc,min_percs%sand_perc,min_percs%silt_perc,.true.) 
	 	   min_percs%clay_perc = a_clay(j + nsoil_pcarbon(1))
	 	   min_percs%silt_perc = a_silt(j + nsoil_pcarbon(1))
	 	   min_percs%sand_perc = a_sand(j + nsoil_pcarbon(1))		   
	       	   table_map (i,2)= table_map (soil_class (min_percs),2)	         
	     endif

     	     min_percs%clay_perc = 100. -((n-1)*10 + 5)
	     min_percs%sand_perc = 100. -  min_percs%clay_perc -2.-(k-1)*5.
	     min_percs%silt_perc = 100. -  min_percs%clay_perc - min_percs%sand_perc

	     if(table_map (i,3)== 0) then   
	       j = center_pix (a_clay(nsoil_pcarbon(2)+1 : nsoil_pcarbon(3)),         &
                               a_sand(nsoil_pcarbon(2)+1 : nsoil_pcarbon(3)),         &
	           min_percs%clay_perc,min_percs%sand_perc,min_percs%silt_perc,.true.) 
	 	   min_percs%clay_perc = a_clay(j + nsoil_pcarbon(2))
	 	   min_percs%silt_perc = a_silt(j + nsoil_pcarbon(2))
	 	   min_percs%sand_perc = a_sand(j + nsoil_pcarbon(2))		   
	       	   table_map (i,3)= table_map (soil_class (min_percs),3)	         	         
	     endif
	  end do         	  
      end do
!
! Now deriving soil types based on NGDC-HWSD-STATSGO merged soil property maps
!
      allocate (soil_class_top (1:maxcat))
      allocate (soil_class_com (1:maxcat))
      soil_class_top =0
      soil_class_com =0
      
      allocate(low_ind(n_threads))
      allocate(upp_ind(n_threads))
      low_ind(1)         = 1
      upp_ind(n_threads) = maxcat

      if (running_omp)  then
       do i=1,n_threads-1  
         upp_ind(i)   = low_ind(i) + (maxcat/n_threads) - 1 
         low_ind(i+1) = upp_ind(i) + 1
      end do 
     end if

!$OMP PARALLELDO DEFAULT(NONE)                          &
!$OMP SHARED( n_threads, low_ind, upp_ind, tileid_vec,  &
!$OMP         sf,data_vec1,data_vec2,data_vec3,         &
!$OMP         data_vec4,data_vec5,data_vec6,cF_lim,     &
!$OMP         table_map,soil_class_top,soil_class_com,  &
!$OMP         soc_vec,poc_vec,ncells_top,ncells_top_pro,&
!$OMP         ncells_sub_pro)    &
!$OMP PRIVATE(n,i,j,k,icount,t_count,i1,i2,ss_clay,     &
!$OMP         ss_sand,ss_clay_all,ss_sand_all,          &
!$OMP         ss_oc_all,cFamily,factor,o_cl,o_clp,ktop, &
!$OMP         min_percs, fac_count, write_file)

    DO t_count = 1,n_threads
      DO n = low_ind(t_count),upp_ind(t_count)

	write_file = .false.

!	if (n==171010)  write_file = .true.
!	if (n==178734)  write_file = .true.
!	if (n==19304)   write_file = .true.
!	if (n==181132)  write_file = .true.
!	if (n==181940)  write_file = .true.
!	if (n==182347)  write_file = .true.

        if(n==low_ind(t_count)) then
             icount = 1
             do k=1,low_ind(t_count) - 1
                do while (tileid_vec(icount)== k)
                   icount = icount + 1
                end do
             end do
        endif
             
        i1 = icount 
        
	loop: do while (tileid_vec(icount)== n)
	   if(icount <= size(tileid_vec,1)) icount = icount + 1
           if(icount > size(tileid_vec,1)) exit loop
	end do loop 

	i2 = icount -1
 	i = i2 - i1 + 1

        allocate(ss_clay    (1:2*i))
        allocate(ss_sand    (1:2*i))
        allocate(ss_clay_all(1:2*i))
        allocate(ss_sand_all(1:2*i))
        allocate(ss_oc_all  (1:2*i))
	  
        ss_clay    = 0    
        ss_sand    = 0	
        ss_clay_all= 0
        ss_sand_all= 0
        ss_oc_all  = 0

	ss_clay_all (1:i)     = data_vec1(i1:i2)
	ss_sand_all (1:i)     = data_vec2(i1:i2)
	ss_oc_all   (1:i)     = data_vec3(i1:i2)	
	ss_clay_all (1+i:2*i) = data_vec4(i1:i2) 
	ss_sand_all (1+i:2*i) = data_vec5(i1:i2)
	ss_oc_all   (1+i:2*i) = data_vec6(i1:i2)	

	cFamily = 0.

	do j=1,i
	   if(j <= i) factor = 1.
	   if((ss_oc_all(j)*sf >=  cF_lim(1)).and. (ss_oc_all(j)*sf < cF_lim(2))) cFamily(1) = cFamily(1) + factor
	   if((ss_oc_all(j)*sf >=  cF_lim(2)).and. (ss_oc_all(j)*sf < cF_lim(3))) cFamily(2) = cFamily(2) + factor
	   if((ss_oc_all(j)*sf >=  cF_lim(3)).and. (ss_oc_all(j)*sf < cF_lim(4))) cFamily(3) = cFamily(3) + factor
	   if((ss_oc_all(j)*sf >=  cF_lim(4) ))                                   cFamily(4) = cFamily(4) + factor
	end do

	if (sum(cFamily) == 0.) o_cl  = 1
	if (sum(cFamily)  > 0.) o_cl  = maxloc(cFamily, dim = 1)

	cFamily = 0.

	do j=1,2*i
	   if(j <= i) factor = 1.
	   if(j  > i) factor = 2.33
	   if((ss_oc_all(j)*sf >=  cF_lim(1)).and. (ss_oc_all(j)*sf < cF_lim(2))) cFamily(1) = cFamily(1) + factor
	   if((ss_oc_all(j)*sf >=  cF_lim(2)).and. (ss_oc_all(j)*sf < cF_lim(3))) cFamily(2) = cFamily(2) + factor
	   if((ss_oc_all(j)*sf >=  cF_lim(3)).and. (ss_oc_all(j)*sf < cF_lim(4))) cFamily(3) = cFamily(3) + factor
	   if((ss_oc_all(j)*sf >=  cF_lim(4) ))                                   cFamily(4) = cFamily(4) + factor
	end do

	if (sum(cFamily) == 0.) o_clp = 1
	if (sum(cFamily)  > 0.) o_clp = maxloc(cFamily, dim = 1)

        if(o_cl == 4) then 
           soil_class_top(n) = n_SoilClasses
	   ktop = 0
	   do j=1,i
	     if(ss_oc_all(j)*sf >= cF_lim(4)) then
	        soc_vec (n) = soc_vec(n) + ss_oc_all(j)*sf
		ktop = ktop + 1
             endif
	   end do
	   if(ktop.ne.0) soc_vec (n)   = soc_vec(n)/ktop
	   ncells_top(n) = 100.*float(ktop)/float(i)
        else 
            k = 1
	    ktop = 1

	    do j=1,i
	        if((ss_oc_all(j)*sf >= cF_lim(o_cl)).and.(ss_oc_all(j)*sf < cF_lim(o_cl + 1))) then 
                   if((ss_clay_all(j)*sf >= 0.).and.(ss_sand_all(j)*sf >= 0.)) then   
    			   ss_clay (k) = ss_clay_all(j)
			   ss_sand (k) = ss_sand_all(j)
                           if((ss_clay (k) + ss_sand (k)) > 9999) then
                              if(ss_clay (k) >= ss_sand (k)) then
                                 ss_sand (k) = 10000 - ss_clay (k)
                               else
                                 ss_clay (k) = 10000 - ss_sand (k)
                               endif
                            endif
			   soc_vec (n) = soc_vec(n) + ss_oc_all(j)*sf
                           k = k + 1
                           ktop = ktop + 1
		   endif
                endif	
	    end do
	    
	    k = k - 1
	    ktop = ktop -1
	    if(ktop.ne.0) soc_vec (n) = soc_vec(n)/ktop
	    ncells_top(n) = 100.*float(ktop)/float(i)
            if (write_file) write(80+n,*)ktop,o_cl
            if(ktop > 0) then 
               if (write_file) write (80+n,*)ss_clay(1:ktop)
               if (write_file) write (80+n,*)ss_sand(1:ktop)
            endif
  	    j = center_pix_int0(sf, ktop,ktop, ss_clay(1:ktop),ss_sand(1:ktop))
            if (write_file) write(80+n,*)j
!  	    j = center_pix_int (sf, ktop,ktop, ss_clay(1:ktop),ss_sand(1:ktop),             &
!	           	       min_percs%clay_perc,min_percs%sand_perc,min_percs%silt_perc,.false.) 
            if (write_file) write(80+n,*)j
            if(j >=1) then 
	 	               min_percs%clay_perc = ss_clay(j)*sf
	 	               min_percs%sand_perc = ss_sand(j)*sf
	 	               min_percs%silt_perc = 100. - ss_clay(j)*sf - ss_sand(j)*sf
                  soil_class_top (n) = table_map(soil_class (min_percs),o_cl)   
            else
                  soil_class_top (n) = -9999
            endif
        endif
            if (write_file) write(80+n,*)soil_class_top (n) 
        if(o_clp == 4) then 
           soil_class_com(n) = n_SoilClasses
	   fac_count = 0.
	   k =0
	   ktop =0
	   do j=1,2*i
	     if(ss_oc_all(j)*sf >= cF_lim(4)) then
                 if(j <= i) factor = 1.
	         if(j  > i) factor = 2.33
		 if(j  > i) k = k + 1
		 if(j <= i) ktop = ktop + 1
	         poc_vec (n) = poc_vec(n) + ss_oc_all(j)*sf*factor
		 fac_count = fac_count + factor
             endif
	   end do
	   if(fac_count.ne.0) poc_vec (n) = poc_vec (n)/fac_count
           ncells_sub_pro(n) = 100.*float(k)/float(i)
           ncells_top_pro(n) = 100.*float(ktop)/float(i)
        else
            k = 1
	    ktop = 1

            ss_clay=0
            ss_sand=0
	    fac_count = 0.

	    do j=1,2*i
	        if((ss_oc_all(j)*sf >=  cF_lim(o_clp)).and.(ss_oc_all(j)*sf < cF_lim(o_clp + 1))) then 
                   if((ss_clay_all(j)*sf >= 0.).and.(ss_sand_all(j)*sf >= 0.)) then 
  	              if(j <= i) factor = 1.
	              if(j  > i) factor = 2.33
	              poc_vec (n) = poc_vec(n) + ss_oc_all(j)*sf*factor
		      fac_count = fac_count + factor
		      if(j <= i) then
    			   ss_clay (k) = ss_clay_all(j)
			   ss_sand (k) = ss_sand_all(j)
                           if((ss_clay (k) + ss_sand (k)) > 9999) then
                              if(ss_clay (k) >= ss_sand (k)) then
                                 ss_sand (k) = 10000 - ss_clay (k)
                               else
                                 ss_clay (k) = 10000 - ss_sand (k)
                               endif
                            endif
                           k = k + 1
			   ktop = ktop + 1
		       else
    			   ss_clay (k) = ss_clay_all(j)
			   ss_sand (k) = ss_sand_all(j)
                           if((ss_clay (k) + ss_sand (k)) > 9999) then
                              if(ss_clay (k) >= ss_sand (k)) then
                                 ss_sand (k) = 10000 - ss_clay (k)
                               else
                                 ss_clay (k) = 10000 - ss_sand (k)
                               endif
                            endif
                           k = k + 1                         
		       endif   
		   endif
                endif	
	    end do
	    
	    k = k - 1
	    ktop = ktop -1
	    if(fac_count.ne.0) poc_vec (n) = poc_vec(n)/fac_count
	    ncells_top_pro(n) = 100.*float(ktop)/float(i)
	    ncells_sub_pro(n) = 100.*float(k-ktop)/float(i)

            if (write_file) write (80+n,*)ktop,k,o_cl
            if (write_file) write (80+n,*)ss_clay(1:k)
            if (write_file) write (80+n,*)ss_sand(1:k)
	    j = center_pix_int0 (sf, ktop,k, ss_clay(1:k),ss_sand(1:k))
            if (write_file) write(80+n,*)j
!	    j = center_pix_int (sf, ktop,k, ss_clay(1:k),ss_sand(1:k),  &
!	           min_percs%clay_perc,min_percs%sand_perc,min_percs%silt_perc,.false.) 

            if (write_file) write(80+n,*) j
            if(j >=1) then 
	 	   min_percs%clay_perc = ss_clay(j)*sf
	 	   min_percs%sand_perc = ss_sand(j)*sf
	 	   min_percs%silt_perc = 100. - ss_clay(j)*sf - ss_sand(j)*sf
	    soil_class_com (n) = table_map(soil_class (min_percs),o_clp)  
            else
             soil_class_com (n) =    soil_class_com (n-1)
            endif
            if (write_file) write(80+n,*) soil_class_com (n) 
            if (write_file) close(80+n)          
        endif

        if(soil_class_top (n) == -9999) soil_class_top (n) = soil_class_com (n) 
!        if ((n==155519).or.(n==219302)) then
!           write (95,*) ktop,k,i
!           write (95,*) ss_oc_all
!           write (95,*) ss_clay
!           write (95,*) ss_sand
!           close (95,status='keep')        
!        endif
	deallocate (ss_clay,ss_sand,ss_clay_all,ss_sand_all,ss_oc_all)
      END DO
      END DO
!$OMP ENDPARALLELDO
            
      fname='clsm/catchment.def'
      open (10,file=fname,status='old',action='read',form='formatted')
      read(10,*) maxcat
      fname ='clsm/soil_param.first'
      open (11,file=trim(fname),form='formatted',status='unknown',action = 'write')

      fname ='clsm/tau_param.dat'
      open (12,file=trim(fname),form='formatted',status='unknown',action = 'write')
 !     fname ='clsm/tau_param_24h.dat'
 !     open (14,file=trim(fname),form='formatted',status='unknown',action = 'write')
 !     fname ='clsm/tau_param_48h.dat'
 !     open (15,file=trim(fname),form='formatted',status='unknown',action = 'write')

      fname ='clsm/mosaic_veg_typs_fracs'
      open (13,file=trim(fname),form='formatted',status='old',action = 'read')

      do n = 1, maxcat

      	 read (10,*) tindex,pfafindex
         read (13,*) tindex,pfafindex,vtype

         fac_surf = soil_class_top(n)
	 fac      = soil_class_com(n)

         wp_wetness = a_wp(fac)     /a_poros(fac)
         write (11,'(i8,i8,i4,i4,3f8.4,f12.8,f7.4,f10.4,3f7.3,4f7.3,2f10.4)')tindex,pfafindex,      &
               soil_class_top(n),soil_class_com(n),a_bee(fac),a_psis(fac),a_poros(fac),&
               a_aksat(fac)/exp(-1.0*zks*gnu),wp_wetness,soildepth(n),                 &
               grav_vec(n),soc_vec(n),poc_vec(n), &
               a_sand(fac_surf),a_clay(fac_surf),a_sand(fac),a_clay(fac), &
	       a_wpsurf(fac_surf)/a_porosurf(fac_surf),a_porosurf(fac_surf)

!         if (vtype <=3 ) then
!            wp_wetness = a_wp(fac)     /a_poros(fac)
! 	    write (11,'(i8,i8,i4,i4,3f8.4,f12.8,f7.4,f10.4,3f7.3,3f8.2,4f7.3,2f10.4)')tindex,pfafindex,      &
!               soil_class_top(n),soil_class_com(n),a_bee(fac),a_psis(fac),a_poros(fac),&
!               a_aksat(fac)/exp(-1.0*zks*gnu),wp_wetness,soildepth(n),                 &
!               grav_vec(n),soc_vec(n),poc_vec(n),ncells_top(n), ncells_top_pro(n),ncells_sub_pro(n),      &
!               a_sand(fac_surf),a_clay(fac_surf),a_sand(fac),a_clay(fac), &
!	       a_wpsurf(fac_surf),a_porosurf(fac_surf)
!            else
!            wp_wetness = a_wpsurf(fac_surf) /a_porosurf(fac_surf)
!            write (11,'(i8,i8,i4,i4,3f8.4,f12.8,f7.4,f10.4,3f7.3,3f8.2,4f7.3,2f10.4)')tindex,pfafindex,      &
!               soil_class_top(n),soil_class_top(n),a_bee(fac_surf),a_psis(fac_surf),a_poros(fac_surf),&
!               a_aksat(fac_surf)/exp(-1.0*zks*gnu),wp_wetness,soildepth(n),                 &
!               grav_vec(n),soc_vec(n),poc_vec(n),ncells_top(n), ncells_top_pro(n),ncells_sub_pro(n),      &
!               a_sand(fac_surf),a_clay(fac_surf),a_sand(fac),a_clay(fac), &
!	       a_wpsurf(fac_surf),a_porosurf(fac_surf)
!         endif


	       	    
         write (12,'(i8,i8,4f10.7)')tindex,pfafindex, &
	       atau_2cm(fac_surf),btau_2cm(fac_surf),atau(fac_surf),btau(fac_surf) 
 
 !        write (14,'(i8,i8,6f10.7)')tindex,pfafindex, &
 !              -9999.,-9999.,24.*atau_day(fac_surf),btau_day(fac_surf)
 !               24.*atau_day(fac_surf),btau_day(fac_surf),24.*atau_day(fac_surf),btau_day(fac_surf)
 !        write (15,'(i8,i8,6f10.7)')tindex,pfafindex, &
 !              48.*atau_2day(fac_surf),btau_2day(fac_surf),48.*atau_2day(fac_surf),btau_2day(fac_surf)
	    	    
      end do
      write (11,'(a)')'                    '
      write (11,'(a)')'FMT=i8,i8,i4,i4,3f8.4,f12.8,f7.4,f10.4,3f7.3,4f7.3,2f10.4'
      write (11,'(a)')'TileIndex PfafID SoilClassTop SoilClassProfile BEE PSIS POROS Ks_at_SURF WPWET SoilDepth %Grav %OCTop %OCProf %Sand_top %Clay_top %Sand_prof %Clay_prof WPWET_SURF POROS_SURF'
      close (10, status = 'keep')	            
      close (11, status = 'keep')	            
      close (12, status = 'keep')	            
      close (13, status = 'keep')
!      close (14, status = 'keep')
!      close (15, status = 'keep')

      deallocate (data_vec1, data_vec2,data_vec3, data_vec4,data_vec5, data_vec6)
      deallocate (tileid_vec)
      deallocate (a_sand,a_clay,a_silt,a_oc,a_bee,a_psis,       &
            a_poros,a_wp,a_aksat,atau,btau,a_wpsurf,a_porosurf, &
            atau_2cm,btau_2cm)
      deallocate (soildepth, grav_vec,soc_vec,poc_vec,&
             ncells_top,ncells_top_pro,ncells_sub_pro,soil_class_top,soil_class_com)

  END SUBROUTINE soil_para_hwsd
!
! ====================================================================
!

INTEGER FUNCTION center_pix_int (sf,ktop, ktot, x,y,x0,y0,z0,ext_point)

implicit none

integer (kind =2), dimension (:), intent (in) :: x,y
integer, intent (in) :: ktop,ktot
real, intent (in) :: sf
real :: xi,xj,yi,yj,xx0,yy0,zz0
real, allocatable, dimension (:,:) :: length_m
real, allocatable, dimension (:) :: length
real, intent (inout) :: x0,y0,z0
integer :: i,j,npix
logical, intent(in) :: ext_point
real :: zi, zj

allocate (length_m (1:ktot,1:ktot))
allocate (length   (1:ktot))
length_m =0.
length   =0.

center_pix_int = -9999
if(ktot /= 0) then
   do i = 1,ktot
      xi = sf*x(i)
      yi = sf*y(i)
      zi = 100. - xi - yi
      if (.not. ext_point) then
         x0 = xi
         y0 = yi
         z0 = zi
      endif
      
      do j = 1,ktot
         xj = sf*x(j)
         yj = sf*y(j)
         zj = 100. - xj - yj
         xx0= xj - x0
         yy0= yj - y0
         zz0= zj - z0
 
         if(ktot > ktop) then 
            if(j <= ktop) then
               length_m (i,j) = (xx0*xx0 +  yy0*yy0 + zz0*zz0)**0.5
            else
               length_m (i,j) = 2.33*((xx0*xx0 +  yy0*yy0 + zz0*zz0)**0.5)
            endif
         else
            length_m (i,j) = (xx0*xx0 +  yy0*yy0 + zz0*zz0)**0.5
         endif
      end do
      length (i) = sum(length_m (i,:))
   end do
   
center_pix_int = minloc(length,dim=1)
endif

END FUNCTION center_pix_int

!
! ====================================================================
!

INTEGER FUNCTION center_pix_int0 (sf,ktop, ktot, x,y)

implicit none
! sf = 0.01 (integer to real scale factor), ktop = # of pixels in top layer
! ktot = total # of pixels, top + subsurface combined
! x (clay), y (sand_
integer (kind =2), dimension (:), intent (in) :: x,y
integer, intent (in) :: ktop,ktot
real, intent (in) :: sf
real :: xi,xj,yi,yj
real :: length

integer :: i,j,npix
real :: zi, zj, mindist,xc,yc,zc

length   =0.

center_pix_int0 = -9999

if(ktot /= 0) then
! There should be some data pixels
   if(ktot > ktop) then 
      ! Have both layers
      if(ktop > 0) then
         ! There are data in top layer
         xc = sf*0.3*sum(real(x(1:ktop)))/real(ktop) + sf*0.7*sum(real(x(ktop + 1 : ktot)))/real(ktot - ktop)  
         yc = sf*0.3*sum(real(y(1:ktop)))/real(ktop) + sf*0.7*sum(real(y(ktop + 1 : ktot)))/real(ktot - ktop)
      else
         ! There are no data in top layer
         xc = sf*sum(real(x(1:ktot)))/real(ktot)  
         yc = sf*sum(real(y(1:ktot)))/real(ktot)         
      endif
   else
      ! working on Top layer alone
      xc = sf*sum(real(x(1:ktot)))/real(ktot)  
      yc = sf*sum(real(y(1:ktot)))/real(ktot)
   endif
   zc = 100. - xc - yc
endif

mindist=100000.*100000.

do i = 1,ktot
   xi = sf*x(i)
   yi = sf*y(i)
   zi = 100. - xi - yi
   length = (xi-xc)**2+(yi-yc)**2+(zi-zc)**2
   if(mindist>length)then
      mindist=length
      center_pix_int0=i
   end if
end do
!print *,ktop,ktot,center_pix_int0
END FUNCTION center_pix_int0


END MODULE  process_hres_data
