!  $Id: GEOS_DataSeaIceGridComp.F90,v 1.10.12.1.2.3.14.2.2.1.2.1.50.2.14.1 2013-03-27 13:53:22 pchakrab Exp $

#include "MAPL_Generic.h"

!=============================================================================
!BOP

! !MODULE: GEOS_DataSeaIce -- A `fake' seaice model

! !INTERFACE:

module GEOS_DataSeaIceGridCompMod

! !USES: 

  use ESMF
  use MAPL_Mod

  implicit none
  private


! !PUBLIC MEMBER FUNCTIONS:

  public SetServices

! !DESCRIPTION:
! 
!   {\tt GEOS\_DataSeaIce} is a gridded component that reads the 
!   ocean\_bcs file 
!   This module interpolates the SST and sea ice fraction data from 
!   either daily or monthly values to the correct time of the simulation.
!   Data are read only if the simulation time is not in the save interval.
!   Surface Albedo and Surface roughness calculations are also takencare of in
!   this module.
!

!EOP

   contains

!BOP

! !IROUTINE: SetServices -- Sets ESMF services for this component

! !INTERFACE:

  subroutine SetServices ( GC, RC )

! !ARGUMENTS:

    type(ESMF_GridComp), intent(INOUT) :: GC  ! gridded component
    integer, optional                  :: RC  ! return code

!  !DESCRIPTION: This version uses the MAPL\_GenericSetServices. This function sets
!                the Initialize and Finalize services, as well as allocating
!   our instance of a generic state and putting it in the 
!   gridded component (GC). Here we only need to set the run method and
!   add the state variable specifications (also generic) to our instance
!   of the generic state. This is the way our true state variables get into
!   the ESMF\_State INTERNAL, which is in the MAPL\_MetaComp.
!
!EOP

!=============================================================================
!
! ErrLog Variables


    character(len=ESMF_MAXSTR)              :: IAm
    integer                                 :: STATUS
    character(len=ESMF_MAXSTR)              :: COMP_NAME

! Local derived type aliases

!=============================================================================

! Begin...

! Get my name and set-up traceback handle
! ---------------------------------------

    Iam = "SetServices"
    call ESMF_GridCompGet( GC, NAME=COMP_NAME, RC=STATUS )
    VERIFY_(STATUS)
    Iam = trim(COMP_NAME) // Iam


! Set the Run entry point
! -----------------------

    call MAPL_GridCompSetEntryPoint ( GC, ESMF_METHOD_RUN,  Run, RC=STATUS)
    VERIFY_(STATUS)


! Set the state variable specs.
! -----------------------------

!BOS

! !Import state:

  call MAPL_AddImportSpec(GC,                            &
    SHORT_NAME         = 'HI',                                &
    LONG_NAME          = 'seaice_skin_layer_depth',            &
    UNITS              = 'm',                                 &
    DIMS               = MAPL_DimsHorzOnly,                   &
    VLOCATION          = MAPL_VLocationNone,                  &
                                                   RC=STATUS  )
  VERIFY_(STATUS)

  call MAPL_AddImportSpec(GC,                            &
    SHORT_NAME         = 'TI',                                &
    LONG_NAME          = 'seaice_skin_temperature',            &
    UNITS              = 'K',                                 &
    DIMS               = MAPL_DimsHorzOnly,                   &
    VLOCATION          = MAPL_VLocationNone,                  &
                                                   RC=STATUS  )
  VERIFY_(STATUS)

  call MAPL_AddImportSpec(GC,                            &
    SHORT_NAME         = 'SI',                                &
    LONG_NAME          = 'seaice_skin_salinity',               &
    UNITS              = 'psu',                               &
    DIMS               = MAPL_DimsHorzOnly,                   &
    VLOCATION          = MAPL_VLocationNone,                  &
                                                   RC=STATUS  )
  VERIFY_(STATUS)

  call MAPL_AddImportSpec(GC,                            &
    SHORT_NAME         = 'HW',                                &
    LONG_NAME          = 'water_skin_layer_mass',            &
    UNITS              = 'kg m-2',                                 &
    DIMS               = MAPL_DimsHorzOnly,                   &
    VLOCATION          = MAPL_VLocationNone,                  &
                                                   RC=STATUS  )
  VERIFY_(STATUS)

  call MAPL_AddImportSpec(GC,                            &
    SHORT_NAME         = 'TW',                                &
    LONG_NAME          = 'water_skin_temperature',            &
    UNITS              = 'K',                                 &
    DIMS               = MAPL_DimsHorzOnly,                   &
    VLOCATION          = MAPL_VLocationNone,                  &
                                                   RC=STATUS  )
  VERIFY_(STATUS)

  call MAPL_AddImportSpec(GC,                            &
    SHORT_NAME         = 'SW',                                &
    LONG_NAME          = 'water_skin_salinity',               &
    UNITS              = 'psu',                               &
    DIMS               = MAPL_DimsHorzOnly,                   &
    VLOCATION          = MAPL_VLocationNone,                  &
                                                   RC=STATUS  )
  VERIFY_(STATUS)


!  !Export state:

  call MAPL_AddExportSpec(GC,                            &
    SHORT_NAME         = 'UI',                                &
    LONG_NAME          = 'zonal_velocity_of_surface_seaice',   &
    UNITS              = 'm s-1 ',                            &
    DIMS               = MAPL_DimsHorzOnly,                   &
    VLOCATION          = MAPL_VLocationNone,                  &
                                                   RC=STATUS  )
  VERIFY_(STATUS)

  call MAPL_AddExportSpec(GC,                            &
    SHORT_NAME         = 'VI',                                &
    LONG_NAME          = 'meridional_velocity_of_surface_seaice',&
    UNITS              = 'm s-1 ',                            &
    DIMS               = MAPL_DimsHorzOnly,                   &
    VLOCATION          = MAPL_VLocationNone,                  &
                                                   RC=STATUS  )
  VERIFY_(STATUS)

  call MAPL_AddExportSpec(GC,                            &
    SHORT_NAME         = 'FRACICE',                           &
    LONG_NAME          = 'fractional_cover_of_seaice',        &
    UNITS              = '1',                                 &
    DIMS               = MAPL_DimsHorzOnly,                   &
    VLOCATION          = MAPL_VLocationNone,                  &
                                                   RC=STATUS  )
  VERIFY_(STATUS)

  call MAPL_AddExportSpec(GC,                            &
    SHORT_NAME         = 'MELTQ',                             &
    LONG_NAME          = 'heat_of_melting_or_freezing',       &
    UNITS              = 'W m-2',                             &
    DIMS               = MAPL_DimsHorzOnly,                   &
    VLOCATION          = MAPL_VLocationNone,                  &
                                                   RC=STATUS  )
  VERIFY_(STATUS)

!EOS

    call MAPL_TimerAdd(GC,    name="RUN"     ,RC=STATUS)
    VERIFY_(STATUS)
    call MAPL_TimerAdd(GC,    name="-UPDATE"     ,RC=STATUS)
    VERIFY_(STATUS)

! Set generic init and final methods
! ----------------------------------

    call MAPL_GenericSetServices    ( GC, RC=STATUS)
    VERIFY_(STATUS)

    RETURN_(ESMF_SUCCESS)
  
  end subroutine SetServices



!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!BOP

! !IROUTINE: RUN -- Run stage for the DataSeaIce component

! !INTERFACE:

subroutine RUN ( GC, IMPORT, EXPORT, CLOCK, RC )


! !ARGUMENTS:

  type(ESMF_GridComp), intent(inout) :: GC     ! Gridded component 
  type(ESMF_State),    intent(inout) :: IMPORT ! Import state
  type(ESMF_State),    intent(inout) :: EXPORT ! Export state
  type(ESMF_Clock),    intent(inout) :: CLOCK  ! The clock
  integer, optional,   intent(  out) :: RC     ! Error code:

! !DESCRIPTION: Periodically refreshes the SST and Ice information.

!EOP


! ErrLog Variables

  character(len=ESMF_MAXSTR)          :: IAm
  integer                             :: STATUS
  character(len=ESMF_MAXSTR)          :: COMP_NAME

! Locals

  type (MAPL_MetaComp),  pointer      :: MAPL
  logical                             :: FRIENDLY
  type(ESMF_FIELD)                    :: FIELD
  type (ESMF_Time)                    :: CurrentTime
  character(len=ESMF_MAXSTR)          :: DATAFRTFILE
  integer                             :: IFCST
  logical                             :: FCST
  real, pointer, dimension(:,:)       :: MELT, F1, TNEW
  real                                :: TAU_SIT
  real                                :: DT
  real                                :: RUN_DT
  real                                :: CTB  ! Ocean-ice turbulent mixing coefficient (m/sec)
  real, parameter   :: CW=MAPL_CAPWTR
  real :: TICE

! pointers to export

   real, pointer, dimension(:,:)  :: UI
   real, pointer, dimension(:,:)  :: VI
   real, pointer, dimension(:,:)  :: FR
   real, pointer, dimension(:,:)  :: MQ

! pointers to import

   real, pointer, dimension(:,:)  :: TI
   real, pointer, dimension(:,:)  :: HI
   real, pointer, dimension(:,:)  :: SI
   real, pointer, dimension(:,:)  :: TW
   real, pointer, dimension(:,:)  :: HW
   real, pointer, dimension(:,:)  :: SW

!  Begin...
!----------

! Get the target components name and set-up traceback handle.
! -----------------------------------------------------------

    Iam = "Run"
    call ESMF_GridCompGet( GC, name=COMP_NAME, RC=STATUS )
    VERIFY_(STATUS)
    Iam = trim(COMP_NAME) // Iam

! Get my internal MAPL_Generic state
!----------------------------------

    call MAPL_GetObjectFromGC(GC, MAPL, STATUS)
    VERIFY_(STATUS)

! Start Total timer
!------------------

   call MAPL_TimerOn(MAPL,"TOTAL")
   call MAPL_TimerOn(MAPL,"RUN" )


! Pointers to Imports
!--------------------

    call MAPL_GetPointer(IMPORT, TI      ,  'TI'     , RC=STATUS)
    VERIFY_(STATUS)
    call MAPL_GetPointer(IMPORT, HI      ,  'HI'     , RC=STATUS)
    VERIFY_(STATUS)
    call MAPL_GetPointer(IMPORT, SI      ,  'SI'     , RC=STATUS)
    VERIFY_(STATUS)
    call MAPL_GetPointer(IMPORT, TW      ,  'TW'     , RC=STATUS)
    VERIFY_(STATUS)
    call MAPL_GetPointer(IMPORT, HW      ,  'HW'     , RC=STATUS)
    VERIFY_(STATUS)
    call MAPL_GetPointer(IMPORT, SW      ,  'SW'     , RC=STATUS)
    VERIFY_(STATUS)

! Check that they are friendly
!-----------------------------

    call ESMF_StateGet (IMPORT, 'TI', FIELD, RC=STATUS)
    VERIFY_(STATUS)
    call ESMF_AttributeGet  (FIELD, NAME="FriendlyToSEAICE", VALUE=FRIENDLY, RC=STATUS)
    VERIFY_(STATUS)
    ASSERT_(FRIENDLY)

    call ESMF_StateGet (IMPORT, 'HI', FIELD, RC=STATUS)
    VERIFY_(STATUS)
    call ESMF_AttributeGet  (FIELD, NAME="FriendlyToSEAICE", VALUE=FRIENDLY, RC=STATUS)
    VERIFY_(STATUS)
    ASSERT_(FRIENDLY)

    call ESMF_StateGet (IMPORT, 'SI', FIELD, RC=STATUS)
    VERIFY_(STATUS)
    call ESMF_AttributeGet  (FIELD, NAME="FriendlyToSEAICE", VALUE=FRIENDLY, RC=STATUS)
    VERIFY_(STATUS)
    ASSERT_(FRIENDLY)

    call ESMF_StateGet(IMPORT, 'TW', FIELD, RC=STATUS)
    VERIFY_(STATUS)
    call ESMF_AttributeGet  (FIELD, NAME="FriendlyToSEAICE", VALUE=FRIENDLY, RC=STATUS)
    VERIFY_(STATUS)
    ASSERT_(FRIENDLY)

    call ESMF_StateGet (IMPORT, 'HW', FIELD, RC=STATUS)
    VERIFY_(STATUS)
    call ESMF_AttributeGet  (FIELD, NAME="FriendlyToSEAICE", VALUE=FRIENDLY, RC=STATUS)
    VERIFY_(STATUS)
    ASSERT_(FRIENDLY)

    call ESMF_StateGet (IMPORT, 'SW', FIELD, RC=STATUS)
    VERIFY_(STATUS)
    call ESMF_AttributeGet  (FIELD, NAME="FriendlyToSEAICE", VALUE=FRIENDLY, RC=STATUS)
    VERIFY_(STATUS)
    ASSERT_(FRIENDLY)

!  Pointers to Exports
!---------------------

    call MAPL_GetPointer(EXPORT,      UI  , 'UI'       , RC=STATUS)
    VERIFY_(STATUS)
    call MAPL_GetPointer(EXPORT,      VI  , 'VI'       , RC=STATUS)
    VERIFY_(STATUS)
    call MAPL_GetPointer(EXPORT,      FR  , 'FRACICE'  , RC=STATUS)
    VERIFY_(STATUS)
    call MAPL_GetPointer(EXPORT,      MQ  , 'MELTQ'    , RC=STATUS)
    VERIFY_(STATUS)

! Set current time and calendar
!------------------------------

    call ESMF_ClockGet(CLOCK, currTime=CurrentTime, rc=STATUS)
    VERIFY_(STATUS)

! Get the file name from the resource file
!-----------------------------------------

    call MAPL_GetResource(MAPL,DATAFRTFILE,LABEL="DATA_FRT_FILE:", RC=STATUS)
    VERIFY_(STATUS)

! In atmospheric forecast mode we don't have future SSTs
!-------------------------------------------------------

    call MAPL_GetResource(MAPL,IFCST,LABEL="IS_FCST:",default=0,   RC=STATUS)
    VERIFY_(STATUS)

    FCST = IFCST==1

! Get relaxation time
!--------------------

    call MAPL_GetResource(MAPL,TAU_SIT, LABEL="TAU_SIT:", default=86400.0,RC=STATUS)
    VERIFY_(STATUS)
    call MAPL_GetResource(MAPL,RUN_DT , LABEL="RUN_DT:" ,                 RC=STATUS)
    VERIFY_(STATUS)
    call MAPL_GetResource(MAPL,DT     , LABEL="DT:"     , default=RUN_DT, RC=STATUS)
    VERIFY_(STATUS)
    call MAPL_GetResource(MAPL,CTB     , LABEL="CTB:"   , default=1.0e-4, RC=STATUS)
    VERIFY_(STATUS)

!  Update the friendly skin values
!---------------------------------

   call MAPL_TimerOn(MAPL,"-UPDATE" )

   if(associated(FR)) then
      call MAPL_ReadForcing(MAPL,'FRT',DATAFRTFILE, CURRENTTIME, FR, INIT_ONLY=FCST, RC=STATUS)
      VERIFY_(STATUS)
   end if

   call MAPL_TimerOff(MAPL,"-UPDATE" )

   where (TI /= MAPL_Undef)
      TI   =(TI + (DT/TAU_SIT)*MAPL_TICE)/(1.+ (DT/TAU_SIT))
   end where

   HI   = HI
   SI   = 30.0

   allocate(MELT(size(TW,1),size(TW,2)), stat=STATUS)
   VERIFY_(STATUS)
   allocate(F1(size(TW,1),size(TW,2)), stat=STATUS)
   VERIFY_(STATUS)
   allocate(TNEW(size(TW,1),size(TW,2)), stat=STATUS)
   VERIFY_(STATUS)


   TICE=MAPL_TICE-1.8
   TNEW=0.0
   F1=0.0

   ! TW below freezing point is set to freezing temperature
   TNEW   = max(TW,TICE)
   
   where(FR == 1.0)
      ! if fraction of ice is 1, set TW to freezing temperature
      TNEW   =  TICE
   elsewhere
      F1=FR*CTB*MAPL_RHOWTR/(HW*(1-FR))
      TNEW=(TNEW+TICE*F1*DT)/(1+F1*DT)
   end where

   MELT=(TW-TNEW)*HW*CW/DT

   where(TW == MAPL_UNDEF)
      MELT=MAPL_UNDEF
      TNEW=MAPL_UNDEF
   end where
   
! Updated Sea-Ice Melting (non-zero diff to Fortuna-2_5_p6)
! ---------------------------------------------------------
   TW=TNEW
   
   if(associated(MQ)) MQ = MELT

!   TW   = max(TW,MAPL_TICE)
!   where(FR>0.0) TW = MAPL_TICE

!  Update the exports
!--------------------

   if(associated(UI)) UI = 0.0
   if(associated(VI)) VI = 0.0

! Clean-up
!---------

   deallocate(MELT,F1,TNEW)

!  All done
!-----------

   call MAPL_TimerOff(MAPL,"RUN"  )
   call MAPL_TimerOff(MAPL,"TOTAL")

   RETURN_(ESMF_SUCCESS)
end subroutine RUN

end module GEOS_DataSeaIceGridCompMod
